#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007-2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Ad;

use strict;
use warnings;

use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ
	$this->{ADLINK1} = '';		# 広告リンク１
	$this->{ADLINK2} = '';		# 広告リンク２
	$this->{ADLIST} = '';		# お知らせ

	bless($this);

	$this->load();

	return $this;
}

# デフォルト広告リンク、お知らせの読み込み
sub load {
	my $this = shift;

	if ($this->{SYSTEMDIR} eq '') {
		return 0;
	}

	# 広告リンクファイルを読み込む
	my $adlinkfile = "$this->{SYSTEMDIR}/adlink.txt";
	open IN, $adlinkfile;
	my $data = <IN>;
	chop $data;
	my ($adlink1,$adlink2) = (split(/\,/, $data))[0..1];
	close IN;
	$this->{ADLINK1} = $adlink1;	# 広告リンク１
	$this->{ADLINK2} = $adlink2;	# 広告リンク２

	# お知らせファイルを読み込む
	my $adlistfile = "$this->{SYSTEMDIR}/adlist.txt";
	my $adlist = "";
	open IN, $adlistfile;
	while (<IN>) {
		$data = $_;	# 1行読み込み
		chop($data);	# 改行を取り除く
		if ($data ne "") {
			$adlist .= "$data<br>";
		}
	}
	close IN;
	$this->{ADLIST} = $adlist;	# お知らせ

	return 1;
}

# 広告リンク１の取得
sub adlink1 {
	my $this = shift;
	return $this->{ADLINK1};
}

# 広告リンク２の取得
sub adlink2 {
	my $this = shift;
	return $this->{ADLINK2};
}

# お知らせの取得
sub adlist {
	my $this = shift;
	my $listtable = '';

	if ($this->{ADLIST} ne '') {
		$listtable = <<"End_of_Temp";
<strong>お知らせ</strong><br>
$this->{ADLIST}
End_of_Temp
	}

	return $listtable;
}

1;
# End of Ad.pm
