#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Calendar;

use strict;
use warnings;

use APPLIB::Conf;
use Time::Local;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	$this->{DATADIR} = $confobj->DataDir();	# データディレクトリ

	$this->{YEAR} = '';
	$this->{MONTH} = '';

	bless($this);
	return $this;
}

# 年
sub year {
	my $this = shift;
	if (@_) {
		$this->{YEAR} = shift;
	}
	return $this->{YEAR};
}

# 月
sub month {
	my $this = shift;
	if (@_) {
		$this->{MONTH} = shift;
	}
	return $this->{MONTH};
}


# カレンダーの取得(HTML)
sub calendartable {
	my $this = shift;
	my $year = $this->{YEAR};
	my $month = $this->{MONTH};
	my $day;
	my @daylist;
	my $data;
	my $topicid;
	my $lastdate;
	my $olddate = '';
	my $curdate;

	# アーカイブデータをもとに投稿日の一覧を取得
	my $archivefile = "$this->{DATADIR}/$year$month" . "ar.data";
	if (-f $archivefile) {	# ファイルが存在する場合
		open IN, $archivefile;
		while (<IN>) {
			$data = $_;	# 1行読み込み
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				($topicid,$lastdate) = (split(/\,/, $data))[0,1];
				$curdate = substr($lastdate, 0, 10);
				if ($curdate ne $olddate) {
					$olddate = $curdate;
					$day = substr($curdate, 8, 2);
					push @daylist, $day;
				}
			}
		}
		close IN;
	}

	my @MONTHS = (31,28,31,30,31,30,31,31,30,31,30,31);
	# 閏年の調整
	if ($year % 4 == 0) {
		$MONTHS[1] = 29;
		if (($year % 400 != 0) && ($year % 100 == 0)) {
			$MONTHS[1]  = 28;
		}
	}

	my $prevyear = $year;
	my $prevmon = $month - 1;
	if ($prevmon < 1) {
		$prevmon = 12;
		$prevyear--;
	}
	$prevmon = sprintf("%2.2d", $prevmon);

	my $nextyear = $year;
	my $nextmon = $month + 1;
	if ($nextmon > 12) {
		$nextmon = 1;
		$nextyear++;
	}
	$nextmon = sprintf("%2.2d", $nextmon);

	# 前月のアーカイブデータファイルの存在チェック
	$archivefile = "$this->{DATADIR}/$prevyear$prevmon" . "ar.data";
	my $prevnavi = '';
	if (-f $archivefile) {
		$prevnavi = <<"End_of_Str";
<a href="view.cgi?year=$prevyear&month=$prevmon">&lt;&lt;</a>
End_of_Str
	}
	# 次月のアーカイブデータファイルの存在チェック
	$archivefile = "$this->{DATADIR}/$nextyear$nextmon" . "ar.data";
	my $nextnavi = '';
	if (-f $archivefile) {
		$nextnavi = <<"End_of_Str";
<a href="view.cgi?year=$nextyear&month=$nextmon">&gt;&gt;</a>
End_of_Str
	}

	my $caltable = <<"End_of_Str";
<table border="0" width="100%" cellpadding="3" cellspacing="0">
<tr>
<td align="center">$prevnavi</td>
<td colspan="5" align="center"><b>$year年$month月</b></td>
<td align="center">$nextnavi</td>
</tr>
<tr>
<td align="center"><font color="red">日</font></td>
<td align="center">月</td>
<td align="center">火</td>
<td align="center">水</td>
<td align="center">木</td>
<td align="center">金</td>
<td align="center"><font color="blue">土</font></td>
</tr>
End_of_Str

	my $startwday;
	($startwday) = (localtime(timelocal(0, 0, 0, 1, $month - 1, $year - 1900)))[6];	# その月の1日の曜日
	my $weekofday = 0;
	my $firstweek = 1;
	my $temp = "<tr>";
	my $i;
	my $notfound;

	for ($i = 1; $i <= $MONTHS[$month - 1]; $i++) {

		# 土曜日が終わったら折り返し
		if ($weekofday == 7) {
			$temp = $temp . "</tr><tr>";
			$weekofday = 0;
		}

		# 第一週の調整
		if ($startwday > 0 && $firstweek == 1) {
			for (; $weekofday < $startwday; $weekofday++) {
				$temp .= "<td> </td>";
			}
			$firstweek = 0;
		}

		$notfound = 1;
		foreach $day (@daylist) {
			if ($day == $i) {
				$notfound = 0;
			}
		}
		if (!$notfound) {
			my $dd = $i;
			if ($dd < 10) {
				$dd = "0" . $dd;
			}
			$temp .= <<"End_of_Str";
<td align="center">
<a href="view.cgi?year=$year&month=$month&day=$dd">$i</a>
</td>
End_of_Str
		}
		else {
			$temp .= "<td align=\"center\">$i</td>";
		}

		# 最後の週の調整
		if ($i == $MONTHS[$month - 1]) {
			for (; $weekofday < 7; $weekofday++) {
				$temp .= "<td> </td>";
			}
		}

		$weekofday++;
	}

	$caltable .= $temp;
	$caltable .= "</tr></table>";

	return $caltable;
}

1;
# End of Calendar.pm
