#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Comment;

use strict;
use warnings;

use APPLIB::DateTime;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{DATADIR} = $confobj->DataDir();		# データディレクトリ
	$this->{GOODPOINT} = $confobj->GoodPoint();
	$this->{NORMALPOINT} = $confobj->NormalPoint();
	$this->{BADPOINT} = $confobj->BadPoint();

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{NEWNUM} = $confobj->NewTopicNum();

	bless($this);
	return $this;
}

# 追加
sub add {
	my $this = shift;
	my $topicid = shift;
	my $commentid = shift;
	my $commentsummary = shift;
	my @newdata = ();
	my $data;

	# 既存の新着コメントファイルを読み込み配列に保存する
	my $newcommentfile = "$this->{DATADIR}/newcomment.data";
	my $commentcount = 0;
	if (-f $newcommentfile) {	# ファイルが存在する場合
		open IN, $newcommentfile;
		while ($data = <IN>) {
			chop $data;
			if ($data ne '') {
				push @newdata, $data;
				$commentcount++;
			}
		}
		close IN;
	}

	open(NEWCOMMENTFILE,">$newcommentfile");
	flock(NEWCOMMENTFILE, $this->{LOCK_EX});
	seek(NEWCOMMENTFILE, 0, 2);

	# 新規作成のデータをファイルの先頭に追加
	print NEWCOMMENTFILE "$topicid,$commentid,$commentsummary\n";
	if ($commentcount > $this->{NEWNUM}) {
		$commentcount = $this->{NEWNUM} - 1; 
	}
	my $i;
	for ($i = 0; $i < $commentcount; $i++) {
		$data = $newdata[$i];
		print NEWCOMMENTFILE "$data\n";
	}
	flock(NEWCOMMENTFILE, $this->{LOCK_UN});
	close(NEWCOMMENTFILE);
	# ファイルのパーミッションを変更
	chmod 0766, $newcommentfile;
}


# 削除
sub delete {
	my $this = shift;
	my $topicid = shift;
	my $commentid = shift;
	my $newtopicid;
	my $newcommentid;
	my @newdata = ();
	my $data;

	# 新着コメントファイルを読み込む
	my $newcommentfile = "$this->{DATADIR}/newcomment.data";
	open IN, $newcommentfile;
	while ($data = <IN>) {
		chop $data;
		($newtopicid,$newcommentid) = (split(/\,/, $data))[0,1];
		if (!($newtopicid eq $topicid && $commentid eq $newcommentid)) {
			push @newdata, $data;
		}
	}
	close IN;

	# 新着コメントファイルを更新
	open(NEWCOMMENTFILE,">$newcommentfile");
	flock(NEWCOMMENTFILE, $this->{LOCK_EX});
	seek(NEWCOMMENTFILE, 0, 2);
	foreach $data (@newdata) {
		print NEWCOMMENTFILE "$data\n";
	}
	flock(NEWCOMMENTFILE, $this->{LOCK_UN});
	close(NEWCOMMENTFILE);
}

# アーカイブ表の取得(HTML)
sub commenttable {
	my $this = shift;
	my $topicid;
	my $commentid;
	my $summary;
	my $data;

	my $table = <<"End_of_Temp";
<table border="0" width="180" cellpadding="3" cellspacing="0">
<tr>
<td><h5>新着コメント</h5></td>
</tr>
End_of_Temp

	# 最近のコメントファイルを読み込みコメントを作成する
	my $newcomment = "$this->{DATADIR}/newcomment.data";
	if (-f $newcomment) {	# ファイルが存在する場合
		open IN, $newcomment;
		while ($data = <IN>) {
			chop($data);	# 改行を取り除く
			if ($data ne '') {
				($topicid,$commentid,$summary) = (split(/\,/, $data))[0,1,2];

				# コンマをデコード
				$summary =~ s/enc_conma/,/g;

				# 詳細のHTMLファイル名
				my $topichtml = "detail$topicid.html";
				my $temp = <<"End_of_Temp";
<tr>
<td>
<a href="$topichtml#comment$commentid">$summary</a>
</td>
</tr>
End_of_Temp
				# 後ろにつけていく
				$table .=  $temp;
			}
		}
		close IN;
	}

	$table .= "</table>";

	return $table;
}

# トピックコメントの読み込み
sub readcomment {
	my $this = shift;
	my $topicid = shift;
	my $commentid = shift;
	my @rec = ();

	my $commentfile = "$this->{DATADIR}/$topicid" . "cm.data";
	if (-f $commentfile) {
		open IN, $commentfile;
		while (my $data = <IN>) {
			chop $data;
			@rec = split(/\,/, $data);
			if ($rec[0] eq $commentid) {
				last;
			}
		}
		close IN;
	}

	return @rec;
}

# トピックコメントの追加
sub addcomment {
	my $this = shift;
	my $topicid = shift;
	my ($author,$mailaddr,$hpurl,$comment,$ipaddr,$evaluate) = @_;

	my $dtobj = APPLIB::DateTime->new();
	my $systemdate = $dtobj->localdatetime(0);
	my $lastcount = 0;

	my $commentfile = "$this->{DATADIR}/$topicid" . "cm.data";
	if (-f $commentfile) {
		# コメントID最大値を取得する
		open IN, $commentfile;
		while (my $data = <IN>) {
			chop $data;
			my $id = (split(/\,/, $data))[0];
			if ($id > $lastcount) {
				$lastcount = $id;
			}
		}
		close IN;
	}
	$lastcount++;
	my $commentid = $lastcount;

	# コメント一覧ファイルに追記
	open(OUT,">>$commentfile");
	flock(OUT, $this->{LOCK_EX});
	seek(OUT, 0, 2);
	print OUT "$commentid,$systemdate,$author,$mailaddr,$hpurl,$comment,$ipaddr,$evaluate\n";
	flock(OUT, $this->{LOCK_UN});
	close(OUT);
	# ファイルのパーミッションを変更
	chmod 0766, $commentfile;

	return 1;
}

# トピックコメントの削除
sub delcomment {
	my $this = shift;
	my $topicid = shift;
	my $id = shift;
	my $lostpoint = 0;

	# コメント一覧ファイルを読み込む
	my $commentfile = "$this->{DATADIR}/$topicid" . "cm.data";
	if (-f $commentfile) {
		open IN, $commentfile;
		my $data;
		my @commentdata = ();
		while ($data = <IN>) {
			chop $data;
			my ($commentid,$postdate,$author,$mailaddr,$hpurl,$text,$ipaddr,$evaluate) = (split(/\,/, $data))[0..7];
			if ($commentid == $id) {	# 削除対象のコメントのIDを0にする
				$data = "0,$postdate,$author,$mailaddr,$hpurl,$text,$ipaddr,$evaluate";
				if ($evaluate > 0) {
					$lostpoint = $this->{GOODPOINT};
				}
				elsif ($evaluate < 0) {
					$lostpoint = $this->{BADPOINT};
				}
				else {
					$lostpoint = $this->{NORMALPOINT};
				}
			}
			push @commentdata, $data;
		}
		close IN;

		# コメントファイルを更新
		open(OUT,">$commentfile");
		flock(OUT, $this->{LOCK_EX});
		seek(OUT, 0, 2);
		foreach $data (@commentdata) {
			print OUT "$data\n";
		}
		flock(OUT, $this->{LOCK_UN});
		close(OUT);

		# ファイルのパーミッションを変更
		chmod 0766, $commentfile;
	}

	return $lostpoint;
}

1;
# End of Comment.pm
