#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::DateTime;

use strict;
use warnings;

use Time::Local;

# コンストラクタ
sub new {
	my $this = {};

	$this->{STRING} = '';
	$this->{FORMAT} = 0; # 0:YYYY/MM/DD HH:NN:SS 1:YYYY-MM-DDTHH:NN:SS+09:00 2:YYYY年MM月DD日 HH:NN:SS 3:RFC 822

	$this->{YEAR} = '';
	$this->{MONTH} = '';
	$this->{DAY} = '';
	$this->{HOUR} = '';
	$this->{MINUTE} = '';
	$this->{SECOND} = '';

	bless($this);
	return $this;
}

# 形式
sub format {
	my $this = shift;
	if (@_) {
		$this->{FORMAT} = shift;
	}
	return $this->{FORMAT};
}

# 文字列
sub string {
	my $this = shift;
	if (@_) {
		$this->{STRING} = shift;
	}
	return $this->{STRING};
}

# 年
sub year {
	my $this = shift;
	if (@_) {
		$this->{YEAR} = shift;
	}
	return $this->{YEAR};
}

# 月
sub month {
	my $this = shift;
	if (@_) {
		$this->{MONTH} = shift;
	}
	return $this->{MONTH};
}

# 日
sub day {
	my $this = shift;
	if (@_) {
		$this->{DAY} = shift;
	}
	return $this->{DAY};
}

# 時
sub hour {
	my $this = shift;
	if (@_) {
		$this->{HOUR} = shift;
	}
	return $this->{HOUR};
}

# 分
sub minute {
	my $this = shift;
	if (@_) {
		$this->{MINUTE} = shift;
	}
	return $this->{MINUTE};
}

# 秒
sub second {
	my $this = shift;
	if (@_) {
		$this->{SECOND} = shift;
	}
	return $this->{SECOND};
}

# 日時の設定(文字列)
sub setstrdate {
	my $this = shift;
	my $datestr = shift;

	# 日付を設定
	my $date = substr($datestr, 0, 10);
	($this->{YEAR},$this->{MONTH},$this->{DAY}) = (split(/\//, $date))[0..2];

	# 時刻を設定
	my $time = substr($datestr, 11, 8);
	($this->{HOUR},$this->{MINUTE},$this->{SECOND}) = (split(/:/, $time))[0..2];

	if ($this->{FORMAT} == 0) {
		$this->{STRING} = "$this->{YEAR}/$this->{MONTH}/$this->{DAY} $this->{HOUR}:$this->{MINUTE}:$this->{SECOND}";
	}
	elsif ($this->{FORMAT} == 1) {
		$this->{STRING} = "$this->{YEAR}-$this->{MONTH}-$this->{DAY}" . "T$this->{HOUR}:$this->{MINUTE}:$this->{SECOND}+09:00";
	}
	elsif ($this->{FORMAT} == 2) {
		$this->{STRING} = "$this->{YEAR}年$this->{MONTH}月$this->{DAY}日 $this->{HOUR}:$this->{MINUTE}:$this->{SECOND}";
	}
	elsif ($this->{FORMAT} == 3) {
		my $wday;
		my @weeks = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
		my @months = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
		($wday) = (localtime(timelocal(0, 0, 0, $this->{DAY}, $this->{MONTH} - 1, $this->{YEAR} - 1900)))[6];	# 曜日を求める
		$wday = $weeks[$wday];
		my $month = $months[$this->{MONTH} - 1];
		$this->{STRING} = "$wday, $this->{DAY} $month $this->{YEAR} $this->{HOUR}:$this->{MINUTE}:$this->{SECOND} +0900";
	}
	elsif ($this->{FORMAT} == 4) {
		$this->{STRING} = "$this->{YEAR}"."$this->{MONTH}"."$this->{DAY}"."T$this->{HOUR}:$this->{MINUTE}:$this->{SECOND}";
	}

	return $this->{STRING};
}


# ローカル日時の取得
sub localdatetime {
	my $this = shift;
	my $format;
	my $localdt;
	my $sec;
	my $min;
	my $hour;
	my $mday;
	my $mon;
	my $year;

	if (@_) {
		# 引数で形式が指定された場合
		$format = shift;
	}
	else {
		# 引数指定なしの場合
		$format = $this->{FORMAT};
	}

	if (!$this->{STRING}) {

		($sec, $min, $hour, $mday, $mon, $year) = (localtime(time))[0..5];
		$mon++;
		$year += 1900;

		$sec = '0' . $sec if ($sec < 10);
		$min = '0' . $min if ($min < 10);
		$hour = '0' . $hour if ($hour < 10);
		$mday = '0' . $mday if ($mday < 10);
		$mon = '0' . $mon if ($mon < 10);

		$this->{YEAR} = $year;
		$this->{MONTH} = $mon;
		$this->{DAY} = $mday;
		$this->{HOUR} = $hour;
		$this->{MINUTE} = $min;
		$this->{SECOND} = $sec;

	}

	if ($format == 0) {
		$localdt = "$this->{YEAR}/$this->{MONTH}/$this->{DAY} $this->{HOUR}:$this->{MINUTE}:$this->{SECOND}";
	}
	elsif ($format == 1) {
		$localdt = "$this->{YEAR}-$this->{MONTH}-$this->{DAY}" . "T$this->{HOUR}:$this->{MINUTE}:$this->{SECOND}+09:00";
	}
	elsif ($format == 2) {
		$localdt = "$this->{YEAR}年$this->{MONTH}月$this->{DAY}日 $this->{HOUR}:$this->{MINUTE}:$this->{SECOND}";
	}
	elsif ($format == 3) {
		my $wday;
		my @weeks = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
		my @months = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');

		($wday) = (localtime(timelocal(0, 0, 0, $this->{DAY}, $this->{MONTH} - 1, $this->{YEAR} - 1900)))[6];	# 曜日を求める
		$wday = $weeks[$wday];
		my $month = $months[$this->{MONTH} - 1];
		$localdt = "$wday, $this->{DAY} $month $this->{YEAR} $this->{HOUR}:$this->{MINUTE}:$this->{SECOND} +0900";
	}
	elsif ($this->{FORMAT} == 4) {
		$this->{STRING} = "$this->{YEAR}"."$this->{MONTH}"."$this->{DAY}"."T$this->{HOUR}:$this->{MINUTE}:$this->{SECOND}";
	}

	$this->{STRING} = $localdt;
	$this->{FORMAT} = $format;

	return $localdt;
}

1;
# End of DateTime.pm
