#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Ping;

use strict;
use warnings;

use XMLRPC::Lite;
use HTTP::Request::Common;
use LWP::UserAgent;

use APPLIB::String;
use APPLIB::Logger;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ

	bless($this);
	return $this;
}

# トラックバックピングの送信
sub trackbackping {
	my $this = shift;
	my $url = shift;
	my $blogname = shift;
	my $title = shift;
	my $text = shift;
	my $length = shift;
	my @pinglist = @_;
	my $tburl;

	my $strobj = APPLIB::String->new();

	$text =~ s/enc_conma/,/g;
	my $excerpt = $strobj->cut($text, $length);	# 要約の長さを調整

	my $loggerobj = APPLIB::Logger->new();

	my $ua = LWP::UserAgent->new();
	$ua->agent("pingtb/1.0");

	foreach $tburl (@pinglist) {
		$loggerobj->write($tburl);

		# PING送信
		my $req = POST $tburl, [title => $title, url => $url, blog_name => $blogname, excerpt => $excerpt, charset => 'UTF-8', ];
		my $response = $ua->request($req);
		unless ($response->is_success) {
			$loggerobj->write("トラックバックPingの送信に失敗しました");
			return 0;
		}
	}

	return 1;
}

# 更新Pingの送信
sub updateping {
	my $this = shift;
	my $blogurl = shift;
	my $blogname = shift;
	my $pingserver;
	my @serverlist = ();

	# ping送信先の取得
	my $pinglist = "$this->{SYSTEMDIR}/pinglist.txt";
	if (-f $pinglist) {	# ファイルが存在する場合
		open IN, $pinglist;
		while (<IN>) {
			$pingserver = $_;	# 1行読み込み
			chop($pingserver);	# 改行を取り除く
			if ($pingserver ne "") {
				push @serverlist, $pingserver;
			}
		}
		close IN;
	}
	else {
		return 0;	
	}

	my $result;
	my $logline;
	my $loggerobj = APPLIB::Logger->new();

	foreach $pingserver (@serverlist) {
		$result = eval { 
			XMLRPC::Lite
			->proxy($pingserver)
			->call('weblogUpdates.ping', $blogname, $blogurl,)
			->result; };

		for my $key (keys %$result) {
    		$logline = "$pingserver $key:$result->{$key}";
			$loggerobj->write($logline);
		}
	}

	return 1;
}

1;
# End of Ping.pm
