#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Search;

use strict;
use warnings;

use APPLIB::Topic;
use APPLIB::Archive;
use APPLIB::Tag;
use APPLIB::String;
use APPLIB::Logger;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ
	$this->{DATADIR} = $confobj->DataDir();		# データディレクトリ

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{RESULT} = '';	# 検索結果
	$this->{USERID} = '';		# ユーザーID

	bless($this);
	return $this;
}

# 前回の検索結果
sub result {
	my $this = shift;
	return $this->{RESULT};
}

# ユーザーID
sub userid {
	my $this = shift;
	if (@_) {
		$this->{USERID} = shift;
	}
	return $this->{USERID};
}

# タグ検索
sub tagsearch {
	my $this = shift;
	my $querystr = shift;

#	my $loggerobj = APPLIB::Logger->new();
	my $tagobj = APPLIB::Tag->new();

	# 半角スペースを区切りとしてキーワードに分解
	my @keywords = split(/ /, $querystr);

	my @resultset = ();
	foreach my $item (@keywords) {

#		$loggerobj->write("tagsearch - item:$item");	# for debug

		$tagobj->find($item);
		my $list = $tagobj->topiclist();

		my @idset = split(/:/, $list);
		if (@resultset == 0) {
			@resultset = @idset;
			next
		}
		else {	# and 検索
			my @newset = ();
			foreach my $id (@idset) {
				foreach my $tempid (@resultset) {
					if ($id eq $tempid) {
						push @newset, $id;
						last;
					}
				}
			}
			@resultset = @newset;
		}
	}

	my $result = '';
	foreach my $id (@resultset) {
		if ($result ne '') {
			$result .= ":";
		}
		$result .= $id;
	} 

	$this->{RESULT} = $result;

	return $result;
}

# トピック検索
sub search {
	my $this = shift;
	my $querystr = shift;
	my $result;
	my @resultset = ();

#	my $loggerobj = APPLIB::Logger->new();
#	$loggerobj->write("querystr:$querystr");	# for debug

	if ($querystr eq '') {
		# 全てのトピックの一覧を生成する
		my $archiveobj = APPLIB::Archive->new();
		my $topicnum = $archiveobj->lastseq();

		my $topicobj = APPLIB::Topic->new();
		my $i;
		for ($i = 0; $i <= $topicnum; $i++) {
			my $topicid = $i;
#			$loggerobj->write("topicid:$topicid");	# for debug
			my $ret = $topicobj->load($topicid);
			if ($ret) {
				my $status = $topicobj->status();
				if ($status == 1 || $status == 2) {	# 公開されているトピックのみ検索対象
					push @resultset, $topicid;
				}
			}
			else {	# トピックデータ無し
				next;
			}
		}
	}
	else {
		if ($querystr =~ /^[0-9\,]+$/) {	# 数字とコンマだけの場合はコンマをコロンに置換して返す
			$result = $querystr;
			$result =~ s/,/:/g;
			$this->{RESULT} = $result;
			return $result;
		}

		@resultset = $this->indexsearch($querystr);
	}

	$result = '';
	foreach my $id (@resultset) {
		if ($result ne '') {
			$result .= ":";
		}
		$result .= $id;
	} 

#	$loggerobj->write("result:$result");	# for debug
	$this->{RESULT} = $result;

	return $result;
}

# インデックスの再構築
sub rebuild{
	my $this = shift;

	# 既存のインデックスファイルを全て削除
	my $dir = "$this->{DATADIR}/search/*.data";
	my @files = <${dir}>;
	foreach my $fname (@files) {
		unlink $fname;
	}

	my $result = $this->makeindex(0);
	return $result;
}

# 指定された範囲のインデックスを作成
sub makeindex{
	my $this = shift;
	my $startpos = shift;
	my $endpos = 0;
	my $result = 0;
	my $archiveobj = APPLIB::Archive->new();
	my $topicnum = $archiveobj->lastseq();
	if (@_) {
		$endpos = shift;
		if ($topicnum < $endpos) {
			$endpos = $topicnum;
		}
	}
	else {
		$endpos = $topicnum;
	}
#	print "makeindex - startpos:$startpos\n";
#	print "makeindex - endpos:$endpos\n";

	# 検索インデックスを再作成
	my $topicobj = APPLIB::Topic->new();
	for (my $i = $startpos; $i <= $endpos; $i++) {
#		print "makeindex - i:$i\n";

		my $topicid = $i;
		my $ret = $topicobj->load($topicid);
		if ($ret) {
			my $title = $topicobj->title();
			my $text = $topicobj->text();
			my $editor = $topicobj->editor();
			my $str = "$title $text $editor";
			$this->addindex($topicid, $str);
			$result++;
		}
		else {	# トピックデータ無し
			next;
		}
	}

	return $result;
}

# インデックスの追加
sub addindex {
	my $this = shift;
	my $id = shift;		# トピックID
	my $srcstr = shift;

#	my $loggerobj = APPLIB::Logger->new();

	$srcstr =~ s/<[^>]*>//g;	# HTMLタグを取り除く
	$srcstr = $this->adjustquery($srcstr);

	my @srcstrings = split(/ /, $srcstr);
	my @destkeywords = ();
	foreach my $str (@srcstrings) {
		my $ch = substr($str, 0, 1);
		if ($ch =~ /[\x20-\x7E]/) {	# ASCII文字
			my $hit = 0;
			foreach my $keyword (@destkeywords) {
				if ($keyword eq $str) {
					$hit = 1;
					last;
				}
			}
			if (!$hit) {
				push @destkeywords, $str;
			}
		}
		else {
			@destkeywords = $this->decompose($str, @destkeywords);
		}
	}
	foreach my $keyword (@destkeywords) {
#		print "$keyword\n";
		$this->registeritem($id, $keyword);
	}

	return 1;
}

# 入力された2バイト文字からなる文を分解してキーワードの配列として返す
sub decompose{
	my $this = shift;
	my $srcstr = shift;		# 変換前の文字列
	my @keywords = @_;		# 分解後のキーワード配列
	my $len = length($srcstr) / 2;	# 変換対象の文字列の文字数
	my $cutstr;
	my $cutsize;
	my $width = 1;
	my $found;

	while ($width <= $len) {
		for (my $i = 0; $i < $len; $i++) {
			my $pos = $i * 2;
			$cutsize = $width * 2;
			$cutstr = substr($srcstr, $pos, $cutsize);	# 文字の切り出し

			# 残りの文字サイズが切り出しサイズより小さくなったらループを抜ける
			if (length($cutstr) < $cutsize) {
				last;
			}

			# 既に登録されているかチェック
			$found = 0;
			foreach my $keyword (@keywords) {
				if ($cutstr eq $keyword) {
					$found = 1;
					last;
				}
			}
			# 登録されていないときは
			if (!$found) {
				push @keywords, $cutstr;
			}
		}
		$width++;
	}

	return @keywords;
}

# キーワードをインデックスに登録
sub registeritem {
	my $this = shift;
	my $id = shift;			# トピックIDまたはブックマークID
	my $item = shift;		# キーワード

#	my $loggerobj = APPLIB::Logger->new();

	my $indexfile = $this->getfilename($item);
#	$loggerobj->write("registeritem - indexfile:$indexfile");	# for debug
	if ($indexfile eq '') {
		return 0;
	}

	my $data;
	my $keyword;
	my $topiclist;
	my $topicnum;
	my $list;
	my $num;
	my $notfound = 1;
	my @idlist;
	my @newdata = ();

	# 既存のインデックスファイルを読み込む
	if (-f $indexfile) {
		open IN, $indexfile;
		while (<IN>) {
			$data = $_;	# 1行読み込み
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				($keyword,$topiclist,$topicnum) = (split(/\,/, $data))[0..2];
				if ($item eq $keyword) {
					$list = $topiclist;
					$num = $topicnum;

					# 既に登録されているかチェック
					@idlist = split(/:/, $list);
					my $i;
					for ($i = 0; $i < $num; $i++) {
						if ($idlist[$i] == $id) {
							last;
						}
					}
					if ($i >= $num) {
						if ($num > 0) {
							$list .= ":";
						}
						$list .= "$id";
						$num++;

						$topiclist = $list;
						$topicnum = $num;
					}

					$data = "$keyword,$topiclist,$topicnum";
					$notfound = 0;
				}
				push @newdata, $data;
			}
		}
		close IN;
	}

	if ($notfound) {	# インデックス未登録
		$list = $id;
		$num = 1;
		$topiclist = $list;
		$topicnum = $num;
		$data = "$item,$topiclist,$topicnum";
		push @newdata, $data;
	}

	# 検索インデックスファイルを更新
	open(OUT, ">$indexfile");
	flock(OUT, $this->{LOCK_EX});
	seek(OUT, 0, 2);
	foreach $data (@newdata) {
		print OUT "$data\n";
	}
	flock(OUT, $this->{LOCK_UN});
	close(OUT);
	# ファイルのパーミッションを変更
	chmod 0766, $indexfile;

	return 1;
}

# 検索条件の調整（検索クエリーを文字種別毎半角スペースで区切る）
sub adjustquery {
	my $this = shift;
	my $srcstr = shift;
	my $deststr = '';

	# 文字エンコードをシフトJIS変換
	my $strobj = APPLIB::String->new();
	$srcstr = $strobj->encodeex('utf8', 'sjis', $srcstr);

	# 全角英数字を半角に変換
	$srcstr = $strobj->zen2han($srcstr);

	# 半角カナを全角に変換
	$srcstr = $strobj->kanah2z($srcstr);

	my $len = length($srcstr);
	my $prech = 0;
	my $ch;
	my $ch2;

	for (my $i = 0; $i < $len; $i++) {
		$ch = substr($srcstr, $i, 1);
		# 全角文字
		if ($ch =~ /[\x81-\x9F\xE0-\xEF]/) { 
			$i++;
			$ch2 = substr($srcstr, $i, 1);
			if ($ch =~ /[\x81]/) {
				if ($ch2 =~ /[\x5B]/) {	# 長音記号
					if ($prech != 2 && $deststr ne '') {
						$deststr .= ' ';
					}
					$deststr .= $ch;
					$deststr .= $ch2;
					$prech = 2;
				}
				else {
					$deststr .= ' ';
					$prech = 0;
				}
			}
			elsif ($ch =~ /[\x82]/) {
				if ($ch2 =~ /[\x9F-\xF1]/) {	# ひらがな
					if ($prech != 1 && $deststr ne '') {
						$deststr .= ' ';
					}
					$deststr .= $ch;
					$deststr .= $ch2;
					$prech = 1;
				}
			}
			elsif ($ch =~ /[\x83]/)  {
				if ($ch2 =~ /[\x40-\x96]/) {
					if ($prech != 2 && $deststr ne '') {	# カタカナ（全角）
						$deststr .= ' ';
					}
					$deststr .= $ch;
					$deststr .= $ch2;
					$prech = 2;
				}
			}
			elsif ($ch =~ /[\x88-\x9F\xE0-\xEE]/) {	# 漢字
				if ($prech != 3 && $deststr ne '') {
					$deststr .= ' ';
				}
				$deststr .= $ch;
				$deststr .= $ch2;
				$prech = 3;
			}
			else {
				$deststr .= ' ';
				$prech = 0;
			}
			next;
		}

		# 半角英数
		if ($ch =~ /[\x20-\x7E]/) {	# ASCII文字
			if ($prech != 0 && $deststr ne '') {
				$deststr .= ' ';
			}
			if ($ch =~ /[\x20\x30-\x39\x41-\x5a\x61-\x7a]/) {
				$ch = uc($ch);	# 大文字に統一
				$deststr .= $ch;
			}
			else {	# 英数字以外のASCII文字は全てスペースに変換
				$deststr .= ' ';
			}
			$prech = 0;
			next;
		}
	}

	# 連続した半角スペースを取り除く
	$deststr =~ s/[ ]+/ /g;

	return $deststr;
}

# 索引検索 (スペース区切りのAND検索のみサポート)
sub indexsearch {
	my $this = shift;
	my $querystr = shift;

#	my $loggerobj = APPLIB::Logger->new();
#	$loggerobj->write("indexsearch - querystr:$querystr");	# for debug

	# 検索条件の調整
	$querystr = $this->adjustquery($querystr);

	# 半角スペースを区切りとしてキーワードに分解
	my @keywords = split(/ /, $querystr);

	my @resultset = ();
	foreach my $item (@keywords) {
#		$loggerobj->write("indexsearch - item:$item");	# for debug
		my $list = $this->find($item);
		my @idset = split(/:/, $list);
		if (@resultset == 0) {
			@resultset = @idset;
			next
		}
		else {	# and 検索
			my @newset = ();
			foreach my $id (@idset) {
				foreach my $tempid (@resultset) {
					if ($id eq $tempid) {
						push @newset, $id;
						last;
					}
				}
			}
			@resultset = @newset;
		}
	}

	return @resultset;
}

# インデックスデータを検索
sub find {
	my $this = shift;
	my $item = shift;
	my $list;

#	my $loggerobj = APPLIB::Logger->new();
#	$loggerobj->write("find - item:$item");	# for debug

	my $indexfile = $this->getfilename($item);
#	$loggerobj->write("find - indexfile:$indexfile");	# for debug
	if ($indexfile eq '') {
		return $list;
	}

	# インデックスファイルを検索
	open IN, $indexfile;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne '') {
			my ($keyword,$topiclist,$topicnum) = (split(/\,/, $data))[0..2];
			if ($keyword eq $item) {
				$list = $topiclist;
				last;
			}
		}
	}
	close IN;

#	$loggerobj->write("find - list:$list");	# for debug

	return $list;
}

# インデックスファイル名を取得
sub getfilename {
	my $this = shift;
	my $item = shift;
	my $filename = '';
	if ($item eq '') {
		return $filename;
	}
	my $len = length($item);
	my $ch = substr($item, 0, 1);
	my $ch2;
	if ($ch !~ /[\x30-\x39\x41-\x5a]/) {	# 英数以外
		$ch2 = substr($item, 1, 1);
	}

	if ($ch =~ /[\x30-\x39]/) {	# 数字
		$filename = "num$ch" . "l$len.data";
	}
	elsif ($ch =~ /[\x41-\x5a]/) {	# アルファベット
		$filename = "alpha$ch" . "l$len.data";
	}
	elsif (($ch =~ /[\x82]/) && ($ch2 =~ /[\x9F-\xF1]/)) {	# ひらがな
		if ($ch2 =~ /[\x9F-\xA8]/) {	# 0 829F - 82A8  ぁ あ ぃ い ぅ う ぇ え ぉ お
			$filename = "hiragana0l$len.data";
		}
		elsif ($ch2 =~ /[\xA9-\xB2]/) {	# 1 82A9 - 82B2  か が き ぎ く ぐ け げ こ ご
			$filename = "hiragana1l$len.data";
		}
		elsif ($ch2 =~ /[\xB3-\xBC]/) {	# 2 82B3 - 82BC  さ ざ し じ す ず せ ぜ そ ぞ
			$filename = "hiragana2l$len.data";
		}
		elsif ($ch2 =~ /[\xBD-\xC7]/) {	# 3 82BD - 82C7  た だ ち ぢ っ つ づ て で と ど
			$filename = "hiragana3l$len.data";
		}
		elsif ($ch2 =~ /[\xC8-\xCC]/) {	# 4 82C8 - 82CC  な に ぬ ね の
			$filename = "hiragana4l$len.data";
		}
		elsif ($ch2 =~ /[\xCD-\xDB]/) {	# 5 82CD - 82DB  は ば ぱ ひ び ぴ ふ ぶ ぷ へ べ ぺ ほ ぼ ぽ
			$filename = "hiragana5l$len.data";
		}
		elsif ($ch2 =~ /[\xDC-\xE0]/) {	# 6 82DC - 82E0  ま み む め も
			$filename = "hiragana6l$len.data";
		}
		elsif ($ch2 =~ /[\xE1-\xE6]/) {	# 7 82E1 - 82E6  ゃ や ゅ ゆ ょ よ
			$filename = "hiragana7l$len.data";
		}
		elsif ($ch2 =~ /[\xE7-\xEB]/) {	# 8 82E7 - 82EB  ら り る れ ろ
			$filename = "hiragana8l$len.data";
		}
		else {	# 9 82EC - 82F1  ゎ わ ゐ ゑ を ん
			$filename = "hiragana9l$len.data";
		}
	}
	elsif (($ch =~ /[\x83]/) && ($ch2 =~ /[\x40-\x96]/)) {	# カタカナ（全角）
		if ($ch2 =~ /[\x40-\x49]/) {	# 0 8340 - 8349  ァ ア ィ イ ゥ ウ ェ エ ォ オ
			$filename = "katakana0l$len.data";
		}
		elsif ($ch2 =~ /[\x4A-\x53]/) {	# 1 834A - 8353  カ ガ キ ギ ク グ ケ ゲ コ ゴ
			$filename = "katakana1l$len.data";
		}
		elsif ($ch2 =~ /[\x54-\x5D]/) {	# 2 8354 - 835D  サ ザ シ ジ ス ズ セ ゼ ソ ゾ
			$filename = "katakana2l$len.data";
		}
		elsif ($ch2 =~ /[\x5E-\x68]/) {	# 3 835E - 8368  タ ダ チ ヂ ッ ツ ヅ テ デ ト ド
			$filename = "katakana3l$len.data";
		}
		elsif ($ch2 =~ /[\x69-\x6D]/) {	# 4 8369 - 836D  ナ ニ ヌ ネ ノ
			$filename = "katakana4l$len.data";
		}
		elsif ($ch2 =~ /[\x6E-\x7C]/) {	# 5 836E - 837C  ハ バ パ ヒ ビ ピ フ ブ プ ヘ ベ ペ ホ ボ ポ
			$filename = "katakana5l$len.data";
		}
		elsif ($ch2 =~ /[\x7D-\x82]/) {	# 6 837D - 8382  マ ミ ム メ モ
			$filename = "katakana6l$len.data";
		}
		elsif ($ch2 =~ /[\x83-\x88]/) {	# 7 8383 - 8388  ャ ヤ ュ ユ ョ ヨ
			$filename = "katakana7l$len.data";
		}
		elsif ($ch2 =~ /[\x89-\x8D]/) {	# 8 8389 - 838D  ラ リ ル レ ロ
			$filename = "katakana8l$len.data";
		}
		else {	# 9 838E - 8396  ヮ ワ ヰ ヱ ヲ ン ヴ ヵ ヶ
			$filename = "katakana9l$len.data";
		}
	}
	elsif ($ch =~ /[\x88-\x9F\xE0-\xEE]/) {	# 漢字
		if ($ch =~ /[\x88-\x89]/) {	# 0 88 - 89
			$filename = "kanji0l$len.data";
		}
		elsif ($ch =~ /[\x90-\x91]/) {	# 1 90 - 91
			$filename = "kanji1l$len.data";
		}
		elsif ($ch =~ /[\x92-\x93]/) {	# 2 92 - 93
			$filename = "kanji2l$len.data";
		}
		elsif ($ch =~ /[\x94-\x95]/) {	# 3 94 - 95
			$filename = "kanji3l$len.data";
		}
		elsif ($ch =~ /[\x96-\x97]/) {	# 4 96 - 97
			$filename = "kanji4l$len.data";
		}
		elsif ($ch =~ /[\x98-\x99]/) {	# 5 98 - 99
			$filename = "kanji5l$len.data";
		}
		elsif ($ch =~ /[\x9A-\x9B]/) {	# 6 9A - 9B
			$filename = "kanji6l$len.data";
		}
		elsif ($ch =~ /[\x9C-\x9D]/) {	# 7 9C - 9D
			$filename = "kanji7l$len.data";
		}
		elsif ($ch =~ /[\x9E-\x9F]/) {	# 8 9E - 9F
			$filename = "kanji8l$len.data";
		}
		else {	# 1 E040 - EEE0  漾 ～ 黑
			$filename = "kanji9l$len.data";
		}
	}
	if ($filename ne '') {
		$filename = "$this->{DATADIR}/search/" . $filename;
	}

#	print "getfilename - item:$item\n";
#	print "getfilename - filename:$filename\n";

	return $filename;
}

# 検索ボックスの作成
sub searchbox {
	my $this = shift;
	my $temp =  <<"End_of_Temp";
<form method="post" action="\$PROGRAM\$">
<input type="text" name="querystr" size="35" value="\$PREVQUERY\$">&nbsp;<input type="submit" value="検索">
</form>
End_of_Temp

	return $temp;
}

# タグ統合版検索ボックスの作成
sub searchbox2 {
	my $this = shift;
	my $temp =  <<"End_of_Temp";
<form method="post" action="\$PROGRAM\$">
<input type="text" name="querystr" size="35" value="\$PREVQUERY\$">&nbsp;
<select name="target">
 <option value="fulltext" \$TARGETTEXT\$>フルテキスト
 <option value="tag" \$TARGETTAG\$>タグ
</select>
&nbsp;
<input type="submit" value="検索">
</form>
End_of_Temp

	return $temp;
}

1;
# End of Search.pm
