#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Session;

use strict;
use warnings;

use APPLIB::Logger;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{USERID} = '';	# ユーザーID
	$this->{AUTH} = 0;			# ユーザー権限
	$this->{ERROR} = '';		# エラー情報

	bless($this);
	return $this;
}

# ユーザー認証、ログインOKの場合はクッキーに使用するためのセッションIDを返す
sub login {
	my $this = shift;
	my $userid = shift;		# 第一引数はユーザーID
	my $password = shift;	# 第二引数はパスワード
	my $ipaddr = shift;		# 第三引数はIPアドレス
	my $sessionid = 0;
	my $logincheck = "ng";

	$this->{ERROR} = '';		# エラー情報をクリア

	# userlist.txt を読み込んでユーザー登録されているかチェック
	my $userlist = "$this->{SYSTEMDIR}/userlist.txt";
	if (-f $userlist) {	# ファイルが存在する場合
		open IN, $userlist;
		while (<IN>) {
			my $data = $_;	# 1行読み込み
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				my ($olduserid, $oldpassword, $oldauth, $oldstatus) = (split(/\,/, $data))[0,1,5,6];
				if ($olduserid eq $userid && $oldpassword eq $password && $oldstatus eq "1") {
					$this->{AUTH} = $oldauth;	# ユーザー権限
					$logincheck = "ok";
					last;
				}
			}
		}
		close IN;
	}

	if ($logincheck eq "ok") {
		$this->{USERID} = $userid;	# ユーザーID

		my $sessionfile;
		for (;;) {
			$sessionid = int(rand(89999)) + 10000;	# 10000 ～ 99999 のセッションIDを生成
			$sessionfile = "$this->{SYSTEMDIR}/session/$sessionid.txt";
			if (-f $sessionfile) {	# 同じセッションIDのファイルが存在する場合はセッションIDを再生成
				next;
			}
			else {
				last;
			}
		}

		# セッション一覧ファイル(session.txt)を更新
		if (!open(SESSIONFILE, ">$sessionfile")) {
			$this->{ERROR} = "Can't open session file";		# エラー情報を設定
			return 0;
		}

		# 最後にログインしたユーザーのセッション情報を追加
		print SESSIONFILE "$sessionid,$userid,$ipaddr\n";
		flock(SESSIONFILE, $this->{LOCK_UN});
		close(SESSIONFILE);
		# ファイルのパーミッションを変更
		chmod 0766, $sessionfile;
	}

	return $sessionid;
}

# ログアウト(セッションの明示的な終了)
sub logout {
	my $this = shift;
	my $sessionid = shift;	# 第一引数はセッションID
	my $result = 1;			# とりあえず常に１を返す

	$this->{ERROR} = undef;		# エラー情報をクリア

	my $sessionfile = "$this->{SYSTEMDIR}/session/$sessionid.txt";
	unlink $sessionfile;

	return $result;
}

# セッションが有効かチェックする
sub check {
	my $this = shift;
	my $cksessionid = shift;	# 第一引数はクッキーのセッションID
	my $sessionid;
	my $userid = 0;

	$this->{ERROR} = '';		# エラー情報をクリア

	my $loggerobj = APPLIB::Logger->new();

	my $sessionfile = "$this->{SYSTEMDIR}/session/$cksessionid.txt";
#	$loggerobj->write("sessionfile:$sessionfile");	# for debug

	if (-f $sessionfile) {	# ファイルが存在する場合
		open IN, $sessionfile;
		my $data = <IN>;	# 1行読み込み
		chop($data);	# 改行を取り除く
#		$loggerobj->write("data:$data");	# for debug
		($sessionid,$userid) = (split(/\,/, $data))[0,1];
		if ($cksessionid eq $sessionid) {
			$this->{USERID} = $userid;	# ユーザーID
		}
		close IN;
	}

	$loggerobj->write("USERID:$this->{USERID}");	# for debug
#	$loggerobj->write("userid:$userid");	# for debug

	return $userid;
}

# ユーザー種別の取得
sub usertype {
	my $this = shift;
	my $userid = shift;	# 第一引数：ユーザーID
	my $id;
	my $auth = 0;

	my $userlist = "$this->{SYSTEMDIR}/userlist.txt";
	if (-f $userlist) {	# ファイルが存在する場合
		open IN, $userlist;
		while (my $data = <IN>) {
			chop($data);	# 改行を取り除く
			if ($data ne '') {
				($id,$auth) = (split(/\,/, $data))[0,5];
				if ($id eq $userid) {
					last;
				}
			}
		}
		close IN;
	}

	return $auth;
}

# 管理者メニュー
sub adminmenu {
	my $this = shift;

	my $menu = <<"End_of_Temp";
<a href="password.cgi?opecode=1">パスワード</a>&nbsp;&nbsp;
<a href="topic.cgi">新規投稿</a>&nbsp;&nbsp;
<a href="userlist.cgi">ユーザー管理</a>&nbsp;&nbsp;
<a href="rebuild.cgi">再構築</a>&nbsp;&nbsp;
<a href="logout.cgi">ログアウト</a><br />
<a href="control.cgi?opecode=3">お知らせ</a>&nbsp;&nbsp;
<a href="control.cgi?opecode=4">広告設定</a>&nbsp;&nbsp;
<a href="control.cgi?opecode=2">Ping設定</a>&nbsp;&nbsp;
<a href="template.cgi?opecode=0&filecode=1">テンプレート</a>&nbsp;&nbsp;
<a href="control.cgi?opecode=1">フィルター</a>
End_of_Temp

	return $menu;
}

# 編集者メニュー
sub editormenu {
	my $this = shift;

	my $menu = <<"End_of_Temp";
<a href="topic.cgi">新規投稿</a>&nbsp;&nbsp;
<a href="password.cgi?opecode=1">パスワード</a>&nbsp;&nbsp;
<a href="user.cgi?opecode=2&userid=$this->{USERID}">ユーザー情報</a>&nbsp;&nbsp;
<a href="logout.cgi">ログアウト</a>
End_of_Temp

	return $menu;
}

# ユーザーメニュー
sub usermenu {
	my $this = shift;

	my $menu = <<"End_of_Temp";
<a href="newtopic.cgi">新規投稿</a>&nbsp;&nbsp;
<a href="login.cgi">ログイン</a>
End_of_Temp

	return $menu;
}


1;
# End of Session.pm
