#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Tag;

use strict;
use warnings;

use APPLIB::Logger;
use APPLIB::String;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ
	$this->{DATADIR} = $confobj->DataDir();	# データディレクトリ
	$this->{DOCROOTDIR} = $confobj->DocrootDir;	# ドキュメントルート

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{TAGID} = 0;		# タグID
	$this->{TAGITEM} = '';	# タグ項目
	$this->{TOPICLIST} = '';	# トピックリスト（関連付けられているトピックのリスト、トピックIDを":"区切で格納）
	$this->{TOPICNUM} = 0;	# トピック数（関連付けられているトピック数）
	$this->{REFFERNUM} = 0;	# 参照回数

	bless($this);
	return $this;
}

# タグIDの取得
sub tagid {
	my $this = shift;
	if (@_) {
		$this->{TAGID} = shift;
	}
	return $this->{TAGID};
}

# タグ広告の取得
sub tagitem {
	my $this = shift;
	if (@_) {
		$this->{TAGITEM} = shift;
	}
	return $this->{TAGITEM};
}

# トピックリストの取得
sub topiclist {
	my $this = shift;
	if (@_) {
		$this->{TOPICLIST} = shift;
	}
	return $this->{TOPICLIST};
}

# トピック数の取得
sub topicnum {
	my $this = shift;
	if (@_) {
		$this->{TOPICNUM} = shift;
	}
	return $this->{TOPICNUM};
}

# 参照回数の取得
sub refernum {
	my $this = shift;
	if (@_) {
		$this->{REFFERNUM} = shift;
	}
	return $this->{REFFERNUM};
}

# タブの入力チェック（全角文字と半角英数のみタグとして使用可能）
sub check {
	my $this = shift;
	my $str = shift;
	my $result = 0;

	# UTF-8からシフトJISに変換
	my $strobj = APPLIB::String->new();
	$str = $strobj->encodeex('utf8', 'sjis', $str);

	my $len = length($str);
	for (my $i = 0; $i < $len; $i++) {
		my $ch = substr($str, $i, 1);
		if ($ch =~ /[\x81-\x9F\xE0-\xEF]/) { # 全角文字
			$i++;
			next;
		}
		elsif ($ch =~ /[\x20\x30-\x39\x41-\x5a\x61-\x7a]/) {	# 半角英数
			next;
		}
		else {
			return $result;
		}
	}
	$result = 1;

	return $result;
}

# タグを追加する
sub add {
	my $this = shift;
	my $item = shift;
	my $id = shift;
	my $result = 0;
	my $notfound = 1;
	my @newdata = ();
	my ($tagid,$tagitem,$topiclist,$topicnum,$refernum) = ();

	# 既存のタグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open IN, $tagindex;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne '') {
			($tagid,$tagitem,$topiclist,$topicnum,$refernum) = (split(/\,/, $data))[0..4];
			if (uc($tagitem) eq uc($item)) {
				my $list = $topiclist;
				my $num = $topicnum;

				# 既に登録されているかチェック
				my @idlist = split(/:/, $list);
				my $i;
				for ($i = 0; $i < $num; $i++) {
					if ($idlist[$i] == $id) {
						last;
					}
				}
				if ($i >= $num) {
					if ($num > 0) {
						$list .= ":";
					}
					$list .= "$id";
					$num++;
					
					$topiclist = $list;
					$topicnum = $num;
				}

				$data = "$tagid,$tagitem,$topiclist,$topicnum,$refernum";
				$result = 1;
				$notfound = 0;
			}
			push @newdata, $data;
		}
	}
	close IN;

	if ($notfound) {

		my $list = $id;
		my $num = 1;
		my $topiclist = $list;
		my $topicnum = $num;
		my $tagid = 0;

		# 新規タグIDを取得
		my $counterfile = "$this->{SYSTEMDIR}/tagcounter.txt";
		open IN, $counterfile;
		my $data = <IN>;	# 1行読み込み
		chop($data);	# 改行を取り除く
		if ($data ne "") {
			$tagid = $data;
		}
		close IN;
		$tagid++;

		# タグカウンタファイルを更新
		open(OUT,">$counterfile");
		flock(OUT, $this->{LOCK_EX});
		seek(OUT, 0, 2);
		print OUT "$tagid\n";
		flock(OUT, $this->{LOCK_UN});
		close(OUT);

		$data = "$tagid,$item,$topiclist,$topicnum,0";
		push @newdata, $data;
		$result = 1;
	}

	# タグデータファイルを更新
	open(OUT, ">$tagindex");
	flock(OUT, $this->{LOCK_EX});
	seek(OUT, 0, 2);
	foreach my $data (@newdata) {
		print OUT "$data\n";
	}
	flock(OUT, $this->{LOCK_UN});
	close(OUT);
	# ファイルのパーミッションを変更
	chmod 0766, $tagindex;

	return 	$result;
}

# タグIDをキーにタグ情報を読み込む
sub load {
	my $this = shift;
	my $id = shift;
	my $result = 0;

	# 既存のタグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open IN, $tagindex;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne '') {
			my ($tagid,$tagitem,$topiclist,$topicnum,$refernum) = (split(/\,/, $data))[0..4];
			if ($tagid == $id) {
				$this->{TAGID} = $tagid;
				$this->{TAGITEM} = $tagitem;
				$this->{TOPICLIST} = $topiclist;
				$this->{TOPICNUM} = $topicnum;
				$this->{REFFERNUM} = $refernum;
				$result = 1;
				last;
			}
		}
	}
	close IN;

	return $result;
}

# タグ項目配列をキーにタグ情報を読み込む
sub find {
	my $this = shift;
	my $item = shift;
	my $result = 0;

	# 既存のタグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open IN, $tagindex;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne '') {
			my ($tagid,$tagitem,$topiclist,$topicnum,$refernum) = (split(/\,/, $data))[0..4];
			if ($tagitem eq $item) {
				$this->{TAGID} = $tagid;
				$this->{TAGITEM} = $tagitem;
				$this->{TOPICLIST} = $topiclist;
				$this->{TOPICNUM} = $topicnum;
				$this->{REFFERNUM} = $refernum;
				$result = 1;
				last;
			}
		}
	}
	close IN;

	return $result;
}

# タグ項目配列からタグ一覧呼び出しリストを作成する
sub list {
	my $this = shift;
	my @items = @_;
	my $result = '';
	my $itemlist = '';

#	my $loggerobj = Logger->new();

	# 既存のタグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open IN, $tagindex;
	while (<IN>) {
		my $data = $_;
		chop $data;
#		$loggerobj->write("data:$data");	# for debug
		if ($data ne '') {
			my ($tagid,$tagitem) = (split(/\,/, $data))[0,1];
			foreach my $item (@items) {
				if (uc($tagitem) eq uc($item)) {
					$item =~ s/(\W)/'%' . unpack('H2', $1)/eg;	# uriパック
					$itemlist .= "<a href=\"$this->{DOCROOTDIR}/search.cgi?querystr=$item&target=tag\" title=\"$tagitem\">$tagitem</a>&nbsp;";
				}
			}
		}
	}
	close IN;

	if ($itemlist ne '') {
		$result = <<"End_of_Temp";
<table border="0" width="100%" cellpadding="3" cellspacing="0" bgcolor="#FFFFFF">
<tr bgcolor="#EEEEEE"><td valign="top" width="30"><b>タグ</b></td><td valign="top">$itemlist</td></tr>
</table>
End_of_Temp
	}

	return $result;
}

# タグをキーに指定されたIDを削除する
sub delete {
	my $this = shift;
	my $item = shift;
	my $id = shift;
	my $result = 0;
	my $notfound = 1;
	my @newdata = ();

	my $loggerobj = APPLIB::Logger->new();

	# 既存のタグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open IN, $tagindex;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne '') {
			my ($tagid,$tagitem,$topiclist,$topicnum,$refernum) = (split(/\,/, $data))[0..4];
			if ($tagitem eq $item) {

				my $list = $topiclist;
				my $num = $topicnum;

				# 登録されているかチェック
				my $newlist = '';
				my @idlist = split(/:/, $list);
				for (my $i = 0; $i < $num; $i++) {
					if ($idlist[$i] != $id) {
						if ($newlist ne '') {
							$newlist .= ":";
						}
						$newlist .= $idlist[$i];
					}
					else {
						$notfound = 0;
					}
				}

				# 削除した場合は再編成
				if (!$notfound) {
					$num--;
					$topiclist = $newlist;
					$topicnum = $num;
					$data = "$tagid,$tagitem,$topiclist,$topicnum,$refernum";
					$result = 1;
				}
			}
			push @newdata, $data;
		}
	}
	close IN;

	if (!$notfound) {
		# タグデータファイルを更新
		open(OUT,">$tagindex");
		flock(OUT, $this->{LOCK_EX});
		seek(OUT, 0, 2);
		foreach my $data (@newdata) {
			print OUT "$data\n";
		}
		flock(OUT, $this->{LOCK_UN});
		close(OUT);
		# ファイルのパーミッションを変更
		chmod 0766, $tagindex;
	}

	return $result;
}

# タグクラウドの取得
sub clouds {
	my $this = shift;
	my $format = shift; # 0:強調なし 1:強調あり
	my $itemlist = '';
	my ($tagid,$tagitem,$topicnum,$refernum) = ();
	my $max = 0;	# 最大値
	my $p1;			# 平均値 + ((最大値 - 平均値) / 2)
	my $avrg;		# 平均値
	my $m1;			# 平均値 - ((平均値 - 最小値) / 2)
	my $min = 0;	# 最小値
	my $num;
	my @dataarray = ();
	my $itemnum = 0;
	my $totalnum = 0;

	my $loggerobj = APPLIB::Logger->new();

	# タグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open IN, $tagindex;
	while (<IN>) {
		my $data = $_;
		chop $data;
		if ($data ne '') {
			($tagid,$tagitem,$topicnum,$refernum) = (split(/\,/, $data))[0,1,3,6];
			$num = $topicnum + $refernum;
			if ($num > $max) {
				$max = $num;
			}
			if ($num < $min) {
				$min = $num;
			}
			if ($num > 0) {
				$totalnum += $num;
				$itemnum++;
				push(@dataarray, $data);
			}
		}
	}
	close IN;

	if ($itemnum > 0) {
		$avrg = $totalnum / $itemnum;
	}
	else {
		$avrg = 0;
	}
	$p1 = $avrg + (($max - $avrg) / 2);
	$m1 = $avrg - (($avrg - $min) / 2);

#	$loggerobj->write("max:$max");		# for debug
#	$loggerobj->write("avrg:$avrg");	# for debug
#	$loggerobj->write("min:$min");		# for debug

	foreach my $data (@dataarray) {
		($tagid,$tagitem,$topicnum,$refernum) = (split(/\,/, $data))[0,1,3,6];
		$num = $topicnum + $refernum;

		# フォントサイズを設定
		my $fontsize;
		if ($num == $avrg) {
			$fontsize = "+0";
		}
		elsif ($num > $avrg) {
			if ($num <= $p1) {
				$fontsize = "+1";
			}
			else {
				$fontsize = "+2";
			}
		}
		else {
			if ($num >= $m1) {
				$fontsize = "-1";
			}
			else {
				$fontsize = "-2";
			}
		}

		my $item = $tagitem;
		$item =~ s/(\W)/'%' . unpack('H2', $1)/eg;	# uriパック
		$itemlist .= <<"End_of_Temp";
<a href="search.cgi?querystr=$item&target=tag" title="$tagitem"><font size="$fontsize">$tagitem</font></a>&nbsp;
End_of_Temp
	}
	close IN;

	my $html = <<"End_of_Temp";
<div>$itemlist</div>
End_of_Temp

	return $html;
}

# 縮小版タグクラウドの取得
sub smallclouds {
	my $this = shift;
	my $format = shift; # 0:強調なし 1:強調あり
	my $itemlist = '';
	my ($tagid,$tagitem,$topicnum,$refernum) = ();
	my $max = 0;	# 最大値
	my $p1;			# 平均値 + ((最大値 - 平均値) / 2)
	my $avrg;		# 平均値
	my $m1;			# 平均値 - ((平均値 - 最小値) / 2)
	my $min = 0;	# 最小値
	my $num;
	my @dataarray = ();
	my $itemnum = 0;
	my $totalnum = 0;

	my $loggerobj = APPLIB::Logger->new();

	# タグ情報を読み込む
	my $tagindex = "$this->{DATADIR}/tag/tagindex.data";
	open TAGINDEX, $tagindex;
	while (my $data = <TAGINDEX>) {
		chop $data;
		if ($data ne '') {
			($tagid,$tagitem,$topicnum,$refernum) = (split(/\,/, $data))[0,1,3,6];
			$num = $topicnum + $refernum;
			if ($num > $max) {
				$max = $num;
			}
			if ($num < $min) {
				$min = $num;
			}
			if ($num > 0) {
				$totalnum += $num;
				$itemnum++;
				push(@dataarray, $data);
			}
		}
	}
	close TAGINDEX;

	if ($itemnum > 0) {
		$avrg = $totalnum / $itemnum;
	}
	else {
		$avrg = 0;
	}
	$p1 = $avrg + (($max - $avrg) / 2);
	$m1 = $avrg - (($avrg - $min) / 2);

	foreach my $data (@dataarray) {
		($tagid,$tagitem,$topicnum,$refernum) = (split(/\,/, $data))[0,1,3,6];
		$num = $topicnum + $refernum;

		# フォントサイズを設定
		my $fontsize;
		if ($num > $avrg) {
			if ($num <= $p1) {
				$fontsize = "+0";
			}
			else {
				$fontsize = "+1";
			}
		}

		# 平均値より多いものだけ表示
		if ($num > $avrg) {
			my $item = $tagitem;
			$item =~ s/(\W)/'%' . unpack('H2', $1)/eg;	# uriパック
			$itemlist .= <<"End_of_Temp";
<a href="search.cgi?querystr=$item&target=tag" title="$tagitem"><font size="$fontsize">$tagitem</font></a>&nbsp;
End_of_Temp
		}
	}
	close IN;

	my $html = <<"End_of_Temp";
<div>$itemlist</div>
End_of_Temp

	return $html;
}

1;
# End of Tag.pm
