#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Topic;

use strict;
use warnings;

use APPLIB::DateTime;
use APPLIB::Calendar;
use APPLIB::String;
use APPLIB::Archive;
use APPLIB::Ad;
use APPLIB::User;
use APPLIB::Search;
use APPLIB::Tag;
use APPLIB::Logger;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ
	$this->{DATADIR} = $confobj->DataDir();		# データディレクトリ
	$this->{DOCROOT} = $confobj->DocrootDir();	# ドキュメントルート
	$this->{COMMENTTARGET} = $confobj->CommentTarget();
	$this->{TBTARGET} = $confobj->TrackbackTarget();
	$this->{LINKTARGET} = $confobj->LinkTarget();
	$this->{SITETITLE} = $confobj->SiteTitle();	# サイト名
	$this->{SUBTITLE} = $confobj->SubTitle();	# サイトの副題
	$this->{ANONYMOUS} = $confobj->Anonymous();	# 匿名の場合の代替文字列

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{MODE} = 1;			# 動作モード 0:HTMLの生成のみ(編集メニューあり) 1:HTMLファイルに出力(編集メニューなし)
	$this->{USERTYPE} = 1;		# ユーザー種別 0:管理者 1:編集者
	$this->{USERID} = '';		# ユーザーID

	$this->{TOPICID} = '';		# トピックID
	$this->{LASTDATE} = '';		# 最終更新日
	$this->{STATUS} = 0;		# 状態
	$this->{EDITOR} = '';		# 編集者
	$this->{HANDLE} = '';		# ハンドル名
	$this->{TITLE} = '';		# タイトル
	$this->{TEXT} = '';			# 本文
	$this->{COMMENTNUM} = 0;	# コメント数
	$this->{TBNUM} = 0;			# トラックバック数
	$this->{LINKNUM} = 0;		# 関連情報数
	$this->{ADLINK1} = '';		# 広告リンク１（468x60）
	$this->{ADLINK2} = '';		# 広告リンク２（300x250）
	$this->{POINT} = 0;			# ポイント
	$this->{TAGS} = '';			# タグ

	$this->{HTMLNAME} = '';		# HTMLファイル名
	$this->{ERROR} = '';		# エラー情報

	bless($this);
	return $this;
}

# 動作モード 0:HTMLの生成のみ 1:HTMLファイルに出力
sub mode {
	my $this = shift;
	if (@_) {
		$this->{MODE} = shift;
	}
	return $this->{MODE};
}

# ユーザー種別 0:管理者 1:編集者
sub usertype {
	my $this = shift;
	if (@_) {
		$this->{USERTYPE} = shift;
	}
	return $this->{USERTYPE};
}

# ユーザーID
sub userid {
	my $this = shift;
	if (@_) {
		$this->{USERID} = shift;
	}
	return $this->{USERID};
}

# トピックID
sub topicid {
	my $this = shift;
	if (@_) {
		$this->{TOPICID} = shift;
	}
	return $this->{TOPICID};
}

# トピックの最終更新日 YYYY/MM/DD HH:MM:SS
sub lastdate {
	my $this = shift;
	if (@_) {
		$this->{LASTDATE} = shift;
	}
	return $this->{LASTDATE};
}

# 状態
sub status {
	my $this = shift;
	if (@_) {
		$this->{STATUS} = shift;
	}
	return $this->{STATUS};
}

# トピック編集者
sub editor {
	my $this = shift;
	if (@_) {
		$this->{EDITOR} = shift;
	}
	return $this->{EDITOR};
}

# ハンドル名
sub handle {
	my $this = shift;
	if (@_) {
		$this->{HANDLE} = shift;
	}
	return $this->{HANDLE};
}

# タイトル
sub title {
	my $this = shift;
	if (@_) {
		$this->{TITLE} = shift;
	}
	return $this->{TITLE};
}

# 本文
sub text {
	my $this = shift;
	if (@_) {
		$this->{TEXT} = shift;
	}
	return $this->{TEXT};
}

# コメント数
sub commentnum {
	my $this = shift;
	if (@_) {
		$this->{COMMENTNUM} = shift;
	}
	return $this->{COMMENTNUM};
}

# トラックバック数
sub tbnum {
	my $this = shift;
	if (@_) {
		$this->{TBNUM} = shift;
	}
	return $this->{TBNUM};
}

# 関連情報数
sub linknum {
	my $this = shift;
	if (@_) {
		$this->{LINKNUM} = shift;
	}
	return $this->{LINKNUM};
}

# 広告リンク１
sub adlink1 {
	my $this = shift;
	if (@_) {
		$this->{ADLINK1} = shift;
	}
	return $this->{ADLINK1};
}

# 広告リンク２
sub adlink2 {
	my $this = shift;
	if (@_) {
		$this->{ADLINK2} = shift;
	}
	return $this->{ADLINK2};
}

# ポイント
sub point {
	my $this = shift;
	if (@_) {
		$this->{POINT} = shift;
	}
	return $this->{POINT};
}

# タグ
sub tags {
	my $this = shift;
	if (@_) {
		$this->{TAGS} = shift;
	}
	return $this->{TAGS};
}

# HTMLファイル名
sub htmlname {
	my $this = shift;
	if (@_) {
		$this->{HTMLNAME} = shift;
	}
	return $this->{HTMLNAME};
}

# 更新
sub update {
	my $this = shift;
	my $datafile = "$this->{DATADIR}/$this->{TOPICID}tp.data";
	my $result = 1;

	# トピックデータファイルを更新
	open(DATAFILE,">$datafile");
	flock(DATAFILE, $this->{LOCK_EX});
	seek(DATAFILE, 0, 2);
	print DATAFILE "$this->{TOPICID},$this->{LASTDATE},$this->{STATUS},$this->{EDITOR},$this->{HANDLE},";
	print DATAFILE "$this->{TITLE},$this->{TEXT},";
	print DATAFILE "$this->{COMMENTNUM},$this->{LINKNUM},$this->{TBNUM},";
	print DATAFILE "$this->{ADLINK1},$this->{ADLINK2},$this->{POINT}\n";
	flock(DATAFILE, $this->{LOCK_UN});
	close(DATAFILE);
	chmod 0766, $datafile;

	# タグ情報を保存
	my $tagfile = "$this->{DATADIR}/$this->{TOPICID}" . "tg.data";
	open(TAGFILE,">$tagfile");
	flock(TAGFILE, $this->{LOCK_EX});
	seek(TAGFILE, 0, 2);
	print TAGFILE "$this->{TAGS}\n";
	flock(TAGFILE, $this->{LOCK_UN});
	close(TAGFILE);
	chmod 0766, $tagfile;

	return $result;
}

# 削除
sub delete {
	my $this = shift;
	$this->{STATUS} = 9;	# 削除ステータスに設定して更新
	my $result = $this->update();
	return $result;
}

# 読込み
sub load {
	my $this = shift;
	my $topicid;

	if (@_) {
		$topicid = shift;
	}
	else {
		$topicid = $this->{TOPICID};
	}

	# トピックデータを読み込む
	my $datafile = $this->{DATADIR} . "/" . $topicid . "tp.data";
	my $result = $this->read($datafile);

	return $result;
}


# ファイル名指定して読込み
sub read {
	my $this = shift;
	my $filename = shift;
	my $result = 1;
	my $data;

	# トピックデータを読み込む
	my $datafile = $filename;
	if (-f $datafile) {	# ファイルが存在する場合
		open IN, $datafile;
		$data = <IN>;	# 1行読み込み
		chop($data);	# 改行を取り除く
		if ($data ne "") {
			($this->{TOPICID},$this->{LASTDATE},$this->{STATUS},$this->{EDITOR},$this->{HANDLE},$this->{TITLE},$this->{TEXT},$this->{COMMENTNUM},$this->{LINKNUM},$this->{TBNUM},$this->{ADLINK1},$this->{ADLINK2},$this->{POINT}) = (split(/\,/, $data))[0..12];
		}
		close IN;
	}
	else {
		$this->{ERROR} = "ファイルが存在しません";		# エラー情報
		return 0;
	}

	# タグ情報を取得
	my $tagfile = $this->{DATADIR} . "/" . $this->{TOPICID} . "tg.data";
	open IN, $tagfile;
	$data = <IN>;	# 1行読み込み
	chop($data);	# 改行を取り除く
	$this->{TAGS} = $data;
	close IN;

	return $result;
}

# HTMLの生成
sub genhtml {
	my $this = shift;
	my $html = undef;
	my $topicid = $this->{TOPICID};

	my $loggerobj = APPLIB::Logger->new();	# for debug

	# トピックHTMLファイル名を生成
	my $htmlname = "detail$topicid.html";
	$this->{HTMLNAME} = $htmlname;

	my $permalink = "$this->{DOCROOT}/$htmlname";

	# タイトルの編集
	my $caption = $this->{TITLE} . "：" . $this->{SITETITLE};
	my $sitetitle = "<a class=\"SITETITLE\" href=\"$this->{DOCROOT}/\">$this->{SITETITLE}</a>";
	my $subtitle = $this->{SUBTITLE};
	my $topictitle = $this->{TITLE};
	my $rssurl = "$this->{DOCROOT}/rss.xml";
	my $topicdetail = $this->gentopicdetail();		# トピック詳細を取得
	my $tburl = "$this->{DOCROOT}/rcvtb.cgi/$topicid";# トラックバックURL
	my $tblist = $this->gentblist();				# トラックバック一覧を取得
	my $topiccommentlist = $this->gencommentlist();	# コメント一覧を取得
	my $linklist = $this->genlinklist();			# 関連情報一覧を取得
	my $taglist ='';

	$topictitle =~ s/enc_conma/,/g;

	# メタタグ"description"の設定
	my $strobj = APPLIB::String->new();
	$strobj->tail('');
	my $descstr = $strobj->cut($this->{TEXT}, 200);
	my $metadisc = "<meta name=\"description\" content=\"$descstr\">";

	my $metakeywords = '';
	my $tagobj = APPLIB::Tag->new();

	# タグ情報を取得してメタタグの"keywords"を設定
	if ($this->{TAGS} ne '') {
		$metakeywords = $this->{TAGS};
		$metakeywords =~ s/[ ]/,/g;
		$metakeywords = "<meta name=\"keywords\" content=\"$metakeywords\">";

		# タグリストの作成
		my @tags = split(/ /, $this->{TAGS});
		$taglist = $tagobj->list(@tags);
	}

	my $calendarobj = APPLIB::Calendar->new();
	my ($year,$month,$day) = (split(/\//, $this->{LASTDATE}))[0..2];

	# カレンダーを作成
	$calendarobj->year($year);
	$calendarobj->month($month);
	my $calendar = $calendarobj->calendartable();

	# 最新アーカイブ一覧を取得
	my $archiveobj = APPLIB::Archive->new();
	my $archivelist = $archiveobj->archivetable();
	my $newtopic = $archiveobj->newtopic();

	# 広告の設定
	my $adlink1 = $this->{ADLINK1};
	my $adlink2 = $this->{ADLINK2};

	# ユーザー広告の取得
	if ($this->{EDITOR} ne '') {
		my $userobj = APPLIB::User->new();
		$userobj->load($this->{EDITOR});
		if ($adlink1 eq '') {
			$adlink1 = $userobj->adlink1();
		}
		if ($adlink2 eq '') {
			$adlink2 = $userobj->adlink2();
		}
	}

	# デフォルト広告の取得
	my $adobj = APPLIB::Ad->new();
	if ($this->{EDITOR} eq '') {
		$adlink1 = $adobj->adlink1();
		$adlink2 = $adobj->adlink2();
	}
	my $adlist = $adobj->adlist();
	$adlink1 =~ s/enc_conma/,/g;
	$adlink2 =~ s/enc_conma/,/g;
	$adlink1 =~ s/<br>/\n/g;
	$adlink2 =~ s/<br>/\n/g;

	# タグクラウドを取得
	my $tagclouds = $tagobj->smallclouds(0);

#	$loggerobj->write("adlist:$adlist");	# for debug

	# 検索ボックスの取得
	my $searchobj = APPLIB::Search->new();
	my $searchbox = $searchobj->searchbox2();
	my $program = "search.cgi";
	$searchbox =~ s/\$PROGRAM\$/$program/g;
	$searchbox =~ s/\$RESULT\$//g;
	$searchbox =~ s/\$PREVQUERY\$//g;
	$searchbox =~ s/\$TARGETTEXT\$/selected/g;
	$searchbox =~ s/\$TARGETTAG\$//g;

	# トピックHTMLファイルの生成
	open(TEMPLATE, "$this->{SYSTEMDIR}/detail.htmlt");
	local($/) = undef;
	my $template = <TEMPLATE>;
	close(TEMPLATE);

	my $commentform;
	if ($this->{STATUS} eq "1" || $this->{STATUS} eq "0") {
	 	$commentform = $this->gencommentform();
	}
	else {
	 	$commentform = "";
	}

	$template =~ s/\$METADISC\$/$metadisc/g;
	$template =~ s/\$METAKEYWORDS\$/$metakeywords/g;
	$template =~ s/\$RSSURL\$/$rssurl/g;
	$template =~ s/\$CAPTION\$/$caption/g;
	$template =~ s/\$SITETITLE\$/$sitetitle/g;
	$template =~ s/\$SUBTITLE\$/$subtitle/g;
	$template =~ s/\$TOPICTITLE\$/$topictitle/g;
	$template =~ s/\$TOPICDETAIL\$/$topicdetail/g;
	$template =~ s/\$TOPICCOMMENTLIST\$/$topiccommentlist/g;
	$template =~ s/\$COMMENTFORM\$/$commentform/g;
	$template =~ s/\$TBURL\$/$tburl/g;
	$template =~ s/\$TOPICTBLIST\$/$tblist/g;
	$template =~ s/\$TOPICLINKLIST\$/$linklist/g;
	$template =~ s/\$ADLINK1\$/$adlink1/g;
	$template =~ s/\$ADLINK2\$/$adlink2/g;
	$template =~ s/\$ADLIST\$/$adlist/g;
	$template =~ s/\$NEWTOPIC\$/$newtopic/g;
	$template =~ s/\$CALENDAR\$/$calendar/g;
	$template =~ s/\$ARCHIVELIST\$/$archivelist/g;
	$template =~ s/\$SEARCHBOX\$/$searchbox/g;
	$template =~ s/\$TAGLIST\$/$taglist/g;
	$template =~ s/\$TAGCLOUDS\$/$tagclouds/g;

	$html = $template;

	if ($this->{MODE} == 1) {
		$html =~ s/\$PERMALINK\$/$permalink/g;
		$html =~ s/\$LOGIN\$//g;
		# 詳細をHTMLファイルに出力
		open(HTMLFILE,">$htmlname");
		flock(HTMLFILE, $this->{LOCK_EX});
		seek(HTMLFILE, 0, 2);
		print HTMLFILE $html;
		flock(HTMLFILE, $this->{LOCK_UN});
		close(HTMLFILE);
		# ファイルのパーミッションを変更
		chmod 0766, $htmlname;
	}
	else {
		# 動的なページの場合はブログ内のトピックへのリンクに置換
		$html =~ s/href=\"detail([0-9]+).html/href=\"detail.cgi?topicid=$1/g;
		$html =~ s/href=\"$this->{DOCROOT}\/detail([0-9]+).html/href=\"detail.cgi?topicid=$1/g;
		$html =~ s/\$PERMALINK\$/$permalink/g;
	}

	return $html;
}

# トピック詳細を作成し、作成したHTMLを返す、loadが前の処理で呼び出されていることが前提
sub gentopicdetail {
	my $this = shift;
	my $topicid = $this->{TOPICID};
	my $mode = $this->{MODE};
	my $attribute;
	my $status = $this->{STATUS};
	my $text = $this->{TEXT};
	my $menu;
	my $topichtml;

	# トピックHTMLファイル名を生成
	$topichtml = "detail$topicid.html";
	$this->{HTMLNAME} = $topichtml;

	# コンマをデコード
	$text =~ s/enc_conma/,/g;

	my $fontcolor;
	if ($status == 0) {
		$fontcolor = "Orange";
	}
	elsif ($status == 1) {
		$fontcolor = "Darkgreen";
	}
	elsif ($status == 2) {
		$fontcolor = "Darkred";
	}
	else {
		$fontcolor = "Black";
	}

	if ($mode == 0) {
		if ($this->{USERTYPE} == 0 || ($this->{USERTYPE} == 1 && $this->{USERID} eq $this->{EDITOR})) {
			$menu = <<"End_of_Temp";
<a href="topic.cgi?topicid=$topicid&opecode=1">新規投稿</a>&nbsp;&nbsp;
<a href="topic.cgi?topicid=$topicid&opecode=2">編集</a>&nbsp;&nbsp;
<a href="link.cgi?topicid=$topicid&linkid=0&opecode=1">関連情報</a>&nbsp;&nbsp;
End_of_Temp
		}
		else {
			if ($this->{USERTYPE} == 1) {
				$menu = <<"End_of_Temp";
<a href="topic.cgi?topicid=$topicid&opecode=1">新規投稿</a>&nbsp;&nbsp;
End_of_Temp
			}
		}
	}
	else {	# HTMLファイル
		$menu = <<"End_of_Temp";
<a href="newtopic.cgi?topicid=$topicid&opecode=1">新規投稿</a>
End_of_Temp
	}

	my $editor = $this->{EDITOR};
	if ($editor eq '') {
		$editor = $this->{ANONYMOUS};
		if ($this->{HANDLE} ne '') {
			$editor = $this->{HANDLE};
		}
		# 匿名の場合はURLをリンク
		$text =~ s/(http:\/\/[^()<>\[\]{}"'\x7F-\xFF\x00-\x20]+)/<a href="$1" target="_blank">$1<\/a>/g;
	}
	else {
		my $userobj = APPLIB::User->new();
		$userobj->load($editor);
		my $mailaddr = $userobj->mailaddr();
		my $siteurl = $userobj->siteurl();
		my $username = $userobj->username();
		if ($username ne '') {
			$editor = $username;
		}
		if ($siteurl ne '') {
			$editor = "<a href=\"$siteurl\">$username</a>";
		}
		else {
			if ($mailaddr ne '') {
				$editor = "<a href=\"mailto:$mailaddr\">$username</a>";
			}
		}
	}

	my $detail = <<"End_of_Temp";
<table border="0" width="100%" cellpadding="3" cellspacing="0">
<tr>
<td valign="top">$text</td>
</tr>
<tr>
<td valign="top">
<font color="$fontcolor">[$topicid]</font> posted by <strong>$editor</strong> at $this->{LASTDATE} |
 <a href="$topichtml#comment">Comment ($this->{COMMENTNUM})</a> |
 <a href="$topichtml#trackback">Trackback ($this->{TBNUM})</a> |
 <b><font class="POINT" size="+1">$this->{POINT}</font> point</b>
</td>
</tr>
<tr>
<td>$menu</td>
</tr>
</table>
End_of_Temp

	return $detail;
}

# トピック概要のHTMLを作成して返す、loadが前の処理で呼び出されていることが前提
sub genoutline {
	my $this = shift;
	my $open = shift;
	my $topicid = $this->{TOPICID};
	my $attribute;
	my $status = $this->{STATUS};
	my $title = $this->{TITLE};
	my $text = $this->{TEXT};
	my $menu;
	my $topichtml;

	# トピックHTMLファイル名を生成
	$topichtml = "detail$topicid.html";
	$this->{HTMLNAME} = $topichtml;

	$title =~ s/enc_conma/,/g;
	$text =~ s/enc_conma/,/g;

	my $editor = $this->{EDITOR};
	if ($editor eq '') {
		$editor = $this->{ANONYMOUS};
		if ($this->{HANDLE} ne '') {
			$editor = $this->{HANDLE};
		}
		# 匿名の場合はURLをリンク
		$text =~ s/(http:\/\/[^()<>\[\]{}"'\x7F-\xFF\x00-\x20]+)/<a href="$1" target="_blank">$1<\/a>/g;
	}
	else {
		my $userobj = APPLIB::User->new();
		$userobj->load($editor);
		my $mailaddr = $userobj->mailaddr();
		my $siteurl = $userobj->siteurl();
		my $username = $userobj->username();
		if ($username ne '') {
			$editor = $username;
		}
		if ($siteurl ne '') {
			$editor = "<a href=\"$siteurl\">$username</a>";
		}
		else {
			if ($mailaddr ne '') {
				$editor = "<a href=\"mailto:$mailaddr\">$username</a>";
			}
		}
	}

	my $showblock = "display:block";
	my $hideblock = "display:none";
	if ($open == 1) {
		$showblock = "display:none";
		$hideblock = "display:block";
	}
	my $html = <<"End_of_Temp";
<div id="toggleShow$topicid" style="$showblock"><a href="javascript:;" onmousedown="toggleText('toggleText$topicid','toggleShow$topicid','toggleHide$topicid');">＋</a>&nbsp;<strong>[$topicid] <a href="$topichtml">$title</a></strong></div>
<div id="toggleHide$topicid" style="$hideblock"><a href="javascript:;" onmousedown="toggleText('toggleText$topicid','toggleShow$topicid','toggleHide$topicid');">－</a>&nbsp;<strong>[$topicid] <a href="$topichtml">$title</a></strong></div>
<div id="toggleText$topicid" style="$hideblock">$text</div>
<div>
<b><font class="POINT" size="+2">$this->{POINT}</font> point</b> |
 <a href="$topichtml#comment">Comment ($this->{COMMENTNUM})</a> |
 <a href="$topichtml#trackback">Trackback ($this->{TBNUM})</a> |
 posted by $editor at $this->{LASTDATE}
</div>
<br>
End_of_Temp

	return $html;
}

# コメント一覧作成、トピックのコメントの一覧を作成する
sub gencommentlist {
	my $this = shift;
	my $topicid = $this->{TOPICID};
	my $mode = $this->{MODE};
	my $htmlname = $this->{HTMLNAME};
	my $commentfile = $this->{DATADIR} . "/" . $topicid . "cm.data";
	my $list = '';
	my $menu;

	if (-f $commentfile) {	# ファイルが存在する場合
		$list = <<"End_of_Temp";
<table border="0" width="100%" cellpadding="3" cellspacing="0">
End_of_Temp

		open IN, $commentfile;
		while (<IN>) {
			my $data = $_;	# 1行読み込み
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				my ($commentid,$postdate,$author,$mailaddr,$hpurl,$text,$ipaddr) = (split(/\,/, $data))[0..6];
				if ($commentid ne "0") {
					# 匿名の置き換え
					if ($author eq "") {
						$author = $this->{ANONYMOUS};
					}

					# 編集メニューの表示
					if ($mode == 0) {
						if ($this->{USERTYPE} == 0 || ($this->{USERTYPE} == 1 && $this->{USERID} eq $this->{EDITOR})) {
							$menu =  <<"End_of_Temp";
<a href="delete.cgi?topicid=$topicid&id=$commentid&opecode=2">削除</a>
End_of_Temp
						}
					}

					# >>$commentidに対してリンクを張る。
					$text =~ s/>>([0-9]+)/<A HREF="$htmlname#comment$1">&gt;&gt;$1<\/A>/g;
					$text =~ s/&gt;&gt;([0-9]+)/<A HREF="$htmlname#comment$1">&gt;&gt;$1<\/A>/g;
					# URLに対してリンクを張る。
					$text =~ s/(http:\/\/[^()<>\[\]{}"'\x7F-\xFF\x00-\x20]+)/<A HREF="$1" $this->{COMMENTTARGET}>$1<\/A>/g;
					# コンマをデコード
					$text =~ s/enc_conma/,/g;
					# メールアドレスやホームページURLが入力されていた場合はリンク
					if ($hpurl ne "") {
						$author = "<a href=\"$hpurl\">$author</a>";
					}
					else {
						if ($mailaddr ne "") {
							$author = "<a href=\"mailto:$mailaddr\">$author</a>";
						}
					}
					my $temp = <<"End_of_Temp";
<tr>
<td><a name="comment$commentid"></a><b>$commentid</b> posted by <b>$author</b> at $postdate</td>
<td align="right" valign="top" nowrap>$menu</td>
</tr>
<tr>
<td colspan="2">$text</td>
</tr>
End_of_Temp
					# 後ろにつけていく
					$list .=  $temp;
				}
			}
		}
		close IN;

		$list .= "</table>";
	}

	return $list;
}


# トラックバック一覧作成、トピックのトラックバックの一覧を作成する
sub gentblist {
	my $this = shift;
	my $topicid = $this->{TOPICID};
	my $mode = $this->{MODE};
	my $htmlname = $this->{HTMLNAME};
	my $tbfile = $this->{DATADIR} . "/" . $topicid . "tb.data";
	my $list = '';
	my $menu;

	if (-f $tbfile) {	# ファイルが存在する場合
		$list = <<"End_of_Temp";
<table border="0" width="100%" cellpadding="3" cellspacing="0">
End_of_Temp

		open IN, $tbfile;
		while (<IN>) {
			my $data = $_;	# 1行読み込み
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				my ($tbid,$recvdate,$url,$blogname,$title,$excerpt) = (split(/\,/, $data))[0..5];
				if ($tbid ne "0") {

					# 編集メニューの表示
					if ($mode == 0) {
						if ($this->{USERTYPE} == 0 || ($this->{USERTYPE} == 1 && $this->{USERID} eq $this->{EDITOR})) {
							$menu = <<"End_of_Temp";
<a href="delete.cgi?topicid=$topicid&id=$tbid&opecode=1">削除</a>
End_of_Temp
						}
					}

					# コンマをデコード
					$title =~ s/enc_conma/,/g;
					$excerpt =~ s/enc_conma/,/g;
					$url =~ s/enc_conma/,/g;
					$blogname =~ s/enc_conma/,/g;

					my $temp = <<"End_of_Temp";
<tr>
<td><a name="trackback$tbid"></a>Linked from <b>$blogname</b> : <a href="$url" $this->{TBTARGET}>$title</a> at $recvdate</td>
<td align="right" valign="top" nowrap>$menu</td>
</tr>
<tr>
<td colspan="2">$excerpt</td>
</tr>
End_of_Temp
					# 後ろにつけていく
					$list .=  $temp;
				}
			}
		}
		close IN;

		$list .= "</table>";
	}

	return $list;
}

# 関連情報一覧作成、トピックの関連情報の一覧を作成する
sub genlinklist {
	my $this = shift;
	my $topicid = $this->{TOPICID};
	my $mode = $this->{MODE};
	my $htmlname = $this->{HTMLNAME};
	my $linkfile = $this->{DATADIR}. "/" . $topicid . "ln.data";
	my $list = '';
	my $menu = '';
	my $nodata = 1;

	if (-f $linkfile) {	# ファイルが存在する場合
		$list = <<"End_of_Temp";
<a name="link"></a>
<h4>関連情報</h4>
<table border="0" width="100%" cellpadding="3" cellspacing="0">
End_of_Temp

		open IN, $linkfile;
		while (<IN>) {
			my $data = $_;	# 1行読み込み
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				my ($linkid,$title,$url,$summary) = (split(/\,/, $data))[0..3];
				if ($linkid != 0) {

					# 編集メニューの表示
					if ($mode == 0) {
						if ($this->{USERTYPE} == 0 || ($this->{USERTYPE} == 1 && $this->{USERID} eq $this->{EDITOR})) {
							$menu = <<"End_of_Temp";
<a href="link.cgi?topicid=$topicid&linkid=$linkid&opecode=2">編集</a> <a href="link.cgi?topicid=$topicid&linkid=$linkid&opecode=3">削除</a>
End_of_Temp
						}
					}

					$nodata = 0;

					# コンマをデコード
					$title =~ s/enc_conma/,/g;
					$summary =~ s/enc_conma/,/g;

					my $temp = <<"End_of_Temp";
<tr>
<td><a href="$url" $this->{LINKTARGET}>$title</a></td>
<td align="right" valign="top" nowrap>$menu</td>
</tr>
<tr>
<td colspan="2">$summary</td>
</tr>
End_of_Temp
					# 後ろにつけていく
					$list .=  $temp;
				}
			}
		}
		close IN;

		$list .= "</table>";
	}

	if ($nodata) {
		$list = '';
	}

	return $list;
}

# 関連情報を追加
sub addlink {
	my $this = shift;
	my $topicid = shift;
	my ($title, $topicurl, $summary) = @_;
	my $linkid = $this->getlinkid($topicid);

	# 関連情報に追加
	my $linkfile = "$this->{DATADIR}/$topicid" . "ln.data";
	open(LINKFILE,">>$linkfile");
	flock(LINKFILE, $this->{LOCK_EX});
	seek(LINKFILE, 0, 2);
	print LINKFILE "$linkid,$title,$topicurl,$summary\n";
	flock(LINKFILE, $this->{LOCK_UN});
	close(LINKFILE);
	# ファイルのパーミッションを変更
	chmod 0766, $linkfile;

	return 1;
}

# 新規リンクIDの取得
sub getlinkid {
	my $this = shift;
	my $topicid = shift;

	# 関連情報IDの最大値を取得
	my $linkfile = "$this->{DATADIR}/$topicid" . "ln.data";
	my $lastlinkid = 0;
	if (-f $linkfile) {	# ファイルが存在する場合
		open IN, $linkfile;
		while (my $data = <IN>) {
			chop $data;
			if ($data ne '') {
				my $id = (split(/\,/, $data))[0];
				if ($id > $lastlinkid) {
					$lastlinkid = $id;
				}
			}
		}
		close IN;
	}

	$lastlinkid++;
	return $lastlinkid;
}

# コメント入力用フォームの生成
sub gencommentform {
	my $this = shift;

	my $commentform = <<"End_of_Form";
<!-- ここからコメント入力フォーム -->
<a name="commentform"></a>
<table border="0" cellpadding="3" cellspacing="0">
<tr><form method="post" action="comment.cgi">
<td>
<input type="hidden" name="topicid" value="$this->{TOPICID}">
名前：<br>
<input type="text" name="author" size="60"><br>
メールアドレス：<br>
<input type="text" name="mailaddr" size="60"><br>
ホームページURL：<br>
<input type="text" name="hpurl" size="60"><br>
<br>
<input type="checkbox" name="evaluate" value="1"><strong>おすすめ</strong> &nbsp;(チェックしてコメントすると更新情報に掲載)<br>
コメント：<br>
<textarea name="comment" rows="10" cols="60"></textarea>
<br>
<input type="submit" value="投稿">
</td>
</form></tr>
</table>
End_of_Form

    return $commentform;
}

1;
# End of Topic.pm
