#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::Trackback;

use strict;
use warnings;

use APPLIB::DateTime;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{DATADIR} = $confobj->DataDir();		# データディレクトリ
	$this->{NORMALPOINT} = $confobj->NormalPoint();

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{NEWNUM} = $confobj->NewTopicNum();

	bless($this);
	return $this;
}

# 最新トラックバックに追加
sub add {
	my $this = shift;
	my $topicid = shift;
	my $tbid = shift;
	my $tbtitle = shift;
	my @newdata = ();

	# 既存の最近のトラックバックファイルを読み込み配列に保存する
	my $newtbfile = "$this->{DATADIR}/newtb.data";
	if (-f $newtbfile) {	# ファイルが存在する場合
		open IN, $newtbfile;
		while (my $data = <IN>) {
			chop $data;
			if ($data ne '') {
				push @newdata, $data;
			}
		}
		close IN;
	}

	open(NEWTBFILE,">$newtbfile");
	flock(NEWTBFILE, $this->{LOCK_EX});
	seek(NEWTBFILE, 0, 2);

	# 新規作成のデータをファイルの先頭に追加
	print NEWTBFILE "$topicid,$tbid,$tbtitle\n";
	my $tbcount = 1;
	foreach my $data (@newdata) {
		print NEWTBFILE "$data\n";
		$tbcount++;
		last if ($this->{NEWNUM} == $tbcount);
	}

	flock(NEWTBFILE, $this->{LOCK_UN});
	close(NEWTBFILE);
	# ファイルのパーミッションを変更
	chmod 0766, $newtbfile;
}

# 新着トラックバックから削除
sub delete {
	my $this = shift;
	my $topicid = shift;
	my $tbid = shift;
	my @newdata = ();
	my $data;

	# 新着トラックバックファイルを読み込む
	my $newtbfile = "$this->{DATADIR}/newtb.data";
	open IN, $newtbfile;
	while ($data = <IN>) {
		chop $data;
		my ($newtopicid,$newtbid) = (split(/\,/, $data))[0,1];
		if (!($newtopicid eq $topicid && $tbid eq $newtbid)) {
			push @newdata, $data;
		}
	}
	close IN;

	# 新着トラックバックファイルを更新
	open(NEWTBFILE,">$newtbfile");
	flock(NEWTBFILE, $this->{LOCK_EX});
	seek(NEWTBFILE, 0, 2);
	foreach $data (@newdata) {
		print NEWTBFILE "$data\n";
	}
	flock(NEWTBFILE, $this->{LOCK_UN});
	close(NEWTBFILE);
}

# トラックバック表の取得(HTML)
sub trackbacktable {
	my $this = shift;

	my $table = <<"End_of_Temp";
<table border="0" width="180" cellpadding="3" cellspacing="0">
<tr>
<td><h5>最近のトラックバック</h5></td>
</tr>
End_of_Temp

	# 最近のトラックバックファイルを読み込み一覧を作成する
	my $newtbfile = "$this->{DATADIR}/newtb.data";
	if (-f $newtbfile) {	# ファイルが存在する場合
		open IN, $newtbfile;
		while (my $data = <IN>) {
			chop($data);	# 改行を取り除く
			if ($data ne '') {
				my ($topicid, $tbid, $title) = (split(/\,/, $data))[0,1,2];
				# コンマをデコード
				$title =~ s/enc_conma/,/g;
				# トピックのHTMLファイル名
				my $topichtml = "detail$topicid.html";
				my $temp = <<"End_of_Temp";
<tr>
<td>
<a href="$topichtml#trackback$tbid">$title</a>
</td>
</tr>
End_of_Temp

				$table .= $temp;
			}
		}
		close IN;
	}

	$table .= "</table>";

	return $table;
}

# トラックバックの読み込み
sub readtrackback {
	my $this = shift;
	my $topicid = shift;
	my $tbid = shift;
	my @rec = ();

	# トラックバックデータファイルを読み込む
	my $tbfile = "$this->{DATADIR}/$topicid" . "tb.data";
	if (-f $tbfile) {
		open IN, $tbfile;
		while (my $data = <IN>) {
			chop $data;
			@rec = split(/\,/, $data);
			if ($rec[0] eq $tbid) {
				last;
			}
		}
		close IN;
	}

	return @rec;
}

# トラックバックの追加
sub addtrackback {
	my $this = shift;
	my $topicid = shift;
	my ($url,$blogname,$title,$excerpt,$ipaddr) = @_;

	my $dtobj = APPLIB::DateTime->new();
	my $systemdate = $dtobj->localdatetime(0);
	my $lastcount = 0;

	my $tbfile = "$this->{DATADIR}/$topicid" . "tb.data";
	if (-f $tbfile) {
		# トラックバックID最大値を取得する
		open IN, $tbfile;
		while (my $data = <IN>) {
			chop $data;
			my $id = (split(/\,/, $data))[0];
			if ($id > $lastcount) {
				$lastcount = $id;
			}
		}
		close IN;
	}

	$lastcount++;
	my $tbid = $lastcount;

	# トラックバックデータファイルに追記
	open(OUT,">>$tbfile");
	flock(OUT, $this->{LOCK_EX});
	seek(OUT, 0, 2);
	print OUT "$tbid,$systemdate,$url,$blogname,$title,$excerpt,$ipaddr\n";
	flock(OUT, $this->{LOCK_UN});
	close(OUT);
	# ファイルのパーミッションを変更
	chmod 0766, $tbfile;

	return 1;
}

# トラックバックの削除
sub deltrackback {
	my $this = shift;
	my $topicid = shift;
	my $id = shift;
	my $lostpoint = 0;

	# トラックバックデータファイルを読み込む
	my $tbfile = "$this->{DATADIR}/$topicid" . "tb.data";
	if (-f $tbfile) {
		open IN, $tbfile;
		my $data;
		my @tbdata = ();
		while ($data = <IN>) {
			chop $data;
			my ($tbid,$tbdate,$url,$blogname,$title,$excerpt) = (split(/\,/, $data))[0..5];
			if ($tbid == $id) {	# 削除対象のトラックバックのIDを0にする
				$data = "0,$tbdate,$url,$blogname,$title,$excerpt";
				$lostpoint = $this->{NORMALPOINT};
			}
			push @tbdata, $data;
		}
		close(IN);

		# トラックバックファイルを更新
		open(OUT,">$tbfile");
		flock(OUT, $this->{LOCK_EX});
		seek(OUT, 0, 2);
		foreach $data (@tbdata) {
			print OUT "$data\n";
		}
		flock(OUT, $this->{LOCK_UN});
		close(OUT);

		# ファイルのパーミッションを変更
		chmod 0766, $tbfile;
	}

	return $lostpoint;
}

1;
# End of Trackback.pm
