#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::User;

use strict;
use warnings;

use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	# データの初期化
	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{USERID} = '';	# ユーザーID
	$this->{PASSWORD} = '';	# パスワード
	$this->{MAILADDR} = '';	# メールアドレス
	$this->{SITEURL} = '';	# WebサイトURL
	$this->{USERNAME} = '';	# ユーザー名
	$this->{AUTH} = '';		# 権限
	$this->{STATUS} = '';	# ステータス
	$this->{ADLINK1} = '';	# デフォルトの広告リンク１（ページ右上）
	$this->{ADLINK2} = '';	# デフォルトの広告リンク２（記事下）

	bless($this);
	return $this;
}

# ユーザーID
sub userid {
	my $this = shift;
	if (@_) {
		$this->{USERID} = shift;
	}
	return $this->{USERID};
}

# パスワード
sub password {
	my $this = shift;
	if (@_) {
		$this->{PASSWORD} = shift;
	}
	return $this->{PASSWORD};
}

# メールアドレス
sub mailaddr {
	my $this = shift;
	if (@_) {
		$this->{MAILADDR} = shift;
	}
	return $this->{MAILADDR};
}

# WebサイトURL
sub siteurl {
	my $this = shift;
	if (@_) {
		$this->{SITEURL} = shift;
	}
	return $this->{SITEURL};
}

# ユーザー名
sub username {
	my $this = shift;
	if (@_) {
		$this->{USERNAME} = shift;
	}
	return $this->{USERNAME};
}

# 権限
sub auth {
	my $this = shift;
	if (@_) {
		$this->{AUTH} = shift;
	}
	return $this->{AUTH};
}

# ステータス
sub status {
	my $this = shift;
	if (@_) {
		$this->{STATUS} = shift;
	}
	return $this->{STATUS};
}

# デフォルトの広告リンク１（ページ右上）
sub adlink1 {
	my $this = shift;
	if (@_) {
		$this->{ADLINK1} = shift;
	}
	return $this->{ADLINK1};
}

# デフォルトの広告リンク２（記事下）
sub adlink2 {
	my $this = shift;
	if (@_) {
		$this->{ADLINK2} = shift;
	}
	return $this->{ADLINK2};
}

# userlist.txt にユーザーを登録する
sub add {
	my $this = shift;
	my $result = 1;

	# ユーザーディレクトリを作成
	my $user_dir = "$this->{SYSTEMDIR}/$this->{USERID}";
	mkdir $user_dir, 0777;

	# ユーザー一覧ファイルのパス
	my $userlist = "$this->{SYSTEMDIR}/userlist.txt";

	# ユーザー一覧ファイルの最後に追加
	open(USERLIST,">>$userlist");
	flock(USERLIST, $this->{LOCK_EX});
	seek(USERLIST, 0, 2);
	print USERLIST "$this->{USERID},$this->{PASSWORD},$this->{MAILADDR},$this->{SITEURL},$this->{USERNAME},$this->{AUTH},$this->{STATUS},$this->{ADLINK1},$this->{ADLINK2}\n";
	flock(USERLIST, $this->{LOCK_UN});
	close(USERLIST);

	return $result;
}

# 指定されたユーザーIDでログイン
sub login {
	my $this = shift;
	my $userid = shift;
	my $password = shift;

	$this->load($userid);
	if (($userid eq $this->{USERID}) && ($password eq $this->{PASSWORD})) {
		return 1;
	}

	return 0;
}

# 指定されたユーザーIDの情報を取得
sub load {
	my $this = shift;
	my $userid = shift;
	my $data;
	my @temprec = ();

	my $userlistfile = "$this->{SYSTEMDIR}/userlist.txt";
	open IN, $userlistfile;
	while (<IN>) {
		$data = $_;	# 1行読み込み
		chop($data);	# 改行を取り除く
		@temprec = split(/\,/, $data);
		if ($userid eq $temprec[0]) {
			$this->{USERID} = $temprec[0];	
			$this->{PASSWORD} = $temprec[1];
			$this->{MAILADDR} = $temprec[2];
			$this->{SITEURL} = $temprec[3];
			$this->{USERNAME} = $temprec[4];
			$this->{AUTH} = $temprec[5];
			$this->{STATUS} = $temprec[6];
			$this->{ADLINK1} = $temprec[7];
			$this->{ADLINK2} = $temprec[8];
		}
	}
	close IN;
}

# 指定されたユーザーIDが登録されているかチェック
sub find {
	my $this = shift;
	my $userid = shift;
	my $mailaddr = shift;
	my $data;
	my @temprec = ();

	my $userlistfile = "$this->{SYSTEMDIR}/userlist.txt";
	open IN, $userlistfile;
	while (<IN>) {
		$data = $_;	# 1行読み込み
		chop($data);	# 改行を取り除く
		@temprec = split(/\,/, $data);
		if ($userid eq $temprec[0]) {
			return 1;
		}
		if ($mailaddr eq $temprec[2]) {
			return 2;
		}
	}
	close IN;

	return 0;
}

# ユーザーデータ設定、引数としてユーザー一覧ファイルの１行渡されることが前提
sub setuserdata {
	my $this = shift;
	my $data = shift;
	($this->{USERID},$this->{PASSWORD},$this->{MAILADDR},$this->{SITEURL},$this->{USERNAME},$this->{AUTH},$this->{STATUS},$this->{ADLINK1},$this->{ADLINK2}) = (split(/\,/, $data))[0..8];
}

# 更新
sub update {
	my $this = shift;
	my $result = 1;
	my $data;
	my $olduserid;
	my @userdata;

	# userlist.txt の該当ユーザーを更新する
	my $userlistfile = "$this->{SYSTEMDIR}/userlist.txt";
	open IN, $userlistfile;
	while (<IN>) {
		$data = $_;	# 1行読み込み
		chop($data);	# 改行を取り除く
		($olduserid) = (split(/\,/, $data))[0];
		# 該当ユーザーを差し替え
		if ($this->{USERID} eq $olduserid) {
			$data = "$this->{USERID},$this->{PASSWORD},$this->{MAILADDR},$this->{SITEURL},$this->{USERNAME},$this->{AUTH},$this->{STATUS},$this->{ADLINK1},$this->{ADLINK2}";
		}
		push @userdata, $data;
	}
	close IN;

	# ユーザー一覧ファイルを更新
	open(USERLIST,">$userlistfile");
	flock(USERLIST, $this->{LOCK_EX});
	seek(USERLIST, 0, 2);
	foreach $data (@userdata) {
		print USERLIST "$data\n";
	}
	flock(USERLIST, $this->{LOCK_UN});
	close(USERLIST);

	return $result;
}

1;
# End of User.pm
