#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

package APPLIB::View;

use strict;
use warnings;

use APPLIB::DateTime;
use APPLIB::Calendar;
use APPLIB::Archive;
use APPLIB::Topic;
use APPLIB::Ad;
use APPLIB::Tag;
use APPLIB::Search;
use APPLIB::Logger;
use APPLIB::Conf;

# コンストラクタ
sub new {
	my $this = {};

	my $confobj = APPLIB::Conf->new();

	$this->{SYSTEMDIR} = $confobj->SystemDir();	# システムディレクトリ
	$this->{DATADIR} = $confobj->DataDir();		# データディレクトリ
	$this->{DOCROOT} = $confobj->DocrootDir();		# ドキュメントルート
	$this->{SITETITLE} = $confobj->SiteTitle();	# サイト名
	$this->{SUBTITLE} = $confobj->SubTitle();		# サブタイトル
	$this->{ANONYMOUS} = $confobj->Anonymous();		# 匿名の場合の代替文字列

	$this->{LOCK_EX} = $confobj->LockEx();
	$this->{LOCK_UN} = $confobj->LockUn();

	$this->{FORMAT} = 0;			# 0:トップページ 1:月別 2:日別
	$this->{DATE} = '';				# 月別、日別で使用する年月日

	$this->{ERROR} = '';			# エラー情報

	bless($this);
	return $this;
}

# 出力形式
sub format {
	my $this = shift;
	if (@_) {
		$this->{FORMAT} = shift;
	}
	return $this->{FORMAT};
}

# 年月日
sub date {
	my $this = shift;
	if (@_) {
		$this->{DATE} = shift;
	}
	return $this->{DATE};
}

# htmlソース生成(view.cgiで使用する)
sub genhtml {
	my $this = shift;
	my $format = $this->{FORMAT};
	my $day;
	my $month;
	my $year;
	my $html;
	my $templatefile;
	my $data;
	my $topicid;
	my $lastdate;
	my $newtopiclist = undef;
	my @topicdata = ();

	if ($format == 1 || $format == 2) {
		($year,$month,$day) = (split(/\//, $this->{DATE}))[0..2];
	}
	$templatefile = "$this->{SYSTEMDIR}/index.htmlt";

	my $dtobj = APPLIB::DateTime->new();
	my $caption;
	my $sitetitle = "<a class=\"SITETITLE\" href=\"$this->{DOCROOT}/\">$this->{SITETITLE}</a>";
	my $subtitle;
	my $listtitle;
	my $metadisc = '';

	my $loggerobj = APPLIB::Logger->new();	# for debug
	my $archiveobj = APPLIB::Archive->new();

	my $topiclist;
	if ($format == 0) {	# トップページの作成

		$listtitle = '更新情報';

		# トピック一覧を取得
		$topiclist = $archiveobj->topictable();

		# 現在の日時を取得
		$dtobj->localdatetime(0);
		$year = $dtobj->year();
		$month = $dtobj->month();

		# キャブションを設定
		if ($this->{SUBTITLE}) {
			$caption = "$this->{SITETITLE}：$this->{SUBTITLE}";
			$metadisc = "<meta name=\"description\" content=\"$this->{SUBTITLE}\">";
		}
		else {
			$caption = $this->{SITETITLE};
		}
	}
	elsif ($format == 1) {	# 月別

		# 月別アーカイブファイルを読んで出力するトピックの一覧を生成する
		my $archivefile = "$this->{DATADIR}/$year$month" . "ar.data";
		if (-f $archivefile) {	# ファイルが存在する場合
			open IN, $archivefile;
			while ($data = <IN>) {
				chop($data);	# 改行を取り除く
				if ($data ne '') {
					($topicid,$lastdate) = (split(/\,/, $data))[0,1];
					push(@topicdata, $topicid);
				}
			}
			close IN;

			@topicdata = sort numbercomp @topicdata;

			# トピック一覧を取得
			$topiclist = $archiveobj->topictable(@topicdata);
		}
		# キャブションを設定
		$caption = "$this->{SITETITLE}：$year年$month月";
		$listtitle = "$year年$month月";
	}
	elsif ($format == 2) {	# 日別

		my $ayear;
		my $amonth;
		my $aday;

		# 月別アーカイブファイルを読んで出力するトピックの一覧を生成する
		my $archivefile = "$this->{DATADIR}/$year$month" . "ar.data";
		if (-f $archivefile) {	# ファイルが存在する場合
			open IN, $archivefile;
			while ($data = <IN>) {
				chop($data);	# 改行を取り除く
				if ($data ne "") {
					($topicid,$lastdate) = (split(/\,/, $data))[0,1];
					$data = substr($lastdate, 0, 10);
					($ayear,$amonth,$aday) = (split(/\//, $data))[0..2];
					if ($year eq $ayear && $month eq $amonth && $day eq $aday) {
						push(@topicdata, $topicid);
					}
				}
			}
			close IN;
		}

		@topicdata = sort numbercomp @topicdata;

		# トピック一覧を取得
		$topiclist = $archiveobj->topictable(@topicdata);

		# キャブションを設定
		$caption = "$this->{SITETITLE}：$year年$month月$day日";
		$listtitle = "$year年$month月$day日";
	}

	$subtitle = $this->{SUBTITLE};

	my $newtopic = '';
	if ($format != 0) {	# トップページ以外
		# 更新情報リストを取得
		$newtopic = $archiveobj->newtopic();
	}

	# カレンダーを取得
	my $calendarobj = APPLIB::Calendar->new();
	$calendarobj->year($year);
	$calendarobj->month($month);
	my $calendar = $calendarobj->calendartable();

	# アーカイブ一覧を取得
	my $archivelist = $archiveobj->archivetable();

	# 広告の取得
	my $adobj = APPLIB::Ad->new();
	my $adlink1 = $adobj->adlink1();
	my $adlink2 = $adobj->adlink2();
	my $adlist = $adobj->adlist();

	$adlink1 =~ s/enc_conma/,/g;
	$adlink2 =~ s/enc_conma/,/g;

	$adlink1 =~ s/<br>/\n/g;
	$adlink2 =~ s/<br>/\n/g;

	# タグクラウドを取得
	my $tagobj = APPLIB::Tag->new();
	my $tagclouds = $tagobj->smallclouds(0);

	# 検索ボックスの取得
	my $searchobj = APPLIB::Search->new();
	my $searchbox = $searchobj->searchbox2();
	my $program = "search.cgi";
	$searchbox =~ s/\$PROGRAM\$/$program/g;
	$searchbox =~ s/\$TARGETTEXT\$/selected/g;
	$searchbox =~ s/\$TARGETTAG\$//g;
	$searchbox =~ s/\$PREVQUERY\$//g;
#	$loggerobj->write("searchbox:$searchbox");	# for debug

	# HTMLの生成
	open(TEMPLATE, $templatefile);
	local($/) = undef;
	my $template = <TEMPLATE>;
	close(TEMPLATE);

	my $rssurl = "$this->{DOCROOT}/rss.xml";

	$template =~ s/\$METADISC\$/$metadisc/g;
	$template =~ s/\$RSSURL\$/$rssurl/g;
	$template =~ s/\$CAPTION\$/$caption/g;
	$template =~ s/\$SITETITLE\$/$sitetitle/g;
	$template =~ s/\$LISTTITLE\$/$listtitle/g;
	$template =~ s/\$SUBTITLE\$/$subtitle/g;
	$template =~ s/\$TOPICLIST\$/$topiclist/g;
	$template =~ s/\$TAGCLOUDS\$/$tagclouds/g;
	$template =~ s/\$ADLINK1\$/$adlink1/g;
	$template =~ s/\$ADLINK2\$/$adlink2/g;
	$template =~ s/\$ADLIST\$/$adlist/g;
	$template =~ s/\$SEARCHBOX\$/$searchbox/g;
	$template =~ s/\$NEWTOPIC\$/$newtopic/g;
	$template =~ s/\$CALENDAR\$/$calendar/g;
	$template =~ s/\$ARCHIVELIST\$/$archivelist/g;

	$html = $template;

	return $html;
}

# 最新トピックファイルを読んでRSS 2.0 ファイルを出力する
sub outputrss2 {
	my $this = shift;
	my $siteurl = "$this->{DOCROOT}/";
	my $sitetitle = $this->{SITETITLE};
	my $subtitle = $this->{SUBTITLE};
	my $result = 1;
	my $dtobj = APPLIB::DateTime->new();
	my $pubdate;
	my $data = '';
	my @topicdata;
	my $topicurl;

#	my $loggerobj = Logger->new();

	# 最近のトピックデータファイルを読み込みトピック一覧を作成する
	my $newtopicfile = "$this->{DATADIR}/newtopic.data";
	if (-f $newtopicfile) {	# ファイルが存在する場合
		open IN, $newtopicfile;
		while ($data = <IN>) {
			chop($data);	# 改行を取り除く
			if ($data ne "") {
				push(@topicdata, $data);
			}
		}
		close IN;
	}

	my $topicobj = APPLIB::Topic->new();
	my $tagobj = APPLIB::Tag->new();

	my $lastBuildDate = $dtobj->localdatetime(3);
	my $itemlist = '';

	foreach my $keytopicid (@topicdata) {
		$topicobj->load($keytopicid);
		my $topicid = $topicobj->topicid();
		my $lastdate = $topicobj->lastdate();
		my $title = $topicobj->title();
		my $text = $topicobj->text();
		my $commentnum = $topicobj->commentnum();
		my $tbnum = $topicobj->tbnum();
		my $tags = $topicobj->tags();
		my @tagitem = split(/ /, $tags);
		my $taglist = $tagobj->list(@tagitem);	# トピック

		# コンマをデコード
		$title =~ s/enc_conma/,/g;
		$text =~ s/enc_conma/,/g;

		# トピックのURL
		$topicurl = "$this->{DOCROOT}/detail$topicid.html";

		$text .= "<br>";	# 改行
		$text .= "<a href=\"$topicurl#comment\" target=\"_blank\">Comment($commentnum)</a> | ";	# コメント数
		$text .= "<a href=\"$topicurl#trackback\" target=\"_blank\">Trackback($tbnum)</a>";	# トラックバック数

		$text .= "<br><br>";	# 改行
		$text .= $taglist;	# タグ

		# 関連情報を追加
		my $linklist = $topicobj->genlinklist();
		$text .= $linklist;

		$text =~ s/&/&amp;/g; # アンパサントを置き換え
		$text =~ s/"/&quot;/g; # ダブルクォートを置き換え
		$text =~ s/</&lt;/g; # 右大不等号を置き換え
		$text =~ s/>/&gt;/g; # 左大不等号を置き換え

		$dtobj->format(3);
		$pubdate = $dtobj->setstrdate($lastdate);

		$itemlist .= <<"End_of_Temp";
  <item>
    <title>$title</title>
    <description>$text</description>
    <link>$topicurl</link>
    <pubDate>$pubdate</pubDate>
  </item>
End_of_Temp
	}

	# RSSファイルの生成
	open(TEMPLATE, "$this->{SYSTEMDIR}/rss2.xmlt");
	local($/) = undef;
	my $template = <TEMPLATE>;
	close(TEMPLATE);

	$template =~ s/\$SITEURL\$/$siteurl/g;
	$template =~ s/\$SITETITLE\$/$sitetitle/g;
	$template =~ s/\$SUBTITLE\$/$subtitle/g;
	$template =~ s/\$PUBDATE\$/$lastBuildDate/g;
	$template =~ s/\$LASTBUILDDATE\$/$lastBuildDate/g;
	$template =~ s/\$ITEMLIST\$/$itemlist/g;

	# RSS2.0ファイルに出力
	my $rss2file = "rss.xml";
	open(RSS2FILE,">$rss2file");
	flock(RSS2FILE, $this->{LOCK_EX});
	seek(RSS2FILE, 0, 2);
	print RSS2FILE $template;
	flock(RSS2FILE, $this->{LOCK_UN});
	close(RSS2FILE);
	# ファイルのパーミッションを変更
	chmod 0766, $rss2file;

	return $result;
}


# 数値ソート用
sub numbercomp {
	$a <=> $b;
}

1;
# End of View.pm
