#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Session;
use APPLIB::Message;
use APPLIB::String;
use APPLIB::Topic;
use APPLIB::Comment;
use APPLIB::Notify;
use APPLIB::Logger;
use APPLIB::Archive;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();
my $query = CGI->new();
my $msgobj = APPLIB::Message->new();
my $loggerobj = APPLIB::Logger->new();
my $strobj = APPLIB::String->new();

my $docrootdir = $confobj->DocrootDir();

my $logined = 0;
# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid ne '') {	# クッキー情報がない場合はログインされてない
	# 有効なセッションかチェック
	my $sessionobj = APPLIB::Session->new();
	my $userid = $sessionobj->check($sessionid);
	if ($userid) {
		$logined = 1;
	}
}

# 自サイトのコンテンツからの呼び出しかチェック
my $referer = $ENV{'HTTP_REFERER'};# 呼び出し元を取得
if ($referer !~ /$docrootdir/) {
	$msgobj->printmsg(2, 'コメントが拒否されました');
}

my $ipaddr = $ENV{'REMOTE_ADDR'};	# IPアドレスを取得
my $ret = $strobj->denycheck($ipaddr);
if (!$ret) {
	$msgobj->printmsg(2, 'コメントが拒否されました');
}

# トピックID
my $topicid = $query->param('topicid');
if ($topicid !~ /[0-9]+$/) {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# 名前
my $author = $query->param('author');
if ($author ne '') {
	$author =~ s/[\r\n]//g;		# 改行は削除
	$ret = $strobj->denycheck($author);
	if (!$ret) {
		$msgobj->printmsg(2, 'コメントが拒否されました');
	}
}
#$loggerobj->write("author:$author");	# for debug

# メールアドレス
my $mailaddr = $query->param('mailaddr');
if ($mailaddr ne '') {
	$mailaddr =~ s/[\r\n]//g;		# 改行は削除
	if ($mailaddr !~ /.+\@.+[.].+/) {
		$msgobj->printmsg(2, '入力されたメールアドレスに誤りがあります');
	}
	$ret = $strobj->denycheck($mailaddr);
	if (!$ret) {
		$msgobj->printmsg(2, 'コメントが拒否されました');
	}
}
#$loggerobj->write("mailaddr:$mailaddr");	# for debug

# ホームページURL
my $hpurl = $query->param('hpurl');
if ($hpurl ne '') {
	$hpurl =~ s/[\r\n]//g;		# 改行は削除
	if ($hpurl !~ /(http:\/\/[^()<>\[\]{}"'\x7F-\xFF\x00-\x20]+)/) {
		$msgobj->printmsg(2, '入力されたホームページURLに誤りがあります');
	}
	$ret = $strobj->denycheck($hpurl);
	if (!$ret) {
		$msgobj->printmsg(2, 'コメントが拒否されました');
	}
}
#$loggerobj->write("hpurl:$hpurl");	# for debug

# 評価
my $evaluate = $query->param('evaluate');
if ($evaluate eq '') {
	$evaluate = 0;
}

# コメント
my $comment = $query->param('comment');
if ($comment eq '') {
	$msgobj->printmsg(2, 'コメントが未入力です');
}
else {
	if ($comment =~ /^[\x20-\x7E\r\n]+$/) {
		$msgobj->printmsg(2, '無効なコメントです');
	}
	$ret = $strobj->denycheck($comment);
	if (!$ret) {
		$msgobj->printmsg(2, 'コメントが拒否されました');
	}
}

$comment = $strobj->taginvalidate($comment);	# タグを無効化する
$comment =~ s/\r?\n/<br>/g;		# 改行は<br>に置換
$comment =~ s/,/enc_conma/g;	# コンマはエンコード

$author = $strobj->taginvalidate($author);	# タグを無効化する
$mailaddr = $strobj->taginvalidate($mailaddr);
$hpurl = $strobj->taginvalidate($hpurl);

# コンマはエンコード
$author =~ s/,/enc_conma/g;
$mailaddr =~ s/,/enc_conma/g;
$hpurl =~ s/,/enc_conma/g;

my $topicobj = APPLIB::Topic->new();
$topicobj->mode(1);		# HTMLファイルを作成
$topicobj->load($topicid);	# 既存のトピックデータを読み込む
if ($topicobj->status() != 1) {
	$msgobj->printmsg(2, 'コメントが拒否されました');
}

my $commentnum = $topicobj->commentnum() + 1;
$topicobj->commentnum($commentnum);
my $point = $topicobj->point() ;
if ($evaluate > 0) {
	$point += $confobj->GoodPoint();
}
else {
	$point += $confobj->NormalPoint();
}

$topicobj->point($point);
$topicobj->update();	# トピックデータファイルを更新

my $commentobj = APPLIB::Comment->new();
$commentobj->addcomment($topicid, $author, $mailaddr, $hpurl, $comment, $ipaddr, $evaluate);

# トピックHTMLファイルの更新
$topicobj->genhtml();
my $topictitle = $topicobj->title();
my $topichtml = $topicobj->htmlname();

if ($evaluate > 0) {
	my $archiveobj = APPLIB::Archive->new();
	$archiveobj->add($topicid);
}

if ($confobj->NotifyMail() == 1) {

	# コンマはエンコード
	$author =~ s/enc_conma/,/g;
	$mailaddr =~ s/enc_conma/,/g;
	$hpurl =~ s/enc_conma/,/g;
	$comment =~ s/enc_conma/,/g;

	$topictitle =~ s/enc_conma/,/g;
	my $subject = "[$topictitle] にコメントがつきました";

	my $body = <<"End_of_Temp";
[$topictitle] にコメントがつきました。
$docrootdir/$topichtml

----------

IP  ：$ipaddr
名前：$author
MAIL：$mailaddr
URL ：$hpurl
コメント：
$comment

----------
End_of_Temp

	my $notifyobj = APPLIB::Notify->new();
	$notifyobj->subject($subject);
	$notifyobj->text($body);
	$notifyobj->send($confobj->ToAddress());
}

if ($logined) {
	$topichtml =~ s/detail([0-9]+).html/detail.cgi?topicid=$1/g;
}
#$loggerobj->write("topichtml:$topichtml");	# for debug

# トピック詳細ページを表示
print "Location: $topichtml", "\n\n";

exit;
