#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq "") {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $userid = $sessionobj->check($sessionid);
if (!$userid) {
	$msgobj->autherror();
}

# 管理者かチェック
my $usertype = $sessionobj->usertype($userid);
if ($usertype > 0) {
	$msgobj->printmsg(2, "この機能を実行できるのは管理者のみです");
}

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq "") {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

my $systemdir = $confobj->SystemDir();
my $operation = '';
my $listfile = '';
my $listitem = '';
my $adlink1 = '';
my $adlink2 = '';

if ($opecode == 1) { # フィルター
	$operation = "フィルターの編集";
	$listfile = "$systemdir/filter.txt";
}
elsif ($opecode == 2) { # Ping通知先
	$operation = "Ping設定";
	$listfile = "$systemdir/pinglist.txt";
}
elsif ($opecode == 3) { # お知らせ
	$operation = "お知らせの編集";
	$listfile = "$systemdir/adlist.txt";
}
elsif ($opecode == 4) { # 広告リンク
	$operation = "広告リンクの編集";
}
elsif ($opecode >= 11 && $opecode <= 13) { # ファイル更新
	$listitem = $query->param('listitem');
	updatelist();
}
elsif ($opecode == 14) { # 広告リンクの更新
	$adlink1 = $query->param('adlink1');
	$adlink2 = $query->param('adlink2');
	updateadlink();
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

$opecode += 10;

my $program = "control.cgi";
my $filename = '';

if ($opecode != 14) { # 広告リンク以外
	$listitem = '';
	if (-f $listfile) {
		open IN, $listfile;
		while (<IN>) {
			my $data = $_;	# 1行読み込み
			$listitem .= $data;
		}
		close IN;
	}
	$filename = "$systemdir/control.htmlt";
}
else {
	my $adfile = "$systemdir/adlink.txt";	# 広告リンクファイル
	if (-f $adfile) {
		open IN, $adfile;
		my $data = <IN>;	# 1行読み込み
		chop $data;
		($adlink1,$adlink2) = (split(/\,/, $data))[0..1];
		close IN;
	}
	$filename = "$systemdir/adlink.htmlt";
}

# 画面HTMの生成
open(TEMPLATE, $filename);
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$OPECODE\$/$opecode/g;
if ($opecode != 14) { # デフォルト広告以外
	$listitem =~ s/<br>/\n/g;
	$template =~ s/\$LISTITEM\$/$listitem/g;
}
else {
	$adlink1 =~ s/enc_conma/,/g;
	$adlink2 =~ s/enc_conma/,/g;
	$adlink1 =~ s/<br>/\n/g;
	$adlink2 =~ s/<br>/\n/g;
	$template =~ s/\$ADLINK1\$/$adlink1/g;
	$template =~ s/\$ADLINK2\$/$adlink2/g;
}

my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

# 編集画面を表示
print $temphtml;

exit;

sub updatelist{

	$listitem = $query->param('listitem');
	$listitem =~ s/\r?\n/\n/g;

	if ($opecode == 11) {	# フィルター
		$listfile = "$systemdir/filter.txt";
	}
	elsif ($opecode == 12) {	# Ping通知先
		$listfile = "$systemdir/pinglist.txt";
	}
	else {	# お知らせ
		$listfile = "$systemdir/adlist.txt";
	}

	open(LISTFILE, ">$listfile");
	flock(LISTFILE, $confobj->LockEx());
	seek(LISTFILE, 0, 2);
	print LISTFILE "$listitem";
	flock(LISTFILE, $confobj->LockUn());
	close(LISTFILE);
	chmod 0766, $listfile;

	# マイビュー画面を表示
	print "Location: search.cgi", "\n\n";

	exit;
}

sub updateadlink{

	$adlink1 =~ s/\r?\n/<br>/g;
	$adlink2 =~ s/\r?\n/<br>/g;

	$adlink1 =~ s/,/enc_conma/g;
	$adlink2 =~ s/,/enc_conma/g;

	my $adfile = "$systemdir/adlink.txt";	# 広告リンクファイル

	open(ADFILE,">$adfile");
	flock(ADFILE, $confobj->LockEx());
	seek(ADFILE, 0, 2);
	print ADFILE "$adlink1,$adlink2\n";
	flock(ADFILE, $confobj->LockUn());
	close(ADFILE);
	chmod 0766, $adfile;

	# マイビュー画面を表示
	print "Location: search.cgi", "\n\n";

	exit;
}
