#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Topic;
use APPLIB::Comment;
use APPLIB::Trackback;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq '') {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $userid = $sessionobj->check($sessionid);
if (!$userid) {
	$msgobj->autherror();
}

# トピックID
my $topicid = $query->param('topicid');
if ($topicid eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# コメントまたはトラックバックID
my $id = $query->param('id');
if ($id eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# 既存のトピックデータを読み込む
my $topicobj = APPLIB::Topic->new();
$topicobj->load($topicid);
my $editor = $topicobj->editor();
# 編集権限を持っているかチェック
my $usertype = $sessionobj->usertype($userid);
if ($editor ne $userid && $usertype > 0) {	# 登録者でも管理者でもない場合はエラー
	$msgobj->printmsg(2, 'トピックの編集は編集者もしくは管理者のみ可能です');
}

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

my $operation = '';
my $message = '';
my $content = '';
my $datadir = $confobj->DataDir();

if ($opecode == 1) {
	$operation = "トラックバックの削除";
	$message = "このトラックバックを削除しますか？";

	my ($tbid,$tbdate,$url,$blogname,$title,$excerpt) = ();
	# トラックバック一覧ファイルを読み込む
	my $tbfile = "$datadir/$topicid" . "tb.data";
	open IN, $tbfile;
	while (<IN>) {
		my $data = $_;
		chop $data;
		($tbid,$tbdate,$url,$blogname,$title,$excerpt) = (split(/\,/, $data))[0..5];
		if ($tbid == $id) {
			# コンマをデコード
			$title =~ s/enc_conma/,/g;
			$excerpt =~ s/enc_conma/,/g;
			last;
		}
	}
	close IN;

	my $trackbacktarget = $confobj->TrackbackTarget();
	$content = <<"End_of_HTML";
<div><a href="$url" $trackbacktarget>$title</a>：$blogname ($tbdate)</div>
<div>$excerpt</div>
End_of_HTML

}
elsif ($opecode == 2) {
	$operation = "コメントの削除";
	$message = "このコメントを削除しますか？";

	my ($commentid,$postdate,$author,$mailaddr,$hpurl,$text,$ipaddr,$evaluate) = ();
	# コメントファイルを読み込みコメント情報を取得
	my $commentfile = "$datadir/$topicid" ."cm.data";
	open IN, $commentfile;
	while (<IN>) {
		my $data = $_;
		chop $data;
		($commentid,$postdate,$author,$mailaddr,$hpurl,$text,$ipaddr,$evaluate) = (split(/\,/, $data))[0..7];
		if ($commentid == $id) {
			# コンマをデコード
			$author =~ s/enc_conma/,/g;
			$text =~ s/enc_conma/,/g;
			last;
		}
	}
	close IN;

	# 匿名の場合
	if ($author eq '') {
		$author = $confobj->Anonymous();
	}

	$content = <<"End_of_HTML";
<div>[$commentid] 投稿者：$author（$ipaddr） | 投稿日時：$postdate</div>
<div>$text</td></div>
End_of_HTML

}
elsif ($opecode >= 11 && $opecode <= 12) { # データ更新
	updatedata();	# データ更新処理
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

$opecode += 10;	

my $program = "delete.cgi";
my $systemdir = $confobj->SystemDir();

# 画面HTMの生成
open(TEMPLATE, "$systemdir/delete.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$MESSAGE\$/$message/g;
$template =~ s/\$CONTENT\$/$content/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$TOPICID\$/$topicid/g;
$template =~ s/\$ID\$/$id/g;
$template =~ s/\$OPECODE\$/$opecode/g;

my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

print $temphtml;	# 削除確認画面を表示

exit;

sub updatedata{

	my $lostpoint = 0;

	if ($opecode == 11) {
		my $tbobj = APPLIB::Trackback->new();
		$lostpoint = $tbobj->deltrackback($topicid, $id);
	}
	elsif ($opecode == 12) {
		my $commentobj = APPLIB::Comment->new();
		$lostpoint = $commentobj->delcomment($topicid, $id);
	}

	$topicobj->mode(1);		# HTMLファイルを作成

	if ($opecode == 11) {
		my $tbnum = $topicobj->tbnum() - 1;
		$topicobj->tbnum($tbnum);
	}
	else {
		my $commentnum = $topicobj->commentnum() - 1;
		$topicobj->commentnum($commentnum);
	}
	my $point = $topicobj->point() - $lostpoint;
	$topicobj->point($point);
	$topicobj->update();	# トピックデータファイルを更新
	$topicobj->genhtml();	# HTMLファイルを作成

	print "Location: detail.cgi?topicid=$topicid", "\n\n";

	exit;
}
