#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;
use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Topic;
use APPLIB::Search;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq '') {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $userid = $sessionobj->check($sessionid);
if (!$userid) {
	$msgobj->autherror();
}

# トピックID
my $topicid = $query->param('topicid');
if ($topicid eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# 関連情報ID
my $linkid = $query->param('linkid');
if ($linkid eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# 既存のトピックデータを読み込む
my $topicobj = APPLIB::Topic->new();
$topicobj->load($topicid);
my $editor = $topicobj->editor();
# 編集権限を持っているかチェック
my $usertype = $sessionobj->usertype($userid);
if ($editor ne $userid && $usertype > 0) {	# 登録者でも管理者でもない場合はエラー
	$msgobj->printmsg(2, 'トピックの編集は編集者もしくは管理者のみ可能です');
}

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

my $operation = '';
my $entercaption = '';
my $astmessage = '*は入力必須項目です。';
my $ast = '*';
my $message = '';
my $readonly = '';
my ($oldlinkid,$oldtitle,$oldurl,$oldsummary) = ();
my $datadir = $confobj->DataDir();

if ($opecode == 1) { # 新規作成
	$operation = "関連情報の新規作成";
	$entercaption = "更新";
}
elsif ($opecode == 2 || $opecode == 3) { # 変更

	# 既存の関連情報ファイルを読み込む
	my $linkfile = "$datadir/$topicid" . "ln.data";
	if (-f $linkfile) {	# ファイルが存在する場合
		open IN, $linkfile;
		while (my $data = <IN>) {
			chop($data);	# 改行を取り除く
			if ($data ne '') {
				($oldlinkid,$oldtitle,$oldurl,$oldsummary) = (split(/\,/, $data))[0..3];
				last if $oldlinkid eq $linkid ;
			}
		}
		close IN;
	}
	if ($oldlinkid eq '') {
		$msgobj->printmsg(2, 'パラメータに誤りがあります');
	}

	# <br>を改行に変換
	$oldsummary =~ s/<br>/\n/g; 

	# コンマをデコード
	$oldtitle =~ s/enc_conma/,/g;
	$oldsummary =~ s/enc_conma/,/g;

	if ($opecode == 2) { # 変更
		$operation = "関連情報の変更";
		$entercaption = "更新";
	}
	else { # 削除
		$operation = "関連情報の削除";
		$message = "<br><strong>この関連情報を削除しますか？</strong><br><br>";
		$readonly = "readonly";
		$entercaption = "削除";
		$astmessage = '';
		$ast = '';
	}
}
elsif ($opecode >= 11 && $opecode <= 13) { # データ更新

	# 入力チェック
	my $title = $query->param('title');
	if ($title eq '') {
		$msgobj->printmsg(2, 'タイトルが未入力です');
	}
	my $url = $query->param('url');
	if ($url eq '') {
		$msgobj->printmsg(2, 'URLが未入力です');
	}
	my $summary = $query->param('summary');

	updatelink($title, $url, $summary);	# データ更新処理
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

$opecode += 10;

my $program = "link.cgi";
my $systemdir = $confobj->SystemDir();

# 画面HTMの生成
open(TEMPLATE, "$systemdir/link.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$MESSAGE\$/$message/g;
$template =~ s/\$ASTMESSAGE\$/$astmessage/g;
$template =~ s/\$AST\$/$ast/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$TOPICID\$/$topicid/g;
$template =~ s/\$LINKID\$/$linkid/g;
$template =~ s/\$OPECODE\$/$opecode/g;
$template =~ s/\$TITLE\$/$oldtitle/g;
$template =~ s/\$URL\$/$oldurl/g;
$template =~ s/\$SUMMARY\$/$oldsummary/g;
$template =~ s/\$READONLY\$/$readonly/g;
$template =~ s/\$ENTERCAPTION\$/$entercaption/g;

my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

# 関連情報編集画面を表示
print $temphtml;

exit;

sub updatelink{
	my $title = shift;
	my $url = shift;
	my $summary = shift;

	if ($opecode != 13) { # 削除以外
		# 改行は<br>に置換
		$summary =~ s/\r?\n/<br>/g; 

		# コンマはエンコード
		$title =~ s/,/enc_conma/g;
		$summary =~ s/,/enc_conma/g;
	}

	# 関連情報一覧ファイルを読み込む
	my $linkfile = "$datadir/$topicid" . "ln.data";
	my $lastcount = 0;
	my @linkdata = ();

	open IN, $linkfile;
	while (my $data = <IN>) {
		chop $data;
		if ($data ne '') {
			my ($oldlinkid,$oldtitle,$oldurl,$oldsummary) = (split(/\,/, $data))[0..3];
			if ($oldlinkid eq $linkid) {
				if ($opecode == 12) {	# 変更
					# ログを差し替える
					$data = "$linkid,$title,$url,$summary";
				}
				if ($opecode == 13) {	# 削除
					# 削除対象のログIDを0にする
					$data = "0,$oldtitle,$oldurl,$oldsummary";
				}
			}
			push @linkdata, $data;

			if ($oldlinkid > $lastcount) {
				$lastcount = $oldlinkid;
			}
		}
	}
	close IN;

	if ($opecode == 11) { # 新規作成
		$lastcount++;
		my $data = "$lastcount,$title,$url,$summary";
		push @linkdata, $data;
	}

	# 関連情報ファイルを更新
	open(LINKFILE,">$linkfile");
	flock(LINKFILE, $confobj->LockEx());
	seek(LINKFILE, 0, 2);
	foreach my $data (@linkdata) {
		print LINKFILE "$data\n";
	}
	flock(LINKFILE, $confobj->LockUn());
	close(LINKFILE);

	# ファイルのパーミッションを変更
	chmod 0766, $linkfile;

	$topicobj->mode(1);		# HTMLファイルを作成

	my $linknum = 0;
	if ($opecode == 11) { # 新規作成
		$linknum = $topicobj->linknum() + 1;
	}
	if ($opecode == 13) { # 削除
		$linknum = $topicobj->linknum() - 1;
	}
	$topicobj->linknum($linknum);
	$topicobj->update();	# トピックデータファイルを更新
	$topicobj->genhtml();	# HTMLファイルを作成

	print "Location: detail.cgi?topicid=$topicid", "\n\n";

	exit;
}
