#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Message;
use APPLIB::DateTime;
use APPLIB::String;
use APPLIB::Topic;
use APPLIB::View;
use APPLIB::Archive;
use APPLIB::Tag;
use APPLIB::Search;
use APPLIB::Ping;
use APPLIB::Logger;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

my $loggerobj = APPLIB::Logger->new();

my $docrootdir = $confobj->DocrootDir();

# 自サイトのコンテンツからの呼び出しかチェック
my $referer = $ENV{'HTTP_REFERER'};# 呼び出し元を取得
if ($referer !~ /$docrootdir/) {
	$msgobj->printmsg(2, '投稿が拒否されました');
}

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq '') {
	$opecode = 1;
}

#$loggerobj->write("opecode:$opecode");	# for debug

# トピックのID
my $topicid = $query->param('topicid');

my $topicobj = APPLIB::Topic->new();
my $strobj = APPLIB::String->new();

my $operation = '';
my $tags = '';
my $title = '';
my $text = '';
my $handle = '';
my $status = 0;
my $adlink1 = '';
my $adlink2 = '';
my $systemdir = $confobj->SystemDir();

if ($opecode == 1) {
	$operation = 'トピックの新規投稿';

	if ($topicid ne '') {
		$topicobj->load($topicid);
		# 親トピックのタグを引き継ぐ
		$tags = $topicobj->tags();
	}
}
elsif ($opecode == 11) { # データ追加

	# 入力チェック
	# トピックのタイトル
	$title = $query->param('title');
	if ($title eq '') {
		$msgobj->printmsg(2, 'タイトルが未入力です');
	}
	else {
    	my $ret = $strobj->denycheck($title);
    	if (!$ret) {
    		$msgobj->printmsg(2, '投稿が拒否されました');
    	}
	}

	# トピックの本文
	$text = $query->param('text');
	if ($text eq '') {
		$msgobj->printmsg(2, '本文が未入力です');
	}
	else {
		if ($text =~ /^[\x20-\x7E\r\n]+$/) {
			$msgobj->printmsg(2, '本文が無効です');
		}
    	my $ret = $strobj->denycheck($text);
    	if (!$ret) {
    		$msgobj->printmsg(2, '投稿が拒否されました');
    	}
	}

	# 投稿者のハンドル
	$handle = $query->param('handle');
	my $ret = $strobj->denycheck($handle);
	if (!$ret) {
		$msgobj->printmsg(2, '投稿が拒否されました');
	}

	# タグ
	$tags = $query->param('tags');

	# HTMLタグを無効化する
	$title = $strobj->taginvalidate($title);
	$text = $strobj->taginvalidate($text);

	# トピックのステータス
	$status = 1;

	# 広告リンク１
	$adlink1 = '';
	# 広告リンク２
	$adlink2 = '';

	updatetopic();
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

$opecode += 10;	

my $program = "newtopic.cgi";

# 画面HTMの生成
open(TEMPLATE, "$systemdir/newtopic.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$TOPICID\$/$topicid/g;
$template =~ s/\$OPECODE\$/$opecode/g;
$template =~ s/\$TAGS\$/$tags/g;

# 新規投稿画面を表示
print "Content-Type: text/html\n\n";
print $template;

exit;

sub updatetopic{

	my $docrootdir = $confobj->DocrootDir();
	my $datadir = $confobj->DataDir();

	# 日時を取得
	my $dtobj = APPLIB::DateTime->new();
	my $systemdate = $dtobj->localdatetime(0);

	# 改行は<br>に置換
	$text =~ s/\r?\n/<br \/>/g; 
	$adlink1 =~ s/\r?\n/<br>/g;
	$adlink2 =~ s/\r?\n/<br>/g;

	# インデックスに追加
	my $searchobj = APPLIB::Search->new();

	# コンマはエンコード
	$title =~ s/,/enc_conma/g;
	$text =~ s/,/enc_conma/g;
	$adlink1 =~ s/,/enc_conma/g;
	$adlink2 =~ s/,/enc_conma/g;

	my $str = "$title $text $handle";

	my $topicobj = APPLIB::Topic->new();
	$topicobj->mode(1);

	my $archiveobj = APPLIB::Archive->new();
	my $lastseq = $archiveobj->lastseq();
	$lastseq++;
	$archiveobj->updatecounter($lastseq);

	my $newtopicid = $lastseq;
	$topicobj->topicid($newtopicid);
	$topicobj->lastdate($systemdate);
	$topicobj->status($status);
	$topicobj->editor('');
	$topicobj->handle($handle);

	$topicobj->title($title);
	$topicobj->text($text);
	$topicobj->commentnum(0);
	$topicobj->linknum(1);
	$topicobj->tbnum(0);

	my $topicurl = "$docrootdir/detail$newtopicid.html";

  my $ptopicobj = APPLIB::Topic->new();
	my $ptopicid = $topicid;
	my $ptitle = '';
	my $ptext = '';
	if ($topicid ne '') {
		$ptopicobj->mode(1);
		$ptopicobj->load($ptopicid);
		$ptitle = $ptopicobj->title();
		$ptext = $ptopicobj->text();
		my $ptags = $ptopicobj->tags();

		# 親トピックのタグを引き継ぐ
		$topicobj->tags($tags);

		# タグが設定されていある場合はタグ情報ファイルにトピックを追加
		if ($tags ne '') {
			my @newarray = split(/ /, $tags);
			my $tagobj = APPLIB::Tag->new();
			foreach my $tagitem (@newarray) {
				$tagobj->add($tagitem, $newtopicid);
			}
		}
	}

	$topicobj->update();

	# 検索用索引に追加
	$searchobj->addindex($newtopicid, $str);

	my $strobj = APPLIB::String->new();
	if ($topicid ne '') {
		# 要約の長さを調整
		my $summary = $strobj->cut($ptext, $confobj->SummaryLength());
		my $ptopicurl = "detail$ptopicid.html";

		# 新規投稿トピックの関連情報に親トピックを追加
		$topicobj->addlink($newtopicid, $ptitle, $ptopicurl, $summary);
	}

	# アーカイブファイルの出力
	$archiveobj->addarchive($newtopicid, $systemdate);

	# 更新情報データに追加
	$archiveobj->add($newtopicid);

	# 新規に作成したトピックのHTMLファイルを生成
	$topicobj->genhtml();
	my $htmlname = $topicobj->htmlname();

	if ($topicid ne '') {
		# 親トピックデータファイルを更新
		my $newlinknum = $ptopicobj->linknum() + 1;
		$ptopicobj->linknum($newlinknum);
		$ptopicobj->update();

		# 要約の長さを調整
		my $summary = $strobj->cut($text, $confobj->SummaryLength());
		$summary =~ s/,/enc_conma/g;	# コンマはエンコード

		# 親トピックに関連情報を追加
		$ptopicobj->addlink($ptopicid, $title, "detail$newtopicid.html", $summary);

		# 親トピックのHTMLファイルを生成
		$ptopicobj->genhtml();
	}

	my $viewobj = APPLIB::View->new();
	$viewobj->outputrss2();	# RSS 2.0の出力

	# 更新Ping送信
	my $pingobj = APPLIB::Ping->new();
	$pingobj->updateping($docrootdir, $confobj->SiteTitle());

	print "Location: $htmlname", "\n\n";

	exit;
}
