#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Topic;
use APPLIB::Archive;
use APPLIB::Notify;
use APPLIB::String;
use APPLIB::Ping;
use APPLIB::Trackback;
use APPLIB::Logger;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $tbid = gettbid($query);

sub gettbid {
    my $query = shift;
    my $pathinfo = $query->path_info();
    if ($pathinfo eq '') {
        $pathinfo = $ENV{REQUEST_URI};
    }
    $pathinfo && $pathinfo =~ m!(\w+)$! && return $1;
    return;
}

my $loggerobj = APPLIB::Logger->new();
my $strobj = APPLIB::String->new();

# トピックID
my $topicid = $tbid;
if ($topicid eq '') {
	tberror('トラックバックが拒否されました');
	exit;
}
#$loggerobj->write("rcvtb.cgi - topicid:$topicid");	# for debug

my $ipaddr = $ENV{'REMOTE_ADDR'};	# IPアドレスを取得
my $ret = $strobj->denycheck($ipaddr);
if (!$ret) {
	tberror('トラックバックが拒否されました');
	exit;
}

# エントリのタイトル
my $title = $query->param('title');
if ($title eq '') {
	tberror('トラックバックが拒否されました');
	exit;
}
# エントリの要約
my $excerpt = $query->param('excerpt');
if ($excerpt eq '') {
	tberror('トラックバックが拒否されました');
	exit;
}

# ブログ名
my $blogname = $query->param('blog_name');
if ($blogname eq '') {
	tberror('トラックバックが拒否されました');
	exit;
}
# エントリのURL
my $url = $query->param('url');
if ($url eq '') {
	tberror('トラックバックが拒否されました');
	exit;
}

# 文字コードの種類を取得
my $code = $strobj->getencode("$title $excerpt $blogname");
#$loggerobj->write("rcvtb.cgi code:$code");	# for debug

# 文字コードをUTF8に変換(euc,sjis,jis以外は原則utf8とみなす)
if ($code eq 'euc' || $code eq 'sjis' || $code eq 'jis') {
	$title = $strobj->encodeex($code, 'utf8', $title);
	$excerpt = $strobj->encodeex($code, 'utf8', $excerpt);
	$blogname = $strobj->encodeex($code, 'utf8', $blogname);
	$url = $strobj->encodeex($code, 'utf8', $url);
}

# 要約が半角英数のみの場合は拒否
if ($excerpt =~ /^[\x20-\x7E\r\n]+$/) {
	tberror('トラックバックが拒否されました');
	exit;
}

# 拒否リストに登録されていないかチェック
$ret = $strobj->denycheck($url);
if (!$ret) {
	tberror('トラックバックが拒否されました');
	exit;
}

$ret = $strobj->denycheck($title);
if (!$ret) {
	tberror('トラックバックが拒否されました');
	exit;
}

$ret = $strobj->denycheck($blogname);
if (!$ret) {
	tberror('トラックバックが拒否されました');
	exit;
}

# 要約の長さを調整
$excerpt = $strobj->cut($excerpt, $confobj->ExcerptLength());

# コンマをエンコード
$title =~ s/,/enc_conma/g;
$excerpt =~ s/,/enc_conma/g;
$blogname =~ s/,/enc_conma/g;
$url =~ s/,/enc_conma/g;

my $topicobj = APPLIB::Topic->new();
$topicobj->mode(1);		# HTMLファイルを作成
$topicobj->load($topicid);
if ($topicobj->status() != 1) {
	tberror('トラックバックが拒否されました。');
	exit;
}

my $newtbnum = $topicobj->tbnum() + 1; 
$topicobj->tbnum($newtbnum); 
my $newpoint = $topicobj->point() + $confobj->NormalPoint(); 
$topicobj->point($newpoint);
$topicobj->update();	# トピックデータファイルを更新

$loggerobj->write("update");	# for debug

my $tbobj = APPLIB::Trackback->new();
$tbobj->addtrackback($topicid, $url, $blogname, $title, $excerpt, $ipaddr);

$loggerobj->write("addtrackback");	# for debug

# トピックHTMLファイルの更新
$topicobj->genhtml();
my $topictitle = $topicobj->title();
my $topichtml = $topicobj->htmlname();

my $archiveobj = APPLIB::Archive->new();
$archiveobj->add($topicid);

my $docrootdir = $confobj->DocrootDir();

# 更新Ping送信
my $pingobj = APPLIB::Ping->new();
$pingobj->updateping($docrootdir, $confobj->SiteTtitle());

if ($confobj->NotifyMail() == 1) {

	# コンマをデコード
	$title =~ s/enc_conma/,/g;
	$excerpt =~ s/enc_conma/,/g;
	$blogname =~ s/enc_conma/,/g;
	$url =~ s/enc_conma/,/g;

	$topictitle =~ s/enc_conma/,/g;
	my $subject = "[$topictitle] にトラックバックがつきました";

	my $body = <<"End_of_Temp";
[$topictitle] にトラックバックがつきました。
$docrootdir/$topichtml

----------

IP      ：$ipaddr
ブログ名：$blogname
タイトル：$title
URL     ：$url
要約：
$excerpt

----------
End_of_Temp

	my $notifyobj = APPLIB::Notify->new();-
	$notifyobj->subject($subject);
	$notifyobj->text($body);
	$notifyobj->send($confobj->ToAddress());
}

# 結果を返す
my $tbresult =  <<"End_of_Result";
<?xml encoding="utf-8"?>
<response>
<error>0</error>
<message>Thanks for your ping to $topicid</message>
</response>
End_of_Result

print $query->header('text/xml'), $tbresult;

exit;

sub tberror{
	my $msg = shift;
	my $tbresult =  <<"End_of_Result";
<?xml encoding="utf-8"?>
<response>
<error>1</error>
<message>$msg</message>
</response>
End_of_Result

	print $query->header('text/xml'), $tbresult;
}
