#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;
use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Topic;
use APPLIB::View;
use APPLIB::Search;
use APPLIB::Archive;
use APPLIB::Logger;
use APPLIB::Conf;

my $query = CGI->new();
my $confobj = APPLIB::Conf->new();
my $msgobj = APPLIB::Message->new();

#my $loggerobj = APPLIB::Logger->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq '') {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $userid = $sessionobj->check($sessionid);
if (!$userid) {
	$msgobj->autherror();
}

# 管理者かチェック
my $usertype = $sessionobj->usertype($userid);
if ($usertype > 0) {
	$msgobj->printmsg(2, 'この機能を実行できるのは管理者のみです');
}

rebuildhtml();

if ($confobj->IndexRebuild()) {	# インデックスの再構築
  my $searchobj = APPLIB::Search->new();
  $searchobj->rebuild();
}

#$loggerobj->write("view");	# for debug

my $viewobj = APPLIB::View->new();
$viewobj->outputrss2();	# RSS 2.0の出力

# 検索画面を表示
print "Location: search.cgi", "\n\n";

exit;

sub rebuildhtml {

  my $docrootdir = $confobj->DocrootDir();
  my $systemdir = $confobj->SystemDir();
  my $datadir = $confobj->DataDir();

  # 既存の月別アーカイブを全て削除
  my $dir = "$datadir/*ar.data";
  my @files = <${dir}>;
  foreach my $fname (@files) {
    unlink $fname;
  }

  my $topicobj = APPLIB::Topic->new();
  $topicobj->mode(1);		# HTMLファイルを作成

  my $archiveobj = APPLIB::Archive->new();
  my $lastseq = $archiveobj->lastseq();

  my $urllist = <<"End_of_Temp";
   <url>
    <loc>$docrootdir/</loc>
    <changefreq>daily</changefreq>
    <priority>0.9</priority>
   </url>
End_of_Temp

  # トピックHTMLファイルを再作成
  for (my $i = 0; $i <= $lastseq; $i++) {
    my $topicid = $i;
    my $datafile = "$datadir/$topicid" . "tp.data";
    my $ret = $topicobj->read($datafile); # ファイル名を指定して読み込む
    if ($ret) {
      my $lastdate = $topicobj->lastdate();
      my $status = $topicobj->status();
      if ($status == 1 || $status == 2) {
        $archiveobj->addarchive($topicid, $lastdate);	# アーカイブデータファイルに追加
        $topicobj->genhtml();	# トピックHTMLファイルを更新
        my $htmlurl = "$docrootdir/detail" . "$topicid.html";
        $urllist .= <<"End_of_Temp";
   <url>
    <loc>$htmlurl</loc>
    <changefreq>daily</changefreq>
    <priority>0.8</priority>
   </url>
End_of_Temp
      }
    }
  }

  # タグページをsitemap.xmlに出力
  my $tagindex = "$datadir/tag/tagindex.data";
  open TAGINDEX, $tagindex;
  while (my $data =<TAGINDEX>) {
    chop $data;
    if ($data ne '') {
      my ($tagid,$tagitem,$topicnum) = (split(/\,/, $data))[0,1,3];
      if ($topicnum > 0) {
        $tagitem =~ s/(\W)/'%' . unpack('H2', $1)/eg;	# uriパック
        my $tagurl = "$docrootdir/search.cgi?querystr=$tagitem&amp;target=tag";
        $urllist .= <<"End_of_Temp";
   <url>
    <loc>$tagurl</loc>
    <changefreq>daily</changefreq>
    <priority>0.7</priority>
   </url>
End_of_Temp
      }
    }
  }
  close TAGINDEX;

  # sitemap.xmlを出力
  my $sitemapfile = "sitemap.xml";
  open(SITEMAPTEMP, "$systemdir/sitemap.xmlt");
  local($/) = undef;
  my $sitemaptemp = <SITEMAPTEMP>;
  close(SITEMAPTEMP);

  $sitemaptemp =~ s/\$URLLIST\$/$urllist/g;

  open(SITEMAP,">$sitemapfile");
  print SITEMAP $sitemaptemp;
  close(SITEMAP);

  # ファイルのパーミッションを変更
  chmod 0766, $sitemapfile;
}

