#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Message;
use APPLIB::User;
use APPLIB::Notify;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq "") {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

my $userobj = APPLIB::User->new();
my $operation = '';
my $userid = '';
my $mailaddr = '';
my $systemdir = $confobj->SystemDir();

if ($opecode == 1) { # ユーザー登録画面
	$operation = "ユーザー登録";
}
elsif ($opecode == 11) { # データ更新

	# ユーザーID
	$userid = $query->param('userid');
	if ($userid eq '') {
		$msgobj->printmsg(2, 'ユーザーIDが未入力です');
	}
	# ユーザーIDが英数字かチェック
	if ($userid =~ /[^0-9a-zA-Z]/) {
		$msgobj->printmsg(2, 'ユーザーIDは半角英数字で入力してください');
	}
	my $temp = uc($userid);
	if ($temp eq 'SESSION') {
		$msgobj->printmsg(2, 'ユーザーIDを「SESSION」とすることはできません');
	}

	# メールアドレス
	$mailaddr = $query->param('mailaddr');
	if ($mailaddr eq '') {
		$msgobj->printmsg(2, 'メールアドレスが未入力です');
	}
	# メールアドレス形式が正しいかチェック
	if ($mailaddr !~ /[-_0-9a-z]+\@[-_0-9a-z]+\.[-_0-9a-z.]+/) {
		$msgobj->printmsg(2, 'メールアドレスに誤りがあります');
	}

	# ユーザーIDの重複チェック
	my $userobj = APPLIB::User->new();

	my $ret = $userobj->find($userid, $mailaddr);
	if ($ret == 1) {
		$msgobj->printmsg(2, '指定のユーザーIDは既に登録されています');
	}
	elsif ($ret == 2) {
		$msgobj->printmsg(2, '指定のメールアドレスは既に登録されています');
	}

	updateuser();	# データ更新処理
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

$opecode += 10;

my $program = "register.cgi";

# 画面HTMLの生成
open(TEMPLATE, "$systemdir/register.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$OPECODE\$/$opecode/g;
$template =~ s/\$USERID\$/$userid/g;
$template =~ s/\$MAILADDR\$/$mailaddr/g;

my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

# ユーザー登録画面を表示
print $temphtml;

exit;

sub updateuser{

	my $username = $userid;	# 名前（ユーザーIDを設定）
	my $auth = 1;	# 編集者
	my $status = 1;	# 有効

	# 名前のコンマはエンコード
	$username =~ s/,/enc_conma/g;

	# 初期パスワードを生成
	my $password = int(rand(89999)) + 10000;	# 10000 ～ 99999 のパスワードを生成

	$userobj->userid($userid);
	$userobj->password($password);
	$userobj->mailaddr($mailaddr);
	$userobj->username($username);
	$userobj->auth($auth);
	$userobj->status($status);
	$userobj->add();

	my $notifyobj = APPLIB::Notify->new();
	my $sitetitle = $confobj->SiteTitle();
	my $docrootdir = $confobj->DocrootDir();

	# 登録者にメールを送信
	my $subject = "$sitetitle にユーザー登録されました";
	my $body = <<"End_of_Temp";
$sitetitle にユーザー登録されました。
登録ユーザーID、初期パスワードでログインし、パスワード変更を行なってください。

ユーザーID    ：$userid
初期パスワード：$password
メールアドレス：$mailaddr

End_of_Temp

	$notifyobj->subject($subject);
	$notifyobj->text($body);
	$notifyobj->send($mailaddr);

	# 管理者にメールを送信
	$subject = "$username さんがユーザー登録しました";
	$body = <<"End_of_Temp";
$username さんがユーザー登録しました。
$mailaddr
End_of_Temp

	$notifyobj->subject($subject);
	$notifyobj->text($body);
	$notifyobj->send($confobj->ToAddress());

	# 登録終了メッセージを表示
	my $message = <<"End_of_Temp";
$sitetitle にユーザー登録されました。
登録されたメールアドレスにユーザーID、初期パスワードを送信しました。
ログインして初期パスワードの変更を行なってください。
End_of_Temp

#	$msgobj->printmsg(0, $message);

	my $operation = "ユーザー登録完了";

	my $exitanchor = <<"End_of_Temp";
<a href="$docrootdir">トップページに戻る</a>
End_of_Temp

	# 画面HTMLの生成
	open(TEMPLATE, "$systemdir/notify.htmlt");
	local($/) = undef;
	my $template = <TEMPLATE>;
	close(TEMPLATE);

	$template =~ s/\$OPERATION\$/$operation/g;
	$template =~ s/\$MESSAGE\$/$message/g;
	$template =~ s/\$EXITANCHOR\$/$exitanchor/g;

	my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

	# ユーザー登録完了画面を表示
	print $temphtml;

	exit;
}
