#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;
use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Topic;
use APPLIB::Logger;
use APPLIB::Search;
use APPLIB::String;
use APPLIB::Tag;
use APPLIB::Ad;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

my $loggerobj = APPLIB::Logger->new();
my $logined = 0;

my $sessionobj = APPLIB::Session->new();

my $sessionid = '';
my $userid = '';
my $cookie = '';

# ログインモード
my $login = $query->param('login');
if ($login) {

	# ユーザーID
	$userid = $query->param('userid');
	if ($userid eq '') {
		$msgobj->printmsg(4, 'ユーザーIDが未入力です');
	}

	# パスワード
	my $userpwd = $query->param('userpwd');
	if ($userpwd eq '') {
		$msgobj->printmsg(4, 'パスワードが未入力です');
	}

	my $ipaddr = $ENV{'REMOTE_ADDR'};	# IPアドレスを取得

	$sessionid = $sessionobj->login($userid, $userpwd, $ipaddr);
	if (!$sessionid) {
		$msgobj->printmsg(4, '入力されたユーザーIDまたはパスワードに誤りがあります');
	}

	# クッキー文字列を生成
	$cookie = "Set-Cookie: cookie59b=$sessionid\n";	# ブラウザ終了時にセッション終了

	$logined = 1;
}
else {
	# クッキーからセッションIDを取得
	$sessionid = $query->cookie('cookie59b');
#	$loggerobj->write("sessionid:$sessionid");	# for debug

	if ($sessionid ne '') {
		# 有効なセッションかチェック
		$userid = $sessionobj->check($sessionid);
		if ($userid) {
			$logined = 1;
		}
	}
}

my $menulist = '';
if ($logined) {
	# 管理者かチェック
	my $usertype =  $sessionobj->usertype($userid);
	#$loggerobj->write("usertype:$usertype");	# for debug
	if ($usertype > 0) {
		$menulist = $sessionobj->editormenu();
	}
	else {
		$menulist = $sessionobj->adminmenu();
	}
}
else {
	$menulist = $sessionobj->usermenu();
}

my $querystr = $query->param('querystr');
#$loggerobj->write("search.cgi - querystr:$querystr");	# for debug

# 先頭の空白文字を削除
$querystr =~ s/^[ \t]+//g;

my $target = $query->param('target');
if ($target eq '') {
	$target = 'fulltext';
}

my $searchobj = APPLIB::Search->new();

my $targettext = '';
my $targettag = '';
my $result = '';
if ($target eq 'fulltext') {
	$targettext = 'selected';
	$result = $searchobj->search($querystr);
}
else {
	$targettag = 'selected';
	$result = $searchobj->tagsearch($querystr);
}

my $prevquery = $querystr;

my @array = split(/:/, $result);	# 配列に入れる
@array = reverse (sort numbercomp @array);	# 逆順に並べ替える

my $pos = $query->param('pos');
my $pagenavi = '';
my $program = 'search.cgi';
my $resultcount = $confobj->ResultCount();
my $startpos = 1;	
my $endpos = scalar(@array);

if (@array > $resultcount) {
	if ($pos eq '') {
		$startpos = 1;	
		$endpos = $resultcount;
	}
	else {
		$startpos = $pos;	
		$endpos = $pos + $resultcount - 1;
	}

	my $prev = ' ';
	my $next = ' ';
	if ($startpos > $resultcount) {
		my $prevpos = $startpos - $resultcount;
		$prev = "<a href=\"$program?pos=$prevpos&querystr=$prevquery&target=$target\">前ページ</a>";
	}
	else {
	}
	if (scalar(@array) > $endpos) {
		my $nextpos = $endpos + 1;
		$next = "<a href=\"$program?pos=$nextpos&querystr=$prevquery&target=$target\">次ページ</a>";
	}
	else {
		$endpos = scalar(@array);
	}
	my $current = "$startpos ～ $endpos";

	$pagenavi = <<"End_of_Temp";
<table border="0" width="100%" cellpadding="3" cellspacing="0">
<tr>
<td width="120" align="center">$prev</td>
<td width="120" align="center">$current</td>
<td width="120" align="center">$next</td>
</tr>
</table>
End_of_Temp
}

my $topicobj = APPLIB::Topic->new();
my $index = 1;

my $strobj = APPLIB::String->new();

my $resultlist = '';
foreach my $id (@array) {

	if ($index < $startpos) {
		# 読み飛ばし
		$index++;
		next;
	}

	if ($index > $endpos) {
		# 終了
		last;
	}

	$topicobj->load($id);	# 読み込み
	my $lastdate = $topicobj->lastdate();
	my $editor = $topicobj->editor();
	if ($editor eq '') {
		$editor = $topicobj->handle();
		if ($editor eq '') {
			$editor = $confobj->Anonymous();
		}
	}
	my $title = $topicobj->title();
	my $commentnum = $topicobj->commentnum();
	my $tbnum = $topicobj->tbnum();
	my $point = $topicobj->point();
	my $text = $topicobj->text();
	my $outline = $strobj->cut($text, $confobj->SummaryLength());

	my $htmlurl = '';
	if ($logined == 1) {
		$htmlurl = "detail.cgi?topicid=$id";
	}
	else {
		$htmlurl = "detail$id.html";
	}

	my $newwindow = '';

	$title =~ s/enc_conma/,/g;	# コンマをデコード
	$outline =~ s/enc_conma/,/g;	# コンマをデコード

	$resultlist .= <<"End_of_Temp";
<tr><td bgcolor="#EEEEEE">[$id] <a href="$htmlurl" $newwindow>$title</a></td></tr>
<tr><td>$outline</td></tr>
End_of_Temp

	$resultlist .= <<"End_of_Temp";
<tr><td><b><font class="POINT" size="+1">$point</font> point</b> | <a href="$htmlurl#comment">Comment($commentnum)</a> | <a href="$htmlurl#tarckback">Trackback($tbnum)</a> | posted by $editor at $lastdate</td></tr>
End_of_Temp

	$index++;
}

if ($resultlist eq '') {
	$resultlist = "<tr><td>該当トピックはありません</td></tr>";
}
my $msg = '';
if ($querystr ne '') {
	$msg = "<strong>$querystrの検索結果</strong><br>";
}

my $searchresult = <<"End_of_Temp";
<table border="0" width="100%" cellpadding="3" cellspacing="0" bgcolor="#FFFFFF">
$resultlist
</table>
End_of_Temp

my $caption = $confobj->SiteTitle();
if ($querystr) {
	$caption = "$querystr - " . $caption;
}

my $sitetitle = "<a class=\"SITETITLE\" href=\"" . $confobj->DocrootDir() . "\">" . $confobj->SiteTitle() . "</a>";
my $subtitle = $confobj->SubTitle();

# タグクラウドを取得
my $tagobj = APPLIB::Tag->new();
my $tagclouds = $tagobj->clouds(0);

# 検索ボックスの編集
my $searchbox = $searchobj->searchbox2();
$searchbox =~ s/\$PROGRAM\$/$program/g;
$searchbox =~ s/\$PREVQUERY\$/$querystr/g;
$searchbox =~ s/\$TARGETTEXT\$/$targettext/g;
$searchbox =~ s/\$TARGETTAG\$/$targettag/g;

my $systemdir = $confobj->SystemDir();
#$loggerobj->write("searchtmpl:$searchtmpl");	# for debug

# 画面HTMの生成
open(TEMPLATE, "$systemdir/search.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$CAPTION\$/$caption/g;
$template =~ s/\$SITETITLE\$/$sitetitle/g;
$template =~ s/\$SUBTITLE\$/$subtitle/g;
$template =~ s/\$MENULIST\$/$menulist/g;
$template =~ s/\$SEARCHBOX\$/$searchbox/g;
$template =~ s/\$MESSAGE\$/$msg/g;
$template =~ s/\$SEARCHRESULT\$/$searchresult/g;
$template =~ s/\$PAGENAVI\$/$pagenavi/g;
$template =~ s/\$TAGCLOUDS\$/$tagclouds/g;

print "Content-Type: text/html\n";
print "$cookie\n";
print $template;

exit;

# 数値ソート用
sub numbercomp {
	$a <=> $b;
}
