#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;
use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Logger;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq "") {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $loggerobj = APPLIB::Logger->new();

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $userid = $sessionobj->check($sessionid);
if (!$userid) {
	$msgobj->autherror();
}

# 管理者かチェック
my $usertype = $sessionobj->usertype($userid);
if ($usertype > 0) {
	$msgobj->printmsg(2, "この機能を実行できるのは管理者のみです");
}

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq "") {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

my $filecode = '';
if ($opecode == 0) { # ファイル読み込み

	$filecode = $query->param('filecode');
	if ($filecode eq '') {
		$msgobj->printmsg(2, 'パラメータに誤りがあります');
	}

	# 選択されたファイルに対応する動作コードを設定
	$opecode = $filecode;
}

# $loggerobj->write("filecode:$filecode");	# for debug
# $loggerobj->write("opecode:$opecode");	# for debug

my $systemdir = $confobj->SystemDir();
my $operation = '';
my $filename = '';
my $filepath = '';
my $filedata = '';

if ($opecode == 1) { # index.htmlt
	$operation = "リストページテンプレートの編集";
	$filename = "リストページテンプレート";
	$filepath = "$systemdir/index.htmlt";
}
elsif ($opecode == 2) { # detail.htmlt
	$operation = "詳細ページテンプレートの編集";
	$filename = "詳細ページテンプレート";
	$filepath = "$systemdir/detail.htmlt";
}
elsif ($opecode == 3) { # search.htmlt
	$operation = "検索ページテンプレートの編集";
	$filename = "検索ページテンプレート";
	$filepath = "$systemdir/search.htmlt";
}
elsif ($opecode == 4) { # basic.css
	$operation = "コンテンツスタイルシートの編集";
	$filename = "コンテンツスタイルシート";
	$filepath = "basic.css";
}
elsif ($opecode == 5) { # system.css
	$operation = "システムスタイルシートの編集";
	$filename = "システムスタイルシート";
	$filepath = "system.css";
}
elsif ($opecode >= 11 && $opecode <= 15) { # ファイル更新

	$filedata = $query->param('filedata');
	if ($filedata eq '') {
		$msgobj->printmsg(2, 'パラメータに誤りがあります');
	}

	# 改行コードを調整
	$filedata = $query->param('filedata');
	$filedata =~ s/\r//g; # \rは取り除く

	updatefile($filedata);
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# ファイルリストの設定
my $filelist = '';
if ($filecode == 1) {
	$filelist = <<"End_of_Temp";
<select name="filecode">
<option selected value="1">リストページテンプレート
<option value="2">詳細ページテンプレート
<option value="3">検索ページテンプレート
<option value="4">コンテンツスタイルシート
<option value="5">システムスタイルシート
</select>
End_of_Temp
}
elsif ($filecode == 2) {
	$filelist = <<"End_of_Temp";
<select name="filecode">
<option value="1">リストページテンプレート
<option selected value="2">詳細ページテンプレート
<option value="3">検索ページテンプレート
<option value="4">コンテンツスタイルシート
<option value="5">システムスタイルシート
</select>
End_of_Temp
}
elsif ($filecode == 4) {
	$filelist = <<"End_of_Temp";
<select name="filecode">
<option value="1">リストページテンプレート
<option value="2">詳細ページテンプレート
<option value="3">検索ページテンプレート
<option selected value="4">コンテンツスタイルシート
<option value="5">システムスタイルシート
</select>
End_of_Temp
}
elsif ($filecode == 5) {
	$filelist = <<"End_of_Temp";
<select name="filecode">
<option value="1">リストページテンプレート
<option value="2">詳細ページテンプレート
<option value="3">検索ページテンプレート
<option value="4">コンテンツスタイルシート
<option selected value="5">システムスタイルシート
</select>
End_of_Temp
}
else {
	$filelist = <<"End_of_Temp";
<select name="filecode">
<option value="1">リストページテンプレート
<option value="2">詳細ページテンプレート
<option selected value="3">検索ページテンプレート
<option value="4">コンテンツスタイルシート
<option value="5">システムスタイルシート
</select>
End_of_Temp
}

#$loggerobj->write("filelist:$filelist");	# for debug
#$loggerobj->write("filepath:$filepath");	# for debug

$opecode += 10;

# ファイルデータの読み込み
$filedata = undef;
if (-f $filepath) {
	open IN, $filepath;
	while (<IN>) {
		my $data = $_;	# 1行読み込み
		$filedata .= $data;
	}
	close IN;
}

#$loggerobj->write("filedata:$filedata");	# for debug

my $program = "template.cgi";

# 画面HTMの生成
open(TEMPLATE, "$systemdir/template.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

#$loggerobj->write("template:$template");	# for debug

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$OPECODE\$/$opecode/g;
$template =~ s/\$FILELIST\$/$filelist/g;
$template =~ s/\$FILENAME\$/$filename/g;
$template =~ s/\$FILEDATA\$/$filedata/g;

my $temphtml = <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

# 編集画面を表示
print $temphtml;

exit;

sub updatefile{
	my $filedata = shift;
	my $filepath = '';

	if ($opecode == 11) { # index.htmlt
		$filepath = "$systemdir/index.htmlt";
	}
	elsif ($opecode == 12) { # archive.htmlt
		$filepath = "$systemdir/detail.htmlt";
	}
	elsif ($opecode == 13) { # detail.htmlt
		$filepath = "$systemdir/search.htmlt";
	}
	elsif ($opecode == 14) { # basic.css
		$filepath = "basic.css";
	}
	elsif ($opecode == 15) { # system.css
		$filepath = "system.css";
	}

	if ($filepath) {
		open(OUTFILE, ">$filepath");
		flock(OUTFILE, $confobj->LockEx());
		seek(OUTFILE, 0, 2);
		print OUTFILE $filedata;
		flock(OUTFILE, $confobj->LockUn());
		close(OUTFILE);
		chmod 0766, $filepath;
	}

	# マイビュー画面を表示
	print "Location: search.cgi", "\n\n";

	exit;
}
