#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Session;
use APPLIB::Message;
use APPLIB::DateTime;
use APPLIB::String;
use APPLIB::Topic;
use APPLIB::View;
use APPLIB::Archive;
use APPLIB::Tag;
use APPLIB::Search;
use APPLIB::Ping;
use APPLIB::Logger;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq '') {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();
my $loggerobj = APPLIB::Logger->new();

# 有効なセッションかチェック
my $userid = $sessionobj->check($sessionid);
if (!$userid) {
	$msgobj->autherror();
}

my $opecode = $query->param('opecode');
if ($opecode eq '') {
	$opecode = 1;
}

#$loggerobj->write("opecode:$opecode");	# for debug

# トピックのID
my $topicid = $query->param('topicid');
if ($topicid eq '' && ($opecode != 1 && $opecode != 11)) {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

my $topicobj = APPLIB::Topic->new();

my $operation = '';
my $status = 0;
my $optioncb = '';
my $title = '';
my $text = '';
my $adlink1 = '';
my $adlink2 = '';
my $tags = '';
my $oldtags = '';
my $statuslist = '';
my $toppagecb = '';
my $trackback = '';
my $systemdir = $confobj->SystemDir();
my $datadir = $confobj->DataDir();

if ($opecode == 1) {
	$operation = 'トピックの新規投稿';

	$topicobj->load($topicid);

	# 親トピックのタグを引き継ぐ
	$tags = $topicobj->tags();

	# ステータスを設定
	$statuslist = <<"End_of_Temp";
<input type=radio name="status" value="0">ドラフト 
<input type=radio name="status" value="1" checked>オープン 
<input type=radio name="status" value="2">クローズ
End_of_Temp

	$optioncb = <<"End_of_Temp";
<input type="checkbox" name="updateinfo" value="1" checked>更新情報に掲載
End_of_Temp

	$optioncb .= <<"End_of_Temp";
&nbsp;&nbsp;<input type="checkbox" name="convcrlf" value="1" checked>改行を&lt;br /&gt;に変換
End_of_Temp
}
elsif ($opecode == 2) {
	$operation = 'トピックの編集';

	# 既存のトピックデータを読み込む
	$topicobj->load($topicid);
	my $editor = $topicobj->editor();

	# 編集権限を持っているかチェック
	my $usertype = $sessionobj->usertype($userid);
	if ($editor ne $userid && $usertype > 0) {	# 登録者でも管理者でもない場合はエラー
		$msgobj->printmsg(2, 'トピックの編集は編集者もしくは管理者のみ可能です');
	}

	$status = $topicobj->status();
	$title = $topicobj->title();
	$text = $topicobj->text();
	$adlink1 = $topicobj->adlink1();
	$adlink2 = $topicobj->adlink2();
	$tags = $topicobj->tags();
	$oldtags = $tags;

	# <br>を改行に変換
	$adlink1 =~ s/<br>/\n/g;
	$adlink2 =~ s/<br>/\n/g;

	# コンマをデコード
	$title =~ s/enc_conma/,/g;
	$text =~ s/enc_conma/,/g;
	$adlink1 =~ s/enc_conma/,/g;
	$adlink2 =~ s/enc_conma/,/g;

	# 状態を設定
	if ($status == 0) {
		$statuslist = <<"End_of_Temp";
<input type=radio name="status" value="0" checked>ドラフト 
<input type=radio name="status" value="1">オープン 
<input type=radio name="status" value="2">クローズ 
<input type=radio name="status" value="9">削除
End_of_Temp
	}

	if ($status == 1) {
		$statuslist = <<"End_of_Temp";
<input type=radio name="status" value="0">ドラフト 
<input type=radio name="status" value="1" checked>オープン 
<input type=radio name="status" value="2">クローズ 
<input type=radio name="status" value="9">削除
End_of_Temp
	}

	if ($status == 2) {
		$statuslist = <<"End_of_Temp";
<input type=radio name="status" value="0">ドラフト 
<input type=radio name="status" value="1">オープン 
<input type=radio name="status" value="2" checked>クローズ 
<input type=radio name="status" value="9">削除
End_of_Temp
	}

	if ($status == 9) {
		$statuslist = <<"End_of_Temp";
<input type=radio name="status" value="0">ドラフト 
<input type=radio name="status" value="1">オープン 
<input type=radio name="status" value="2">クローズ 
<input type=radio name="status" value="9" checked>削除
End_of_Temp
	}

	if ($confobj->AddUpdate()) {
		$optioncb = <<"End_of_Temp";
<input type="checkbox" name="updateinfo" value="1" checked>更新情報に掲載
End_of_Temp
	}
	else {
		$optioncb = <<"End_of_Temp";
<input type="checkbox" name="updateinfo" value="1">更新情報に掲載
End_of_Temp
	}

	$optioncb .= <<"End_of_Temp";
&nbsp;&nbsp;<input type="checkbox" name="convcrlf" value="1" checked>改行を&lt;br /&gt;に変換
End_of_Temp

}
elsif ($opecode >= 11 && $opecode <= 12) { # データ更新

	# 入力チェック
	# トピックのタイトル
	$title = $query->param('title');
	if ($title eq '') {
		$msgobj->printmsg(2, 'タイトルが未入力です');
	}

	# トピックのステータス
	$status = $query->param('status');
#	$loggerobj->write("status:$status");	# for debug
	if ($status eq '') {
		$msgobj->printmsg(2, 'ステータスが選択されていません');
	}

	# 最初のトピック(detail0.html) は削除不可
	if ($topicid == 0 && $status == 9) {
		$msgobj->printmsg(2, '最初のトピックは削除できません');
	}

	# トピックの本文
	$text = $query->param('text');
	if ($text eq '') {
		$msgobj->printmsg(2, '本文が未入力です');
	}

	# オプション
	my $updateinfo = $query->param('updateinfo');
	my $convcrlf = $query->param('convcrlf');

	my $tagobj = APPLIB::Tag->new();
	# タグ（スペース区切）
	$tags = $query->param('tags');
	# タグのチェック
	my $ret = $tagobj->check($tags);
	if (!$ret) {
		$msgobj->printmsg(2, 'タグとして使用できるのは半角英数と全角文字です');
	}

	# 編集前のタグ
	$oldtags = $query->param('oldtags');

	# トラックバックURL
	$trackback = $query->param('trackback');
	$trackback =~ s/\r?\n/,/g; # 改行をコンマに置換

	# 広告リンク１
	$adlink1 = $query->param('adlink1');
	# 広告リンク２
	$adlink2 = $query->param('adlink2');

	# トピックの更新
	updatetopic($updateinfo, $convcrlf);
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

$opecode += 10;	

my $program = "topic.cgi";

# 画面HTMの生成
open(TEMPLATE, "$systemdir/topic.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$TOPICID\$/$topicid/g;
$template =~ s/\$OPECODE\$/$opecode/g;
$template =~ s/\$TITLE\$/$title/g;
$template =~ s/\$STATUSLIST\$/$statuslist/g;
$template =~ s/\$OPTIONCB\$/$optioncb/g;
$template =~ s/\$TEXT\$/$text/g;
$template =~ s/\$TAGS\$/$tags/g;
$template =~ s/\$OLDTAGS\$/$oldtags/g;
$template =~ s/\$TRACKBACK\$/$trackback/g;
$template =~ s/\$ADLINK1\$/$adlink1/g;
$template =~ s/\$ADLINK2\$/$adlink2/g;
$template =~ s/\$PROGRAM\$/$program/g;

# トピック編集画面を表示
print "Content-Type: text/html\n\n";
print $template;

exit;

sub updatetopic{
	my ($updateinfo, $convcrlf) = @_;

	my $docrootdir = $confobj->DocrootDir();
	my $sitetitle = $confobj->SiteTitle();

	# 日時を取得
	my $dtobj = APPLIB::DateTime->new();
	my $systemdate = $dtobj->localdatetime(0);

	if ($convcrlf) {
		$text =~ s/\r?\n/<br \/>/g; 
	}
	else {
		$text =~ s/\r?\n//g; # 改行コードを取り除く
	}

	# 改行は<br>に置換
	$adlink1 =~ s/\r?\n/<br>/g;
	$adlink2 =~ s/\r?\n/<br>/g;

	# インデックスに追加
	my $searchobj = APPLIB::Search->new();

	# コンマはエンコード
	$title =~ s/,/enc_conma/g;
	$text =~ s/,/enc_conma/g;
	$adlink1 =~ s/,/enc_conma/g;
	$adlink2 =~ s/,/enc_conma/g;

	my $str = "$title $text $userid";

	my $topicobj = APPLIB::Topic->new();
	$topicobj->mode(1);

	my $archiveobj = APPLIB::Archive->new();

	my $newtopicid = 0;
	if ($opecode == 11) {	# 新規

		my $lastseq = $archiveobj->lastseq();
		$lastseq++;
		$archiveobj->updatecounter($lastseq);

		$newtopicid = $lastseq;
		$topicobj->topicid($newtopicid);
		$topicobj->lastdate($systemdate);
		$topicobj->status($status);
		$topicobj->editor($userid);
		$topicobj->handle('');
		$topicobj->title($title);
		$topicobj->text($text);
		$topicobj->commentnum(0);
		$topicobj->linknum(1);
		$topicobj->tbnum(0);

		my $topicurl = "$docrootdir/detail$newtopicid.html";

		my $ptopicobj = APPLIB::Topic->new();
		my $ptopicid = $topicid;
		my $ptitle = '';
		my $ptext = '';
		if ($topicid ne '') {
			$ptopicobj->mode(1);
			$ptopicobj->load($ptopicid);
			$ptitle = $ptopicobj->title();
			$ptext = $ptopicobj->text();
			my $padlink1 = $ptopicobj->adlink1();
			my $padlink2 = $ptopicobj->adlink2();

			# 広告リンクが未入力の場合は親トピックの広告を引き継ぐ
			if ($adlink1 eq '') {
				$adlink1 = $padlink1;
			}
			$topicobj->adlink1($adlink1);
			if ($adlink2 eq '') {
				$adlink2 = $padlink2;
			}
			$topicobj->adlink2($adlink2);

			# 親トピックのタグを引き継ぐ
			$topicobj->tags($tags);

			# タグが設定されていある場合はタグ情報ファイルにトピックを追加
			if ($tags ne '') {
				my @newarray = split(/ /, $tags);

				my $tagobj = APPLIB::Tag->new();
				foreach my $tagitem (@newarray) {
					$tagobj->add($tagitem, $newtopicid);
				}
			}
		}

		$topicobj->update();

		# 検索用索引に追加
		$searchobj->addindex($newtopicid, $str);

		# アーカイブファイルの出力
		$archiveobj->addarchive($newtopicid, $systemdate);

		my $strobj = APPLIB::String->new();
		if ($topicid ne '') {
			# 要約の長さを調整
			my $summary = $strobj->cut($ptext, $confobj->SummaryLength());

			my $ptopicurl = "detail$ptopicid.html";

			# 新規投稿トピックの関連情報に親トピックを追加
			$topicobj->addlink($newtopicid, $ptitle, $ptopicurl, $summary);
		}

		if ($updateinfo) {
			# 更新情報データに追加
			$archiveobj->add($newtopicid);
		}

		# 新規に作成したトピックのHTMLファイルを生成
		$topicobj->genhtml();
		my $htmlname = $topicobj->htmlname();

		if ($topicid ne '') {
			# 親トピックデータファイルを更新
			my $newlinknum = $ptopicobj->linknum() + 1;
			$ptopicobj->linknum($newlinknum);
			$ptopicobj->update();

			# 要約の長さを調整
			my $summary = $strobj->cut($text, $confobj->SummaryLength());
			$summary =~ s/,/enc_conma/g;	# コンマはエンコード

			# 親トピックに関連情報を追加
			$ptopicobj->addlink($ptopicid, $title, "detail$newtopicid.html", $summary);

			# 親トピックのHTMLファイルを生成
			$ptopicobj->genhtml();
		}

		if ($status != 0) {	# ドラフトでなかったらPing送信
			my $pingobj = APPLIB::Ping->new();

			# 更新Ping送信
			$pingobj->updateping($docrootdir, $sitetitle);

			# トラックバックの送信
			if ($trackback) {
				my @tbpinglist = split(/\,/, $trackback);
				$pingobj->trackbackping($docrootdir, $sitetitle, $title, $text, $confobj->ExcerptLength(), @tbpinglist);
			}
		}
	}
	else {

		# 既存のトピックデータを読み込む
		$topicobj->load($topicid);

		# 日付の更新
		if ($confobj->DateUpdate()) {
			$topicobj->lastdate($systemdate);	
		}

		$topicobj->status($status);
#		$topicobj->editor($userid);
		$topicobj->title($title);
		$topicobj->text($text);
		$topicobj->adlink1($adlink1);
		$topicobj->adlink2($adlink2);

		$topicobj->update();	# トピックデータファイルを更新

		# ドラフトまたは削除の場合はHTMLファイルを削除
		if ($status == 9 || $status == 0) {

			my $detailhtml = "detail$topicid.html";
			my @files = ();
			push @files, $detailhtml;
			unlink @files;

			# 最近のトピックファイルから削除
			$archiveobj->delete($topicid);

			# アーカイブデータから削除
			my $oldlastdate = $topicobj->lastdate();
			$archiveobj->deletearchive($topicid, $oldlastdate);

		}
		else {
			my $pingobj = APPLIB::Ping->new();

			my $htmlname = $topicobj->htmlname();
			if ($updateinfo) {
				$archiveobj->addarchive($topicid, $systemdate);	# アーカイブファイルの出力

				# 新着トピックデータに追加
				$archiveobj->add($topicid);

				# 更新Ping送信
				$pingobj->updateping($docrootdir, $sitetitle);
			}
			# トピックのHTMLファイルを生成
			$topicobj->genhtml();

			# トラックバックの送信
			if ($trackback) {
				my @tbpinglist = split(/\,/, $trackback);
				$pingobj->trackbackping($docrootdir, $sitetitle, $title, $text, $confobj->ExcerptLength(), @tbpinglist);
			}
		}
	}

	my $viewobj = APPLIB::View->new();
	$viewobj->outputrss2();	# RSS 2.0の出力

	if ($opecode == 11) {	# 新規
		$topicid = $newtopicid;
	}

	# タグの更新
	updatetag($topicid, $tags, $oldtags);

	print "Location: detail.cgi?topicid=$topicid", "\n\n";

	exit;
}

sub updatetag{
	my $id = shift;
	my $newtags = shift;
	my $oldtags = shift;

#	$loggerobj->write("oldtags:$oldtags");	# for debug
#	$loggerobj->write("tags:$tags");	# for debug

	if ($oldtags ne $newtags) {

		my $tagobj = APPLIB::Tag->new();

		# タグファイルを更新する
		my @newarray = split(/ /, $newtags);
		my @oldarray = split(/ /, $oldtags);
		my $itemlist = '';
		foreach my $data (@newarray) {
			if ($itemlist ne '') {
				$itemlist .= " ";
			}
			$itemlist .= $data;
		}
		my $tagfile = "$datadir/$id" . "tg.data";
		open(OUT,">$tagfile");
		flock(OUT, $confobj->LockEx());
		seek(OUT, 0, 2);
		print OUT "$itemlist\n";
		flock(OUT, $confobj->LockUn());
		close(OUT);

		# ファイルのパーミッションを変更
		chmod 0766, $tagfile;

#		$loggerobj->write("id:$id");	# for debug

		# タグ情報を更新
		my @addarray = ();
		my @delarray = ();
		@newarray = split(/ /, $newtags);

		# 新しいタグに存在して古いタグに存在しないものは新規追加
		foreach my $new (@newarray) {
			my $found = 0;
			foreach my $old (@oldarray) {
				if ($new eq $old) {
					$found = 1;
					last;
				}
			}
			if (!$found) {
				push @addarray, $new;
			}
		}

		# 古いタグに存在して新しいタグに存在しないものは削除
		foreach my $old (@oldarray) {
			my $found = 0;
			foreach my $new (@newarray) {
				if ($old eq $new) {
					$found = 1;
					last;
				}
			}
			if (!$found) {
				push @delarray, $old;
			}
		}

		my $item = '';
		# 追加
		foreach $item (@addarray) {
			$tagobj->add($item, $id);
		}

		# 削除
		foreach $item (@delarray) {
			$tagobj->delete($item, $id);
		}
	}
}

