#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007,2008 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;

use APPLIB::Session;
use APPLIB::Message;
use APPLIB::User;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq "") {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $adminuserid = $sessionobj->check($sessionid);
if (!$adminuserid) {
	$msgobj->autherror();
}

# 動作コード
my $opecode = $query->param('opecode');
if ($opecode eq '') {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# ユーザーID
my $userid = $query->param('userid');
my $userobj = APPLIB::User->new();

# ユーザー種別を取得
my $usertype = $sessionobj->usertype($adminuserid);

my $operation = '';
my $auth = 1;
my $status = 1;
my $password = '';
my $mailaddr = '';
my $siteurl = '';
my $adlink1 = '';
my $adlink2 = '';
my $username = '';
my $readonly = '';
my $disabled = '';

if ($opecode == 1) { # 新規作成
	# ユーザー作成権限があるかチェック
	if ($usertype > 0) {
		$msgobj->printmsg(2, "この機能を実行できるのは管理者のみです");
	}

	$operation = "ユーザーの追加";
}
elsif ($opecode == 2) { # 変更

	# 変更権限があるかチェック
	if ($usertype > 0 && $userid ne $adminuserid) {
		$msgobj->printmsg(2, "この機能を実行できるのは管理者またはユーザー本人のみです");
	}

	if ($usertype != 0) {
		$disabled = 'disabled';
	}

	$operation = "ユーザー情報の変更";

	$userobj->load($userid);
	$password = '*******';
	$mailaddr = $userobj->mailaddr();
	$siteurl = $userobj->siteurl();
	$username = $userobj->username();
	$auth = $userobj->auth();
	$status = $userobj->status();
	$adlink1 = $userobj->adlink1();
	$adlink2 = $userobj->adlink2();

	# 名前のコンマをデコード
	$username =~ s/enc_conma/,/g;

	# 広告リンクのデコード
	$adlink1 =~ s/enc_conma/,/g;
	$adlink2 =~ s/enc_conma/,/g;
	$adlink1 =~ s/<br>/\n/g;
	$adlink2 =~ s/<br>/\n/g;

	$readonly = 'readonly';

}
elsif ($opecode >= 11 && $opecode <= 12) { # データ更新

	# ユーザーID
	$userid = $query->param('userid');
	if ($userid eq "") {
		$msgobj->printmsg(2, 'ユーザーIDが未入力です');
	}
	# ユーザーIDが英数字かチェック
	if ($userid =~ /[^0-9a-zA-Z]/) {
		$msgobj->printmsg(2, 'ユーザーIDは半角英数字で入力してください');
	}
	my $temp = uc($userid);
	if ($temp eq 'SESSION') {
		$msgobj->printmsg(2, 'ユーザーIDを「SESSION」とすることはできません');
	}

	if ($opecode == 11) {	# 新規の場合
		# パスワード
		$password = $query->param('password');
		if ($password eq "") {
			$msgobj->printmsg(2, 'パスワードが未入力です');
		}
		# パスワードの長さをチェック
		if (length($password) < 4 || length($password) > 9) {
			$msgobj->printmsg(2, 'パスワードは半角英数字4文字～9文字で入力してください');
		}
		# パスワードが英数字かチェック
		if ($password =~ /[^0-9a-zA-Z]/) {
			$msgobj->printmsg(2, 'パスワードは半角英数字で入力してください');
		}
	}

	# メールアドレス
	$mailaddr = $query->param('mailaddr');
	if ($mailaddr eq '') {
		$msgobj->printmsg(2, 'メールアドレスが未入力です');
	}
	# メールアドレス形式が正しいかチェック
	if ($mailaddr !~ /[-_0-9a-z]+\@[-_0-9a-z]+\.[-_0-9a-z.]+/) {
		$msgobj->printmsg(2, 'メールアドレスに誤りがあります');
	}

	# ユーザーIDの重複チェック
	my $ret = $userobj->find($userid, $mailaddr);
	if ($ret == 1) {
		$msgobj->printmsg(2, '指定のユーザーIDは既に登録されています');
	}

	# 名前
	$username = $query->param('username');
	if ($username eq '') {
		$msgobj->printmsg(2, '名前が未入力です');
	}

	# 管理者の場合のみチェック
	if ($usertype == 0) {
		# 権限
		$auth = $query->param('auth');
		if ($auth eq '') {
			$msgobj->printmsg(2, '権限が選択されていません');
		}
		# ステータス
		$status = $query->param('status');
		if ($status eq '') {
			$msgobj->printmsg(2, 'ステータスが選択されていません');
		}
	}

	# WebサイトURL
	$siteurl = $query->param('siteurl');
	# 広告リンク1
	$adlink1 = $query->param('adlink1');
	# 広告リンク2
	$adlink2 = $query->param('adlink2');

	# 名前のコンマをエンコード
	$username =~ s/,/enc_conma/g;

	updateuser();	# データ更新処理
}
else {
	$msgobj->printmsg(2, 'パラメータに誤りがあります');
}

# 権限
my $authlist = '';
if ($auth == 0) {
	$authlist = <<"End_of_Row";
<select name="auth" $disabled>
<option selected value="0">管理者
<option value="1">編集者
</select>
End_of_Row
}
if ($auth == 1) {
	$authlist = <<"End_of_Row";
<select name="auth" $disabled>
<option value="0">管理者
<option selected value="1">編集者
</select>
End_of_Row
}

# 状態
my $statuslist = '';
if ($status == 1) {
	$statuslist = <<"End_of_Row";
<select name="status" $disabled>
<option selected value="1">アクティブ
<option value="9">無効
</select>
End_of_Row
}
else {
	$statuslist = <<"End_of_Row";
<select name="status" $disabled>
<option value="1">アクティブ
<option selected value="9">無効
</select>
End_of_Row
}

$opecode += 10;

my $program = "user.cgi";
my $systemdir = $confobj->SystemDir();

# 画面HTMLの生成
open(TEMPLATE, "$systemdir/user.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$OPERATION\$/$operation/g;
$template =~ s/\$PROGRAM\$/$program/g;
$template =~ s/\$OPECODE\$/$opecode/g;
$template =~ s/\$USERID\$/$userid/g;
$template =~ s/\$READONLY\$/$readonly/g;
$template =~ s/\$PASSWORD\$/$password/g;
$template =~ s/\$MAILADDR\$/$mailaddr/g;
$template =~ s/\$SITEURL\$/$siteurl/g;
$template =~ s/\$USERNAME\$/$username/g;
$template =~ s/\$AUTH\$/$authlist/g;
$template =~ s/\$STATUS\$/$statuslist/g;
$template =~ s/\$ADLINK1\$/$adlink1/g;
$template =~ s/\$ADLINK2\$/$adlink2/g;

my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

# ユーザー情報編集画面を表示
print $temphtml;

exit;

sub updateuser{

	# 広告リンクのエンコード
	$adlink1 =~ s/\r?\n/<br>/g;
	$adlink2 =~ s/\r?\n/<br>/g;
	$adlink1 =~ s/,/enc_conma/g;
	$adlink2 =~ s/,/enc_conma/g;

	my $userobj = APPLIB::User->new();

	if ($opecode == 11) {	# 新規のとき
		$userobj->userid($userid);
		$userobj->password($password);
		$userobj->mailaddr($mailaddr);
		$userobj->siteurl($siteurl);
		$userobj->username($username);
		$userobj->auth($auth);
		$userobj->status($status);
		$userobj->adlink1($adlink1);
		$userobj->adlink2($adlink2);
		$userobj->add();
	}
	else {	# 変更のとき
		$userobj->load($userid);
		$userobj->mailaddr($mailaddr);
		$userobj->siteurl($siteurl);
		$userobj->username($username);
		if ($usertype == 0) {	# 管理者の場合のみ権限、ステータスの変更が可能
			$userobj->auth($auth);
			$userobj->status($status);
		}
		$userobj->adlink1($adlink1);
		$userobj->adlink2($adlink2);
		$userobj->update();
	}

	if ($usertype == 0) {	# 管理者の場合
		print "Location: userlist.cgi", "\n\n";
	}
	else {
		print "Location: search.cgi", "\n\n";
	}
	
	exit;
}
