#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use CGI;
use APPLIB::Session;
use APPLIB::Message;
use APPLIB::Conf;

my $confobj = APPLIB::Conf->new();

my $query = CGI->new();
my $msgobj = APPLIB::Message->new();

# クッキーからセッションIDを取得
my $sessionid = $query->cookie('cookie59b');
if ($sessionid eq "") {	# クッキー情報がない場合はログインされてない
	$msgobj->autherror();
}

my $sessionobj = APPLIB::Session->new();

# 有効なセッションかチェック
my $adminuserid = $sessionobj->check($sessionid);
if (!$adminuserid) {
	$msgobj->autherror();
}

# 管理者かチェック
my $usertype = $sessionobj->usertype($adminuserid);
if ($usertype > 0) {
	$msgobj->printmsg(2, "この機能を実行できるのは管理者のみです");
}

my $systemdir = $confobj->SystemDir();

# ユーザー一覧ファイルを読み込みユーザー一覧を作成
my $userlistfile = "$systemdir/userlist.txt";

my $userlist = <<"End_of_Temp";
<table border="0" cellpadding="3" cellspacing="0">
<tr>
<td width="120" bgcolor="#CCCCCC"><b>ユーザーID</b></td>
<td width="200" bgcolor="#CCCCCC"><b>メールアドレス</b></td>
<td width="200" bgcolor="#CCCCCC"><b>名前</b></td>
<td width="60" bgcolor="#CCCCCC"><b>権限</b></td>
<td width="100" bgcolor="#CCCCCC"> </td>
</tr>
End_of_Temp

if (-f $userlistfile) {	# ファイルが存在する場合
	open IN, $userlistfile;
	while (<IN>) {
		my $data = $_;	# 1行読み込み
		chop($data);	# 改行を取り除く
		if ($data ne '') {
			my ($userid,$mailaddr,$username,$auth,$status) = (split(/\,/, $data))[0,2,4,5,6];
			my $authtype = '読者';
			if ($auth == 0) {
				$authtype = '管理者';
			}
			elsif ($auth == 1) {
				$authtype = '編集者';
			}

			# ユーザー一覧に設定するCGIの呼び出しコマンドを作成
			my $modifycmd = "user.cgi?userid=$userid&opecode=2";
			$userlist .= <<"End_of_Temp";
<tr>
<td width="100" bgcolor="#EEEEEE">$userid</td>
<td width="200" bgcolor="#EEEEEE">$mailaddr</td>
<td width="200" bgcolor="#EEEEEE">$username</td>
<td width="60" bgcolor="#EEEEEE">$authtype</td>
<td width="140" align="center" bgcolor="#EEEEEE"><a href="$modifycmd">変更</a></td>
</tr>
End_of_Temp
		}
	}
	close IN;
}

$userlist .= <<"End_of_Temp";
</table>
End_of_Temp

# 管理者用メニューを取得
my $menulist = $sessionobj->adminmenu();

my $sitetitle = "<a class=\"SITETITLE\" href=\"" . $confobj->DocrootDir() . "\">" . $confobj->SiteTitle() . "</a>";
my $subtitle = $confobj->SubTitle();
my $title = 'ユーザーリスト';

# 画面HTMの生成
open(TEMPLATE, "$systemdir/userlist.htmlt");
local($/) = undef;
my $template = <TEMPLATE>;
close(TEMPLATE);

$template =~ s/\$SITETITLE\$/$sitetitle/g;
$template =~ s/\$SUBTITLE\$/$subtitle/g;
$template =~ s/\$TITLE\$/$title/g;
$template =~ s/\$MENULIST\$/$menulist/g;
$template =~ s/\$USERLIST\$/$userlist/g;

my $temphtml =  <<"End_of_Temp";
Content-Type: text/html

$template
End_of_Temp

print $temphtml;	# ユーザーリスト

exit;
