#!/usr/bin/perl
#------------------------------------------------------------------------------
#    59bbs, bulletin board system for anonymous publisher.
#    Copyright (C) 2007 Kaga, Hiroaki
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#------------------------------------------------------------------------------

use strict;
use warnings;

use Encode;
use XMLRPC::Transport::HTTP;

use APPLIB::User;
use APPLIB::Topic;
use APPLIB::Archive;
use APPLIB::Search;
use APPLIB::DateTime;
use APPLIB::Logger;
use APPLIB::Conf;

my $loggerobj = APPLIB::Logger->new();
my $userobj = APPLIB::User->new();
my $confobj = APPLIB::Conf->new();
my $topicobj = APPLIB::Topic->new();
my $archiveobj = APPLIB::Archive->new();

my $server = XMLRPC::Transport::HTTP::CGI->dispatch_to('metaWeblog', 'blogger', 'mt')->handle;

package metaWeblog;

# metaWeblog.newPost (blogid, username, password, struct, publish) returns string
sub newPost {
  my ($self, $blogid, $username, $password, $content, $publish) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  my $newid = $archiveobj->lastseq() + 1;
  $archiveobj->updatecounter($newid);
  my $dtobj = APPLIB::DateTime->new();
  my $systemdate = $dtobj->localdatetime(0);

  if ($publish) {
    $topicobj->status(1);
  }
  else {
    $topicobj->status(0);
  }
  my $title = $content->{title};
  my $text = $content->{description};
  $title =~ s/,/enc_conma/g;
  $text =~ s/,/enc_conma/g;
  $title =~ s/\r?\n//g;
  $text =~ s/\r?\n//g;

  my $temp = Encode::encode('shiftjis', $title);
  $title = $temp;
  Encode::from_to($title, 'shiftjis', 'utf8');
  $temp = Encode::encode('shiftjis', $text);
  $text = $temp;
  Encode::from_to($text, 'shiftjis', 'utf8');

  $topicobj->topicid($newid);
  $topicobj->lastdate($systemdate);
  $topicobj->editor($username);
  $topicobj->title($title);
  $topicobj->text($text);
  $topicobj->tags($content->{mt_keywords});
  $topicobj->update();

  if ($publish) {
    my $searchobj = APPLIB::Search->new();
    my $str = "$title $text";
    $archiveobj->addarchive($newid, $systemdate);
    $archiveobj->add($newid);
    $searchobj->addindex($newid, $str);
    $topicobj->genhtml();
  }

  return 1;
}

# metaWeblog.editPost (postid, username, password, struct, publish) returns true
sub editPost {
  my ($self, $postid, $username, $password, $content, $publish) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  $topicobj->load($postid);
  if ($publish) {
    $topicobj->status(1);
  }
  else {
    $topicobj->status(0);
  }
  my $title = $content->{title};
  my $text = $content->{description};
  $title =~ s/,/enc_conma/g;
  $text =~ s/,/enc_conma/g;
  $title =~ s/\r?\n//g;
  $text =~ s/\r?\n//g;

  my $temp = Encode::encode('shiftjis', $title);
  $title = $temp;
  Encode::from_to($title, 'shiftjis', 'utf8');
  $temp = Encode::encode('shiftjis', $text);
  $text = $temp;
  Encode::from_to($text, 'shiftjis', 'utf8');

  $topicobj->title($title);
  $topicobj->text($text);
  $topicobj->tags($content->{mt_keywords});
  $topicobj->update();

  if ($publish) {
    $topicobj->genhtml();
  }

  return 1;
}

# metaWeblog.getPost (postid, username, password) returns struct
sub getPost {
  my ($self, $postid, $username, $password) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  my $dtobj = APPLIB::DateTime->new();
  $topicobj->load($postid);

  my $res;
  $res->{userid} = $username;
  my $title = $topicobj->title();
  my $text = $topicobj->text();
  $title =~ s/enc_conma/,/g;
  $text =~ s/enc_conma/,/g;

  my $tags = $topicobj->tags();
  my $lastdate = $topicobj->lastdate();
  $dtobj->format(4);
  $lastdate = $dtobj->setstrdate($lastdate);
  $res->{dateCreated} = $lastdate;
  $res->{postid} = SOAP::Data->type(string => $postid);
  $res->{description} = SOAP::Data->type(string => $text);
  $res->{title} = SOAP::Data->type(string => $title);
  my $htmlurl = $confobj->DocrootDir() . "/detail$postid.html";
  $res->{link} = SOAP::Data->type(string => $htmlurl);
  $res->{permaLink} = SOAP::Data->type(string => $htmlurl);
  $res->{mt_keywords} = SOAP::Data->type(string => $tags);
  return $res;
}

# metaWeblog.getCategories (blogid, username, password) returns struct
sub getCategories {
  my ($self, $blogid, $username, $password) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  my @items = ();
  my $item;
  my $lastdate = $topicobj->lastdate();
  $item->{description} = 'Unclassified';
  $item->{title} = 'Unclassified';
  push(@items, $item);
  my $res = \@items;
  return $res;
}

# metaWeblog.getRecentPosts (blogid, username, password, numberOfPosts) returns array of structs
sub getRecentPosts {
  my ($self, $blogid, $username, $password, $numberOfPosts) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  my $dtobj = APPLIB::DateTime->new();
  my $lastseq = $archiveobj->lastseq();
  my @items = ();
  my $count = 0;
#  $loggerobj->write("before loop");	# for debug
  for (my $i = $lastseq; ($i >= 0) && ($count < $numberOfPosts); $i--) {
    $topicobj->load($i);
    my $status = $topicobj->status();
    if (($topicobj->editor() eq $username) && ($status != 9)) {
      my $item;
      my $title = $topicobj->title();
      my $text = $topicobj->text();
      $title =~ s/enc_conma/,/g;
      $text =~ s/enc_conma/,/g;

      my $tags = $topicobj->tags();
      my $lastdate = $topicobj->lastdate();
      $dtobj->format(4);
      $lastdate = $dtobj->setstrdate($lastdate);
      $item->{dateCreated} = $lastdate;
      $item->{userid} = SOAP::Data->type(string => $username);
      $item->{postid} = SOAP::Data->type(string => $i);
      $item->{description} = SOAP::Data->type(string => $text);
      $item->{title} = SOAP::Data->type(string => $title);
      my $htmlurl = $confobj->DocrootDir() . "/detail$i.html";
      $item->{link} = SOAP::Data->type(string => $htmlurl);
      $item->{permaLink} = $htmlurl;
      $item->{mt_keywords} = SOAP::Data->type(string => $tags);
      push(@items, $item);
      $count++;
    }
  }

  my $res = \@items;
  return $res;
}

# metaWeblog.newMediaObject (blogid, username, password, struct) returns struct
sub newMediaObject {
  my ($self, $blogid, $username, $password, $file) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  use MIME::Base64;
  my $bits = $file->{bits};
  $bits = decode_base64($bits);

  my $name = $file->{name};
  my $filename = $confobj->FileDir() . "/" . $username . "_" . $name;
  my $url = $confobj->DocrootDir() . "/$filename";

  open(OUT, ">$filename"); 
  binmode(OUT); 
  print OUT $file; 
  close(OUT);

  return $url;
}

package blogger;

# blogger.getUsersBlogs (appkey, username, password) returns struct
sub getUsersBlogs {
  my ($self, $appkey, $username, $password) = @_;

  my $result = $userobj->login($username, $password);
  if ($result) {
    my @items = ();
    my $item;
    my $sitetitle = $confobj->SiteTitle();
    $item->{url} = $confobj->DocrootDir() . "/";
    $item->{blogid} = '59Tracker';
    $item->{blogName} = SOAP::Data->type(string => $sitetitle);
    push(@items, $item);

    my $res = \@items;
    return $res;
  }

  my $res;
  $res->{faultCode} = 'fc1';
  $res->{faultString} = "authentication error";

  return $res;
}

# blogger.getUserInfo (appkey, username, password) returns struct
sub getUserInfo {
  my ($self, $appkey, $username, $password) = @_;

  my $result = $userobj->login($username, $password);
  if ($result) {
    my $item;
    $item->{userid} = $userobj->userid();
    $item->{nickname} = '';
    $item->{email} = $userobj->mailaddr();
    $item->{url} = $confobj->DocrootDir() . "/";
    return $item;
  }

  my $res;
  $res->{faultCode} = 'fc1';
  $res->{faultString} = "authentication error";

  return $res;
}

# blogger.deletePost (appkey, postid, username, password, publish) returns true
sub deletePost {
  my ($self, $appkey, $postid, $username, $password, $publish) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  $topicobj->load($postid);
  $topicobj->delete();
  my $htmlfile = "detail$postid.html";
  unlink($htmlfile);

  return 1;
}

package mt;

# mt.getCategoryList (blogid, username, password) returns struct
sub getCategoryList {
  my ($self, $blogid, $username, $password) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }

  my @items = ();
  my $item;
  $item->{categoryId} = 'abcdef';
  $item->{categoryName} = 'Unclassified';
  push(@items, $item);
  my $res = \@items;
  return $res;
}

# mt.getPostCategories (blogid, username, password) returns struct
sub getPostCategories {
  my ($self, $blogid, $username, $password) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }


  my @items = ();
  my $item;
  $item->{categoryId} = 'id';
  $item->{categoryName} = 'Unclassified';
  $item->{isPrimary} = 1;
  push(@items, $item);
  my $res = \@items;
  return $res;
}

# mt.setPostCategories (blogid, username, password, categories) returns struct
sub setPostCategories {
  my ($self, $blogid, $username, $password, $categories) = @_;

  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }
  return 1;
}

# mt.publishPost (postid, username, password) returns struct
sub publishPost {
  my ($self, $postid, $username, $password) = @_;
  my $result = $userobj->login($username, $password);
  if (!$result) {
    my $res;
    $res->{faultCode} = 'fc1';
    $res->{faultString} = "authentication error";
    return $res;
  }
  return 1;
}

